//! Adiabatic Shortcuts Optimizer implementation
//!
//! This module implements shortcuts to adiabaticity (STA) and related protocols
//! for faster quantum optimization while maintaining high fidelity.

use std::collections::HashMap;
use std::time::Duration;

use super::error::{AdvancedQuantumError, AdvancedQuantumResult};
use crate::ising::IsingModel;
use crate::simulator::{AnnealingResult, AnnealingSolution};

/// Adiabatic Shortcuts Optimizer
#[derive(Debug, Clone)]
pub struct AdiabaticShortcutsOptimizer {
    /// Shortcuts configuration
    pub config: ShortcutsConfig,
    /// Shortcut protocols
    pub protocols: Vec<ShortcutProtocol>,
    /// Control optimization
    pub control_optimizer: ControlOptimizer,
    /// Performance statistics
    pub performance_stats: ShortcutsPerformanceStats,
}

/// Configuration for adiabatic shortcuts
#[derive(Debug, Clone)]
pub struct ShortcutsConfig {
    /// Shortcut method
    pub shortcut_method: ShortcutMethod,
    /// Control optimization method
    pub control_method: ControlOptimizationMethod,
    /// Time constraints
    pub time_constraints: TimeConstraints,
    /// Fidelity targets
    pub fidelity_targets: FidelityTargets,
    /// Resource constraints
    pub resource_constraints: ResourceConstraints,
}

impl Default for ShortcutsConfig {
    fn default() -> Self {
        Self {
            shortcut_method: ShortcutMethod::ShortcutsToAdiabaticity,
            control_method: ControlOptimizationMethod::GRAPE,
            time_constraints: TimeConstraints::default(),
            fidelity_targets: FidelityTargets::default(),
            resource_constraints: ResourceConstraints::default(),
        }
    }
}

/// Shortcut methods
#[derive(Debug, Clone, PartialEq)]
pub enum ShortcutMethod {
    /// Shortcuts to adiabaticity (STA)
    ShortcutsToAdiabaticity,
    /// Fast-forward protocols
    FastForward,
    /// Counterdiabatic driving
    CounterdiabaticDriving,
    /// Optimal control theory
    OptimalControl,
    /// Machine learning optimized
    MachineLearning,
}

/// Control optimization methods
#[derive(Debug, Clone, PartialEq)]
pub enum ControlOptimizationMethod {
    /// GRAPE (Gradient Ascent Pulse Engineering)
    GRAPE,
    /// CRAB (Chopped Random Basis)
    CRAB,
    /// Krotov method
    Krotov,
    /// Pontryagin maximum principle
    Pontryagin,
    /// Reinforcement learning
    ReinforcementLearning,
}

/// Time constraints
#[derive(Debug, Clone)]
pub struct TimeConstraints {
    /// Minimum evolution time
    pub min_time: f64,
    /// Maximum evolution time
    pub max_time: f64,
    /// Time discretization
    pub time_steps: usize,
    /// Time optimization tolerance
    pub time_tolerance: f64,
}

impl Default for TimeConstraints {
    fn default() -> Self {
        Self {
            min_time: 0.1,
            max_time: 10.0,
            time_steps: 100,
            time_tolerance: 1e-6,
        }
    }
}

/// Fidelity targets
#[derive(Debug, Clone)]
pub struct FidelityTargets {
    /// Target state fidelity
    pub state_fidelity: f64,
    /// Process fidelity
    pub process_fidelity: f64,
    /// Energy fidelity
    pub energy_fidelity: f64,
    /// Fidelity tolerance
    pub fidelity_tolerance: f64,
}

impl Default for FidelityTargets {
    fn default() -> Self {
        Self {
            state_fidelity: 0.99,
            process_fidelity: 0.95,
            energy_fidelity: 0.98,
            fidelity_tolerance: 1e-4,
        }
    }
}

/// Resource constraints
#[derive(Debug, Clone)]
pub struct ResourceConstraints {
    /// Maximum control amplitude
    pub max_control_amplitude: f64,
    /// Maximum control derivative
    pub max_control_derivative: f64,
    /// Available control fields
    pub available_controls: Vec<ControlField>,
    /// Hardware limitations
    pub hardware_limitations: HardwareLimitations,
}

impl Default for ResourceConstraints {
    fn default() -> Self {
        Self {
            max_control_amplitude: 10.0,
            max_control_derivative: 100.0,
            available_controls: vec![
                ControlField::MagneticX,
                ControlField::MagneticY,
                ControlField::MagneticZ,
            ],
            hardware_limitations: HardwareLimitations::default(),
        }
    }
}

/// Control field types
#[derive(Debug, Clone, PartialEq)]
pub enum ControlField {
    /// Magnetic field in X direction
    MagneticX,
    /// Magnetic field in Y direction
    MagneticY,
    /// Magnetic field in Z direction
    MagneticZ,
    /// Electric field
    Electric,
    /// Microwave drive
    Microwave,
    /// Laser field
    Laser,
}

/// Hardware limitations
#[derive(Debug, Clone)]
pub struct HardwareLimitations {
    /// Maximum field strength
    pub max_field_strength: f64,
    /// Field rise time
    pub field_rise_time: f64,
    /// Control bandwidth
    pub control_bandwidth: f64,
    /// Noise floor
    pub noise_floor: f64,
}

impl Default for HardwareLimitations {
    fn default() -> Self {
        Self {
            max_field_strength: 5.0,
            field_rise_time: 0.01,
            control_bandwidth: 1000.0,
            noise_floor: 1e-6,
        }
    }
}

/// Shortcut protocol
#[derive(Debug, Clone)]
pub struct ShortcutProtocol {
    /// Protocol name
    pub name: String,
    /// Time evolution
    pub time_evolution: Vec<TimePoint>,
    /// Control fields
    pub control_fields: Vec<ControlSequence>,
    /// Expected fidelity
    pub expected_fidelity: f64,
    /// Protocol cost
    pub protocol_cost: f64,
}

/// Time point in protocol
#[derive(Debug, Clone)]
pub struct TimePoint {
    /// Time
    pub time: f64,
    /// Hamiltonian at this time
    pub hamiltonian: HamiltonianComponent,
    /// State vector
    pub state_vector: Vec<f64>,
    /// Energy
    pub energy: f64,
}

/// Hamiltonian component
#[derive(Debug, Clone)]
pub struct HamiltonianComponent {
    /// Pauli string representation
    pub pauli_string: String,
    /// Coefficient
    pub coefficient: f64,
    /// Qubit indices
    pub qubit_indices: Vec<usize>,
}

/// Control sequence
#[derive(Debug, Clone)]
pub struct ControlSequence {
    /// Control field type
    pub field_type: ControlField,
    /// Time points and amplitudes
    pub amplitude_sequence: Vec<(f64, f64)>,
    /// Interpolation method
    pub interpolation: InterpolationMethod,
}

/// Interpolation methods
#[derive(Debug, Clone, PartialEq)]
pub enum InterpolationMethod {
    /// Linear interpolation
    Linear,
    /// Cubic spline
    CubicSpline,
    /// Fourier series
    Fourier,
    /// Piecewise constant
    PiecewiseConstant,
}

/// Control optimizer
#[derive(Debug, Clone)]
pub struct ControlOptimizer {
    /// Optimization algorithm
    pub algorithm: ControlOptimizationAlgorithm,
    /// Cost function
    pub cost_function: CostFunction,
    /// Gradient computation
    pub gradient_computation: GradientComputation,
    /// Convergence criteria
    pub convergence_criteria: ControlConvergenceCriteria,
}

/// Control optimization algorithms
#[derive(Debug, Clone)]
pub struct ControlOptimizationAlgorithm {
    /// Algorithm type
    pub algorithm_type: ControlOptimizationMethod,
    /// Parameters
    pub parameters: HashMap<String, f64>,
    /// Maximum iterations
    pub max_iterations: usize,
    /// Learning rate
    pub learning_rate: f64,
}

/// Cost function for control optimization
#[derive(Debug, Clone)]
pub struct CostFunction {
    /// Fidelity weight
    pub fidelity_weight: f64,
    /// Time weight
    pub time_weight: f64,
    /// Energy weight
    pub energy_weight: f64,
    /// Control effort weight
    pub control_effort_weight: f64,
    /// Regularization parameters
    pub regularization: Vec<RegularizationTerm>,
}

/// Regularization terms
#[derive(Debug, Clone)]
pub struct RegularizationTerm {
    /// Regularization type
    pub term_type: RegularizationType,
    /// Weight
    pub weight: f64,
    /// Parameters
    pub parameters: Vec<f64>,
}

/// Regularization types
#[derive(Debug, Clone, PartialEq)]
pub enum RegularizationType {
    /// L1 regularization
    L1,
    /// L2 regularization
    L2,
    /// Total variation
    TotalVariation,
    /// Smoothness penalty
    Smoothness,
    /// Bandwidth limitation
    Bandwidth,
}

/// Gradient computation methods
#[derive(Debug, Clone, PartialEq)]
pub enum GradientComputation {
    /// Analytical gradients
    Analytical,
    /// Finite differences
    FiniteDifferences,
    /// Automatic differentiation
    AutoDiff,
    /// Adjoint method
    Adjoint,
}

/// Control convergence criteria
#[derive(Debug, Clone)]
pub struct ControlConvergenceCriteria {
    /// Cost function tolerance
    pub cost_tolerance: f64,
    /// Gradient norm tolerance
    pub gradient_tolerance: f64,
    /// Parameter change tolerance
    pub parameter_tolerance: f64,
    /// Maximum stagnation iterations
    pub max_stagnation: usize,
}

/// Shortcuts performance statistics
#[derive(Debug, Clone)]
pub struct ShortcutsPerformanceStats {
    /// Achieved fidelity
    pub achieved_fidelity: f64,
    /// Protocol time
    pub protocol_time: f64,
    /// Optimization time
    pub optimization_time: Duration,
    /// Control effort
    pub control_effort: f64,
    /// Speedup factor
    pub speedup_factor: f64,
}

impl AdiabaticShortcutsOptimizer {
    /// Create new adiabatic shortcuts optimizer
    pub fn new(config: ShortcutsConfig) -> Self {
        Self {
            config,
            protocols: Vec::new(),
            control_optimizer: ControlOptimizer::default(),
            performance_stats: ShortcutsPerformanceStats::default(),
        }
    }

    /// Solve problem using adiabatic shortcuts
    pub fn solve<P>(&mut self, problem: &P) -> AdvancedQuantumResult<AnnealingResult<Vec<i32>>>
    where
        P: Clone,
    {
        // For compatibility with the coordinator, convert to the expected format
        if let Ok(ising_problem) = self.convert_to_ising(problem) {
            let solution = self.optimize(&ising_problem)?;
            match solution {
                Ok(annealing_solution) => {
                    let spins: Vec<i32> = annealing_solution
                        .best_spins
                        .iter()
                        .map(|&s| s as i32)
                        .collect();
                    Ok(Ok(spins))
                }
                Err(err) => Ok(Err(err)),
            }
        } else {
            Err(AdvancedQuantumError::AdiabaticError(
                "Cannot convert problem to Ising model".to_string(),
            ))
        }
    }

    /// Convert generic problem to Ising model (placeholder)
    fn convert_to_ising<P>(&self, _problem: &P) -> Result<IsingModel, AdvancedQuantumError> {
        // Placeholder implementation - in practice would convert problem type
        let mut ising = IsingModel::new(4);
        ising
            .set_bias(0, 1.0)
            .map_err(AdvancedQuantumError::IsingError)?;
        ising
            .set_bias(1, -1.0)
            .map_err(AdvancedQuantumError::IsingError)?;
        ising
            .set_coupling(0, 1, 0.5)
            .map_err(AdvancedQuantumError::IsingError)?;
        Ok(ising)
    }

    /// Optimize using adiabatic shortcuts
    pub fn optimize(
        &mut self,
        problem: &IsingModel,
    ) -> AdvancedQuantumResult<AnnealingResult<AnnealingSolution>> {
        println!("Starting Adiabatic Shortcuts optimization");
        let start_time = std::time::Instant::now();

        // Generate optimal control protocol
        let protocol = self.generate_shortcut_protocol(problem)?;
        self.protocols.push(protocol.clone());

        // Execute the protocol
        let result = self.execute_protocol(&protocol, problem)?;

        // Update performance statistics
        self.performance_stats.optimization_time = start_time.elapsed();
        self.performance_stats.achieved_fidelity = self.calculate_achieved_fidelity(&result)?;
        self.performance_stats.protocol_time = protocol
            .time_evolution
            .last()
            .map(|tp| tp.time)
            .unwrap_or(0.0);
        self.performance_stats.control_effort = self.calculate_control_effort(&protocol)?;
        self.performance_stats.speedup_factor = self.calculate_speedup_factor(&protocol)?;

        println!(
            "Adiabatic shortcuts completed. Energy: {:.6}, Fidelity: {:.6}",
            result.best_energy, self.performance_stats.achieved_fidelity
        );

        Ok(Ok(result))
    }

    /// Generate shortcut protocol for the problem
    fn generate_shortcut_protocol(
        &self,
        problem: &IsingModel,
    ) -> AdvancedQuantumResult<ShortcutProtocol> {
        let protocol_name = format!("{:?}_protocol", self.config.shortcut_method);

        // Generate time evolution points
        let time_evolution = self.generate_time_evolution(problem)?;

        // Generate control sequences
        let control_fields = self.generate_control_sequences(problem)?;

        // Estimate expected fidelity
        let expected_fidelity =
            self.estimate_protocol_fidelity(&time_evolution, &control_fields)?;

        // Calculate protocol cost
        let protocol_cost = self.calculate_protocol_cost(&control_fields)?;

        Ok(ShortcutProtocol {
            name: protocol_name,
            time_evolution,
            control_fields,
            expected_fidelity,
            protocol_cost,
        })
    }

    /// Generate time evolution for the protocol
    fn generate_time_evolution(
        &self,
        problem: &IsingModel,
    ) -> AdvancedQuantumResult<Vec<TimePoint>> {
        let mut time_points = Vec::new();
        let dt = (self.config.time_constraints.max_time - self.config.time_constraints.min_time)
            / self.config.time_constraints.time_steps as f64;

        for i in 0..=self.config.time_constraints.time_steps {
            let time = self.config.time_constraints.min_time + i as f64 * dt;

            // Create Hamiltonian for this time point
            let hamiltonian = HamiltonianComponent {
                pauli_string: format!("Z_{}", problem.num_qubits),
                coefficient: 1.0 - time / self.config.time_constraints.max_time,
                qubit_indices: (0..problem.num_qubits).collect(),
            };

            // Simple state vector (in practice would solve Schrödinger equation)
            let state_vector = self.compute_instantaneous_state(time, problem)?;

            // Compute energy
            let energy = self.compute_instantaneous_energy(time, problem, &state_vector)?;

            time_points.push(TimePoint {
                time,
                hamiltonian,
                state_vector,
                energy,
            });
        }

        Ok(time_points)
    }

    /// Compute instantaneous quantum state (simplified)
    fn compute_instantaneous_state(
        &self,
        time: f64,
        problem: &IsingModel,
    ) -> AdvancedQuantumResult<Vec<f64>> {
        let num_qubits = problem.num_qubits;
        let state_size = 1 << num_qubits;
        let mut state = vec![0.0; state_size];

        // Simple interpolation between uniform superposition and ground state
        let s = time / self.config.time_constraints.max_time;

        if s < 1e-8 {
            // Initial uniform superposition
            let amplitude = 1.0 / (state_size as f64).sqrt();
            for i in 0..state_size {
                state[i] = amplitude;
            }
        } else {
            // Gradually concentrate probability on ground state
            state[0] = s.sqrt();
            let remaining = (1.0 - s) / (state_size - 1) as f64;
            for i in 1..state_size {
                state[i] = remaining.sqrt();
            }
        }

        Ok(state)
    }

    /// Compute instantaneous energy
    fn compute_instantaneous_energy(
        &self,
        _time: f64,
        problem: &IsingModel,
        _state: &[f64],
    ) -> AdvancedQuantumResult<f64> {
        // Simplified energy calculation
        let mut energy = 0.0;

        // Add bias terms
        for i in 0..problem.num_qubits {
            if let Ok(bias) = problem.get_bias(i) {
                energy += bias;
            }
        }

        // Add coupling terms
        for i in 0..problem.num_qubits {
            for j in (i + 1)..problem.num_qubits {
                if let Ok(coupling) = problem.get_coupling(i, j) {
                    energy += coupling;
                }
            }
        }

        Ok(energy)
    }

    /// Generate control sequences
    fn generate_control_sequences(
        &self,
        problem: &IsingModel,
    ) -> AdvancedQuantumResult<Vec<ControlSequence>> {
        let mut control_sequences = Vec::new();

        for control_field in &self.config.resource_constraints.available_controls {
            let sequence = match self.config.shortcut_method {
                ShortcutMethod::ShortcutsToAdiabaticity => {
                    self.generate_sta_control_sequence(control_field.clone(), problem)?
                }
                ShortcutMethod::FastForward => {
                    self.generate_fastforward_control_sequence(control_field.clone(), problem)?
                }
                _ => self.generate_default_control_sequence(control_field.clone(), problem)?,
            };

            control_sequences.push(sequence);
        }

        Ok(control_sequences)
    }

    /// Generate STA control sequence
    fn generate_sta_control_sequence(
        &self,
        field_type: ControlField,
        _problem: &IsingModel,
    ) -> AdvancedQuantumResult<ControlSequence> {
        let mut amplitude_sequence = Vec::new();
        let dt = (self.config.time_constraints.max_time - self.config.time_constraints.min_time)
            / self.config.time_constraints.time_steps as f64;

        for i in 0..=self.config.time_constraints.time_steps {
            let time = self.config.time_constraints.min_time + i as f64 * dt;

            // STA-specific control amplitude calculation
            let s = time / self.config.time_constraints.max_time;
            let amplitude = match field_type {
                ControlField::MagneticX => s * (1.0 - s) * 4.0, // Bang-bang like
                ControlField::MagneticY => 0.5 * (1.0 - (2.0 * s - 1.0).powi(2)), // Smooth
                ControlField::MagneticZ => 1.0 - s,             // Linear decrease
                _ => 0.1 * (time * std::f64::consts::PI).sin(), // Sinusoidal
            };

            amplitude_sequence.push((time, amplitude));
        }

        Ok(ControlSequence {
            field_type,
            amplitude_sequence,
            interpolation: InterpolationMethod::CubicSpline,
        })
    }

    /// Generate fast-forward control sequence
    fn generate_fastforward_control_sequence(
        &self,
        field_type: ControlField,
        _problem: &IsingModel,
    ) -> AdvancedQuantumResult<ControlSequence> {
        let mut amplitude_sequence = Vec::new();
        let dt = (self.config.time_constraints.max_time - self.config.time_constraints.min_time)
            / self.config.time_constraints.time_steps as f64;

        for i in 0..=self.config.time_constraints.time_steps {
            let time = self.config.time_constraints.min_time + i as f64 * dt;

            // Fast-forward specific control
            let amplitude = match field_type {
                ControlField::MagneticX => 2.0 * time.exp() / self.config.time_constraints.max_time,
                ControlField::MagneticY => (time * 2.0 * std::f64::consts::PI).cos(),
                _ => 0.5,
            };

            amplitude_sequence.push((time, amplitude));
        }

        Ok(ControlSequence {
            field_type,
            amplitude_sequence,
            interpolation: InterpolationMethod::Linear,
        })
    }

    /// Generate default control sequence
    fn generate_default_control_sequence(
        &self,
        field_type: ControlField,
        _problem: &IsingModel,
    ) -> AdvancedQuantumResult<ControlSequence> {
        let mut amplitude_sequence = Vec::new();
        let dt = (self.config.time_constraints.max_time - self.config.time_constraints.min_time)
            / self.config.time_constraints.time_steps as f64;

        for i in 0..=self.config.time_constraints.time_steps {
            let time = self.config.time_constraints.min_time + i as f64 * dt;
            let amplitude = 0.1; // Constant small amplitude
            amplitude_sequence.push((time, amplitude));
        }

        Ok(ControlSequence {
            field_type,
            amplitude_sequence,
            interpolation: InterpolationMethod::PiecewiseConstant,
        })
    }

    /// Execute the shortcut protocol
    fn execute_protocol(
        &self,
        protocol: &ShortcutProtocol,
        problem: &IsingModel,
    ) -> AdvancedQuantumResult<AnnealingSolution> {
        let start_time = std::time::Instant::now();

        // Extract final state and energy
        let final_time_point = protocol
            .time_evolution
            .last()
            .ok_or_else(|| AdvancedQuantumError::AdiabaticError("Empty protocol".to_string()))?;

        let final_energy = final_time_point.energy;

        // Convert state to spin configuration
        let best_spins = self.extract_spin_configuration(&final_time_point.state_vector)?;

        Ok(AnnealingSolution {
            best_energy: final_energy,
            best_spins,
            repetitions: 1,
            total_sweeps: protocol.time_evolution.len(),
            runtime: start_time.elapsed(),
            info: format!("Adiabatic shortcuts: {}", protocol.name),
        })
    }

    /// Extract spin configuration from state vector
    fn extract_spin_configuration(&self, state_vector: &[f64]) -> AdvancedQuantumResult<Vec<i8>> {
        // Find the most probable computational basis state
        let max_amplitude_index = state_vector
            .iter()
            .enumerate()
            .max_by(|(_, a), (_, b)| a.abs().partial_cmp(&b.abs()).unwrap())
            .map(|(i, _)| i)
            .unwrap_or(0);

        // Convert to spin configuration
        let num_qubits = (state_vector.len() as f64).log2() as usize;
        let mut spins = Vec::new();

        for qubit in 0..num_qubits {
            let bit = (max_amplitude_index >> qubit) & 1;
            spins.push(if bit == 1 { 1 } else { -1 });
        }

        Ok(spins)
    }

    /// Estimate protocol fidelity
    fn estimate_protocol_fidelity(
        &self,
        _time_evolution: &[TimePoint],
        _control_fields: &[ControlSequence],
    ) -> AdvancedQuantumResult<f64> {
        // Simplified fidelity estimation
        Ok(0.95) // Placeholder high fidelity
    }

    /// Calculate protocol cost
    fn calculate_protocol_cost(
        &self,
        control_fields: &[ControlSequence],
    ) -> AdvancedQuantumResult<f64> {
        let mut total_cost = 0.0;

        for sequence in control_fields {
            let control_effort = sequence
                .amplitude_sequence
                .iter()
                .map(|(_, amplitude)| amplitude.powi(2))
                .sum::<f64>();
            total_cost += control_effort;
        }

        Ok(total_cost)
    }

    /// Calculate achieved fidelity
    fn calculate_achieved_fidelity(
        &self,
        _result: &AnnealingSolution,
    ) -> AdvancedQuantumResult<f64> {
        // Placeholder calculation
        Ok(0.93)
    }

    /// Calculate control effort
    fn calculate_control_effort(&self, protocol: &ShortcutProtocol) -> AdvancedQuantumResult<f64> {
        Ok(protocol.protocol_cost)
    }

    /// Calculate speedup factor
    fn calculate_speedup_factor(&self, protocol: &ShortcutProtocol) -> AdvancedQuantumResult<f64> {
        let adiabatic_time = 100.0; // Typical adiabatic evolution time
        let shortcut_time = protocol
            .time_evolution
            .last()
            .map(|tp| tp.time)
            .unwrap_or(1.0);

        Ok(adiabatic_time / shortcut_time)
    }
}

impl Default for ControlOptimizer {
    fn default() -> Self {
        Self {
            algorithm: ControlOptimizationAlgorithm {
                algorithm_type: ControlOptimizationMethod::GRAPE,
                parameters: HashMap::new(),
                max_iterations: 1000,
                learning_rate: 0.01,
            },
            cost_function: CostFunction {
                fidelity_weight: 1.0,
                time_weight: 0.1,
                energy_weight: 0.5,
                control_effort_weight: 0.01,
                regularization: Vec::new(),
            },
            gradient_computation: GradientComputation::FiniteDifferences,
            convergence_criteria: ControlConvergenceCriteria {
                cost_tolerance: 1e-6,
                gradient_tolerance: 1e-6,
                parameter_tolerance: 1e-8,
                max_stagnation: 50,
            },
        }
    }
}

impl Default for ShortcutsPerformanceStats {
    fn default() -> Self {
        Self {
            achieved_fidelity: 0.0,
            protocol_time: 0.0,
            optimization_time: Duration::from_secs(0),
            control_effort: 0.0,
            speedup_factor: 1.0,
        }
    }
}

/// Create default adiabatic shortcuts optimizer
pub fn create_adiabatic_shortcuts_optimizer() -> AdiabaticShortcutsOptimizer {
    AdiabaticShortcutsOptimizer::new(ShortcutsConfig::default())
}

/// Create custom adiabatic shortcuts optimizer
pub fn create_custom_adiabatic_shortcuts_optimizer(
    shortcut_method: ShortcutMethod,
    control_method: ControlOptimizationMethod,
    max_time: f64,
) -> AdiabaticShortcutsOptimizer {
    let mut config = ShortcutsConfig::default();
    config.shortcut_method = shortcut_method;
    config.control_method = control_method;
    config.time_constraints.max_time = max_time;

    AdiabaticShortcutsOptimizer::new(config)
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_adiabatic_shortcuts_creation() {
        let optimizer = create_adiabatic_shortcuts_optimizer();
        assert!(matches!(
            optimizer.config.shortcut_method,
            ShortcutMethod::ShortcutsToAdiabaticity
        ));
        assert!(matches!(
            optimizer.config.control_method,
            ControlOptimizationMethod::GRAPE
        ));
    }

    #[test]
    fn test_time_evolution_generation() {
        let optimizer = create_adiabatic_shortcuts_optimizer();
        let ising = IsingModel::new(2);

        let time_evolution = optimizer.generate_time_evolution(&ising).unwrap();
        assert!(!time_evolution.is_empty());
        assert!(time_evolution.len() > 10);

        // Check time ordering
        for i in 1..time_evolution.len() {
            assert!(time_evolution[i].time > time_evolution[i - 1].time);
        }
    }

    #[test]
    fn test_control_sequence_generation() {
        let optimizer = create_adiabatic_shortcuts_optimizer();
        let ising = IsingModel::new(2);

        let control_sequences = optimizer.generate_control_sequences(&ising).unwrap();
        assert!(!control_sequences.is_empty());

        for sequence in &control_sequences {
            assert!(!sequence.amplitude_sequence.is_empty());
            // Check time ordering in amplitude sequence
            for i in 1..sequence.amplitude_sequence.len() {
                assert!(sequence.amplitude_sequence[i].0 > sequence.amplitude_sequence[i - 1].0);
            }
        }
    }

    #[test]
    fn test_spin_configuration_extraction() {
        let optimizer = create_adiabatic_shortcuts_optimizer();

        // State vector for 2 qubits with highest amplitude at |01⟩ (index 1)
        let state_vector = vec![0.1, 0.9, 0.3, 0.2];
        let spins = optimizer.extract_spin_configuration(&state_vector).unwrap();

        assert_eq!(spins.len(), 2);
        assert_eq!(spins[0], 1); // bit 0 is 1 -> spin +1
        assert_eq!(spins[1], -1); // bit 1 is 0 -> spin -1
    }
}
