//! Infinite-depth QAOA (∞-QAOA) optimizer implementation
//!
//! This module implements the infinite-depth Quantum Approximate Optimization Algorithm
//! with adaptive depth selection, parameter optimization, and convergence detection.

use rand::{Rng, SeedableRng};
use rand_chacha::ChaCha8Rng;
use std::collections::HashMap;
use std::f64::consts::PI;
use std::time::{Duration, Instant};

use super::error::{AdvancedQuantumError, AdvancedQuantumResult};
use super::utils::{calculate_relative_improvement, normalize_parameters, validate_parameters};
use crate::ising::IsingModel;
use crate::simulator::{AnnealingParams, AnnealingResult, AnnealingSolution};

/// Infinite-depth QAOA (∞-QAOA) optimizer
#[derive(Debug, Clone)]
pub struct InfiniteDepthQAOA {
    /// Configuration for ∞-QAOA
    pub config: InfiniteQAOAConfig,
    /// Current parameter history
    pub parameter_history: Vec<Vec<f64>>,
    /// Energy history
    pub energy_history: Vec<f64>,
    /// Depth progression
    pub depth_progression: Vec<usize>,
    /// Convergence metrics
    pub convergence_metrics: ConvergenceMetrics,
    /// Adaptive depth control
    pub depth_controller: AdaptiveDepthController,
    /// Performance statistics
    pub performance_stats: InfiniteQAOAStats,
}

/// Configuration for infinite-depth QAOA
#[derive(Debug, Clone)]
pub struct InfiniteQAOAConfig {
    /// Initial depth
    pub initial_depth: usize,
    /// Maximum depth (for practical limits)
    pub max_depth: usize,
    /// Depth increment strategy
    pub depth_strategy: DepthIncrementStrategy,
    /// Parameter initialization method
    pub initialization_method: ParameterInitializationMethod,
    /// Optimization tolerance
    pub optimization_tolerance: f64,
    /// Maximum optimization iterations per depth
    pub max_iterations_per_depth: usize,
    /// Convergence criteria
    pub convergence_criteria: ConvergenceCriteria,
    /// Classical optimizer configuration
    pub classical_optimizer: ClassicalOptimizerConfig,
    /// Measurement strategy
    pub measurement_strategy: MeasurementStrategy,
    /// Noise mitigation settings
    pub noise_mitigation: NoiseMitigationConfig,
}

impl Default for InfiniteQAOAConfig {
    fn default() -> Self {
        Self {
            initial_depth: 1,
            max_depth: 100,
            depth_strategy: DepthIncrementStrategy::Adaptive,
            initialization_method: ParameterInitializationMethod::Heuristic,
            optimization_tolerance: 1e-6,
            max_iterations_per_depth: 1000,
            convergence_criteria: ConvergenceCriteria::default(),
            classical_optimizer: ClassicalOptimizerConfig::default(),
            measurement_strategy: MeasurementStrategy::default(),
            noise_mitigation: NoiseMitigationConfig::default(),
        }
    }
}

/// Depth increment strategies
#[derive(Debug, Clone, PartialEq)]
pub enum DepthIncrementStrategy {
    /// Linear increment
    Linear,
    /// Exponential increment
    Exponential,
    /// Adaptive based on convergence
    Adaptive,
    /// Golden ratio increment
    GoldenRatio,
    /// Fibonacci sequence
    Fibonacci,
}

/// Parameter initialization methods
#[derive(Debug, Clone, PartialEq)]
pub enum ParameterInitializationMethod {
    /// Random initialization
    Random,
    /// Heuristic initialization
    Heuristic,
    /// Transfer from previous depth
    Transfer,
    /// Interpolation-based
    Interpolation,
    /// Machine learning guided
    MLGuided,
}

/// Convergence criteria
#[derive(Debug, Clone)]
pub struct ConvergenceCriteria {
    /// Energy improvement threshold
    pub energy_threshold: f64,
    /// Parameter change threshold
    pub parameter_threshold: f64,
    /// Gradient norm threshold
    pub gradient_threshold: f64,
    /// Maximum stagnation iterations
    pub max_stagnation: usize,
    /// Relative improvement threshold
    pub relative_improvement: f64,
}

impl Default for ConvergenceCriteria {
    fn default() -> Self {
        Self {
            energy_threshold: 1e-8,
            parameter_threshold: 1e-6,
            gradient_threshold: 1e-6,
            max_stagnation: 50,
            relative_improvement: 1e-6,
        }
    }
}

/// Classical optimizer configuration
#[derive(Debug, Clone)]
pub struct ClassicalOptimizerConfig {
    /// Optimizer type
    pub optimizer_type: ClassicalOptimizerType,
    /// Learning rate
    pub learning_rate: f64,
    /// Momentum coefficient
    pub momentum: f64,
    /// L-BFGS memory size
    pub lbfgs_memory: usize,
    /// Maximum function evaluations
    pub max_evaluations: usize,
}

impl Default for ClassicalOptimizerConfig {
    fn default() -> Self {
        Self {
            optimizer_type: ClassicalOptimizerType::LBFGS,
            learning_rate: 0.01,
            momentum: 0.9,
            lbfgs_memory: 10,
            max_evaluations: 1000,
        }
    }
}

/// Classical optimizer types
#[derive(Debug, Clone, PartialEq)]
pub enum ClassicalOptimizerType {
    /// Gradient Descent
    GradientDescent,
    /// Adam optimizer
    Adam,
    /// L-BFGS
    LBFGS,
    /// Nelder-Mead
    NelderMead,
    /// Powell's method
    Powell,
    /// Differential Evolution
    DifferentialEvolution,
}

/// Measurement strategy
#[derive(Debug, Clone)]
pub struct MeasurementStrategy {
    /// Number of shots per measurement
    pub shots: usize,
    /// Observable decomposition method
    pub observable_decomposition: ObservableDecomposition,
    /// Error mitigation for measurements
    pub error_mitigation: MeasurementErrorMitigation,
    /// Grouping strategy for observables
    pub grouping_strategy: ObservableGrouping,
}

impl Default for MeasurementStrategy {
    fn default() -> Self {
        Self {
            shots: 8192,
            observable_decomposition: ObservableDecomposition::PauliStrings,
            error_mitigation: MeasurementErrorMitigation::ZeroNoiseExtrapolation,
            grouping_strategy: ObservableGrouping::QubitWise,
        }
    }
}

/// Observable decomposition methods
#[derive(Debug, Clone, PartialEq)]
pub enum ObservableDecomposition {
    /// Pauli string decomposition
    PauliStrings,
    /// Tensor network decomposition
    TensorNetwork,
    /// Clifford decomposition
    Clifford,
    /// Fermionic decomposition
    Fermionic,
}

/// Measurement error mitigation techniques
#[derive(Debug, Clone, PartialEq)]
pub enum MeasurementErrorMitigation {
    /// No mitigation
    None,
    /// Zero-noise extrapolation
    ZeroNoiseExtrapolation,
    /// Readout error correction
    ReadoutCorrection,
    /// Symmetry verification
    SymmetryVerification,
    /// Virtual distillation
    VirtualDistillation,
}

/// Observable grouping strategies
#[derive(Debug, Clone, PartialEq)]
pub enum ObservableGrouping {
    /// Group by qubit
    QubitWise,
    /// Group by commutation
    Commuting,
    /// Graph coloring based
    GraphColoring,
    /// Tensor factorization
    TensorFactorization,
}

/// Noise mitigation configuration
#[derive(Debug, Clone)]
pub struct NoiseMitigationConfig {
    /// Enable noise mitigation
    pub enabled: bool,
    /// Mitigation techniques
    pub techniques: Vec<NoiseMitigationTechnique>,
    /// Noise characterization
    pub noise_characterization: NoiseCharacterization,
    /// Error threshold
    pub error_threshold: f64,
}

impl Default for NoiseMitigationConfig {
    fn default() -> Self {
        Self {
            enabled: true,
            techniques: vec![
                NoiseMitigationTechnique::ZeroNoiseExtrapolation,
                NoiseMitigationTechnique::SymmetryVerification,
            ],
            noise_characterization: NoiseCharacterization::default(),
            error_threshold: 0.01,
        }
    }
}

/// Noise mitigation techniques
#[derive(Debug, Clone, PartialEq)]
pub enum NoiseMitigationTechnique {
    /// Zero-noise extrapolation
    ZeroNoiseExtrapolation,
    /// Probabilistic error cancellation
    ProbabilisticErrorCancellation,
    /// Symmetry verification
    SymmetryVerification,
    /// Virtual distillation
    VirtualDistillation,
    /// Dynamical decoupling
    DynamicalDecoupling,
    /// Composite pulses
    CompositePulses,
}

/// Noise characterization
#[derive(Debug, Clone)]
pub struct NoiseCharacterization {
    /// Gate error rates
    pub gate_errors: HashMap<String, f64>,
    /// Readout error rates
    pub readout_errors: Vec<f64>,
    /// Coherence times
    pub coherence_times: Vec<f64>,
    /// Cross-talk matrix
    pub crosstalk_matrix: Vec<Vec<f64>>,
}

impl Default for NoiseCharacterization {
    fn default() -> Self {
        Self {
            gate_errors: HashMap::new(),
            readout_errors: Vec::new(),
            coherence_times: Vec::new(),
            crosstalk_matrix: Vec::new(),
        }
    }
}

/// Adaptive depth controller
#[derive(Debug, Clone)]
pub struct AdaptiveDepthController {
    /// Current depth
    pub current_depth: usize,
    /// Depth increment factor
    pub increment_factor: f64,
    /// Performance history
    pub performance_history: Vec<DepthPerformance>,
    /// Depth selection strategy
    pub selection_strategy: DepthSelectionStrategy,
    /// Convergence detector
    pub convergence_detector: DepthConvergenceDetector,
}

/// Performance at specific depth
#[derive(Debug, Clone)]
pub struct DepthPerformance {
    /// Depth level
    pub depth: usize,
    /// Best energy achieved
    pub best_energy: f64,
    /// Optimization iterations
    pub iterations: usize,
    /// Convergence time
    pub convergence_time: Duration,
    /// Parameter count
    pub parameter_count: usize,
    /// Improvement over previous depth
    pub improvement: f64,
}

/// Depth selection strategies
#[derive(Debug, Clone, PartialEq)]
pub enum DepthSelectionStrategy {
    /// Conservative increment
    Conservative,
    /// Aggressive increment
    Aggressive,
    /// Performance-based
    PerformanceBased,
    /// Resource-aware
    ResourceAware,
    /// Theoretical limit guided
    TheoreticalGuided,
}

/// Depth convergence detector
#[derive(Debug, Clone)]
pub struct DepthConvergenceDetector {
    /// Energy improvement history
    pub improvement_history: Vec<f64>,
    /// Convergence threshold
    pub convergence_threshold: f64,
    /// Minimum depths to check
    pub min_depths: usize,
    /// Detection confidence
    pub confidence_level: f64,
}

/// Convergence metrics
#[derive(Debug, Clone)]
pub struct ConvergenceMetrics {
    /// Current energy
    pub current_energy: f64,
    /// Best energy found
    pub best_energy: f64,
    /// Energy gradient norm
    pub gradient_norm: f64,
    /// Parameter change magnitude
    pub parameter_change: f64,
    /// Convergence score
    pub convergence_score: f64,
    /// Iterations without improvement
    pub stagnation_count: usize,
}

/// Performance statistics for ∞-QAOA
#[derive(Debug, Clone)]
pub struct InfiniteQAOAStats {
    /// Total depths explored
    pub depths_explored: usize,
    /// Total optimization time
    pub total_time: Duration,
    /// Best energy achieved
    pub best_energy: f64,
    /// Optimal depth found
    pub optimal_depth: usize,
    /// Average convergence time per depth
    pub avg_convergence_time: Duration,
    /// Parameter optimization efficiency
    pub optimization_efficiency: f64,
}

impl InfiniteDepthQAOA {
    /// Create a new infinite-depth QAOA optimizer
    pub fn new(config: InfiniteQAOAConfig) -> Self {
        Self {
            config: config.clone(),
            parameter_history: Vec::new(),
            energy_history: Vec::new(),
            depth_progression: Vec::new(),
            convergence_metrics: ConvergenceMetrics {
                current_energy: f64::INFINITY,
                best_energy: f64::INFINITY,
                gradient_norm: f64::INFINITY,
                parameter_change: f64::INFINITY,
                convergence_score: 0.0,
                stagnation_count: 0,
            },
            depth_controller: AdaptiveDepthController {
                current_depth: config.initial_depth,
                increment_factor: 1.5,
                performance_history: Vec::new(),
                selection_strategy: DepthSelectionStrategy::PerformanceBased,
                convergence_detector: DepthConvergenceDetector {
                    improvement_history: Vec::new(),
                    convergence_threshold: 1e-6,
                    min_depths: 3,
                    confidence_level: 0.95,
                },
            },
            performance_stats: InfiniteQAOAStats {
                depths_explored: 0,
                total_time: Duration::from_secs(0),
                best_energy: f64::INFINITY,
                optimal_depth: 0,
                avg_convergence_time: Duration::from_secs(0),
                optimization_efficiency: 0.0,
            },
        }
    }

    /// Solve problem using infinite-depth QAOA
    pub fn solve<P>(&mut self, problem: &P) -> AdvancedQuantumResult<AnnealingResult<Vec<i32>>>
    where
        P: Clone,
    {
        // For compatibility with the coordinator, convert to the expected format
        if let Ok(ising_problem) = self.convert_to_ising(problem) {
            let solution = self.optimize(&ising_problem)?;
            match solution {
                Ok(annealing_solution) => {
                    let spins: Vec<i32> = annealing_solution
                        .best_spins
                        .iter()
                        .map(|&s| s as i32)
                        .collect();
                    Ok(Ok(spins))
                }
                Err(err) => Ok(Err(err)),
            }
        } else {
            Err(AdvancedQuantumError::ParameterError(
                "Cannot convert problem to Ising model".to_string(),
            ))
        }
    }

    /// Convert generic problem to Ising model (placeholder)
    fn convert_to_ising<P>(&self, _problem: &P) -> Result<IsingModel, AdvancedQuantumError> {
        // Placeholder implementation - in practice would convert problem type
        let mut ising = IsingModel::new(4);
        ising
            .set_bias(0, 1.0)
            .map_err(AdvancedQuantumError::IsingError)?;
        ising
            .set_bias(1, -1.0)
            .map_err(AdvancedQuantumError::IsingError)?;
        ising
            .set_coupling(0, 1, 0.5)
            .map_err(AdvancedQuantumError::IsingError)?;
        Ok(ising)
    }

    /// Optimize using infinite-depth QAOA
    pub fn optimize(
        &mut self,
        problem: &IsingModel,
    ) -> AdvancedQuantumResult<AnnealingResult<AnnealingSolution>> {
        println!("Starting ∞-QAOA optimization");
        let start_time = Instant::now();

        let mut current_depth = self.config.initial_depth;
        let mut best_result = None;
        let mut converged = false;

        while current_depth <= self.config.max_depth && !converged {
            println!("Optimizing at depth {}", current_depth);
            let depth_start_time = Instant::now();

            // Initialize parameters for current depth
            let initial_params = self.initialize_parameters(current_depth)?;

            // Optimize parameters at current depth
            let (optimized_params, energy) =
                self.optimize_at_depth(problem, current_depth, initial_params)?;

            // Record performance
            let depth_performance = DepthPerformance {
                depth: current_depth,
                best_energy: energy,
                iterations: self.config.max_iterations_per_depth, // Simplified
                convergence_time: depth_start_time.elapsed(),
                parameter_count: optimized_params.len(),
                improvement: if let Some(last_perf) =
                    self.depth_controller.performance_history.last()
                {
                    last_perf.best_energy - energy
                } else {
                    0.0
                },
            };

            self.depth_controller
                .performance_history
                .push(depth_performance);
            self.parameter_history.push(optimized_params.clone());
            self.energy_history.push(energy);
            self.depth_progression.push(current_depth);

            // Update best result
            if energy < self.convergence_metrics.best_energy {
                self.convergence_metrics.best_energy = energy;
                self.performance_stats.best_energy = energy;
                self.performance_stats.optimal_depth = current_depth;

                // Create result (simplified)
                best_result = Some(Ok(AnnealingSolution {
                    best_energy: energy,
                    best_spins: self.extract_solution_from_params(&optimized_params, problem)?,
                    repetitions: 1,
                    total_sweeps: current_depth * self.config.max_iterations_per_depth,
                    runtime: start_time.elapsed(),
                    info: format!("Infinite-depth QAOA with depth {}", current_depth),
                }));
            }

            // Check convergence
            converged = self.check_depth_convergence()?;

            // Determine next depth
            if !converged {
                current_depth = self.determine_next_depth(current_depth)?;
            }

            self.performance_stats.depths_explored += 1;
        }

        self.performance_stats.total_time = start_time.elapsed();
        self.performance_stats.avg_convergence_time = Duration::from_nanos(
            self.performance_stats.total_time.as_nanos() as u64
                / self.performance_stats.depths_explored.max(1) as u64,
        );

        println!(
            "∞-QAOA completed. Best energy: {:.6} at depth {}",
            self.performance_stats.best_energy, self.performance_stats.optimal_depth
        );

        best_result.ok_or_else(|| {
            AdvancedQuantumError::ConvergenceError("No valid result found".to_string())
        })
    }

    /// Initialize parameters for given depth
    fn initialize_parameters(&self, depth: usize) -> AdvancedQuantumResult<Vec<f64>> {
        let num_params = 2 * depth; // gamma and beta parameters

        match self.config.initialization_method {
            ParameterInitializationMethod::Random => {
                let mut rng = ChaCha8Rng::seed_from_u64(rand::random());
                Ok((0..num_params)
                    .map(|_| rng.gen_range(0.0..2.0 * PI))
                    .collect())
            }
            ParameterInitializationMethod::Heuristic => {
                let mut params = Vec::new();
                for i in 0..depth {
                    // Gamma parameters (problem Hamiltonian)
                    params.push(0.5 * PI / depth as f64);
                    // Beta parameters (mixer Hamiltonian)
                    params.push(0.25 * PI / depth as f64);
                }
                Ok(params)
            }
            ParameterInitializationMethod::Transfer => {
                if let Some(prev_params) = self.parameter_history.last() {
                    // Interpolate from previous depth
                    Ok(self.interpolate_parameters(prev_params, depth)?)
                } else {
                    // Fallback to heuristic
                    self.initialize_parameters_heuristic(depth)
                }
            }
            _ => {
                // Default to heuristic
                self.initialize_parameters_heuristic(depth)
            }
        }
    }

    /// Initialize parameters using heuristic method
    fn initialize_parameters_heuristic(&self, depth: usize) -> AdvancedQuantumResult<Vec<f64>> {
        let mut params = Vec::new();
        for _i in 0..depth {
            // Gamma parameters (problem Hamiltonian)
            params.push(0.5 * PI / depth as f64);
            // Beta parameters (mixer Hamiltonian)
            params.push(0.25 * PI / depth as f64);
        }
        Ok(params)
    }

    /// Interpolate parameters from previous depth
    fn interpolate_parameters(
        &self,
        prev_params: &[f64],
        new_depth: usize,
    ) -> AdvancedQuantumResult<Vec<f64>> {
        let prev_depth = prev_params.len() / 2;
        let new_param_count = 2 * new_depth;

        if new_depth <= prev_depth {
            // Truncate if new depth is smaller
            Ok(prev_params[..new_param_count].to_vec())
        } else {
            // Extend with interpolated values
            let mut new_params = prev_params.to_vec();

            for i in prev_depth..new_depth {
                // Simple interpolation strategy
                let gamma = if prev_depth > 0 {
                    prev_params[2 * (prev_depth - 1)]
                } else {
                    0.5 * PI / new_depth as f64
                };
                let beta = if prev_depth > 0 {
                    prev_params[2 * (prev_depth - 1) + 1]
                } else {
                    0.25 * PI / new_depth as f64
                };

                new_params.push(gamma * 0.8); // Scale down for stability
                new_params.push(beta * 0.8);
            }

            Ok(new_params)
        }
    }

    /// Optimize parameters at specific depth
    fn optimize_at_depth(
        &mut self,
        problem: &IsingModel,
        depth: usize,
        initial_params: Vec<f64>,
    ) -> AdvancedQuantumResult<(Vec<f64>, f64)> {
        let mut current_params = initial_params;
        let mut best_energy = f64::INFINITY;
        let mut best_params = current_params.clone();

        // Simple gradient-free optimization (in practice would use sophisticated methods)
        let mut rng = ChaCha8Rng::seed_from_u64(rand::random());

        for iteration in 0..self.config.max_iterations_per_depth {
            // Evaluate current parameters
            let energy = self.evaluate_qaoa_energy(problem, depth, &current_params)?;

            if energy < best_energy {
                best_energy = energy;
                best_params = current_params.clone();
            }

            // Simple parameter update (placeholder for actual optimization)
            for param in &mut current_params {
                *param += rng.gen_range(-0.1..0.1);
                *param = param.max(0.0).min(2.0 * PI); // Keep in valid range
            }

            // Check convergence
            if iteration > 10 && (best_energy - energy).abs() < self.config.optimization_tolerance {
                break;
            }
        }

        Ok((best_params, best_energy))
    }

    /// Evaluate QAOA energy (simplified)
    fn evaluate_qaoa_energy(
        &self,
        problem: &IsingModel,
        _depth: usize,
        params: &[f64],
    ) -> AdvancedQuantumResult<f64> {
        // Simplified energy evaluation
        // In practice, would construct quantum circuit and evaluate expectation value

        let mut energy = 0.0;

        // Simple approximation based on problem and parameters
        for i in 0..problem.num_qubits {
            if let Ok(bias) = problem.get_bias(i) {
                energy += bias * self.estimate_qubit_expectation(i, params);
            }
        }

        for i in 0..problem.num_qubits {
            for j in (i + 1)..problem.num_qubits {
                if let Ok(coupling) = problem.get_coupling(i, j) {
                    if coupling.abs() > 1e-10 {
                        energy += coupling * self.estimate_coupling_expectation(i, j, params);
                    }
                }
            }
        }

        Ok(energy)
    }

    /// Estimate single qubit expectation value
    fn estimate_qubit_expectation(&self, _qubit: usize, params: &[f64]) -> f64 {
        // Simplified expectation value estimation
        let depth = params.len() / 2;
        let mut expectation = 0.0;

        for d in 0..depth {
            let gamma = params[2 * d];
            let beta = params[2 * d + 1];

            // Simple approximation
            expectation += (gamma * beta).cos() / depth as f64;
        }

        expectation.tanh() // Keep in [-1, 1]
    }

    /// Estimate two-qubit coupling expectation value
    fn estimate_coupling_expectation(&self, qubit1: usize, qubit2: usize, params: &[f64]) -> f64 {
        // Simplified two-qubit expectation
        let exp1 = self.estimate_qubit_expectation(qubit1, params);
        let exp2 = self.estimate_qubit_expectation(qubit2, params);

        // Simple correlation approximation
        exp1 * exp2 * 0.8 // Reduce correlation strength
    }

    /// Extract solution from optimized parameters
    fn extract_solution_from_params(
        &self,
        params: &[f64],
        problem: &IsingModel,
    ) -> AdvancedQuantumResult<Vec<i8>> {
        let mut solution = Vec::new();

        for i in 0..problem.num_qubits {
            let expectation = self.estimate_qubit_expectation(i, params);
            solution.push(if expectation > 0.0 { 1 } else { -1 });
        }

        Ok(solution)
    }

    /// Check convergence across depths
    fn check_depth_convergence(&mut self) -> AdvancedQuantumResult<bool> {
        let history_len = self.depth_controller.performance_history.len();

        if history_len < self.depth_controller.convergence_detector.min_depths {
            return Ok(false);
        }

        // Check energy improvement trend
        let recent_improvements: Vec<f64> = self
            .depth_controller
            .performance_history
            .windows(2)
            .map(|window| window[0].best_energy - window[1].best_energy)
            .collect();

        let avg_improvement =
            recent_improvements.iter().sum::<f64>() / recent_improvements.len() as f64;

        // Converged if improvements are below threshold
        Ok(avg_improvement
            < self
                .depth_controller
                .convergence_detector
                .convergence_threshold)
    }

    /// Determine next depth to explore
    fn determine_next_depth(&self, current_depth: usize) -> AdvancedQuantumResult<usize> {
        match self.config.depth_strategy {
            DepthIncrementStrategy::Linear => Ok(current_depth + 1),
            DepthIncrementStrategy::Exponential => Ok((current_depth as f64 * 1.5) as usize),
            DepthIncrementStrategy::Adaptive => {
                // Base decision on recent performance
                if let Some(last_perf) = self.depth_controller.performance_history.last() {
                    if last_perf.improvement > 0.01 {
                        Ok(current_depth + 1) // Small increment for good improvement
                    } else {
                        Ok(current_depth + 2) // Larger increment for poor improvement
                    }
                } else {
                    Ok(current_depth + 1)
                }
            }
            DepthIncrementStrategy::GoldenRatio => Ok((current_depth as f64 * 1.618) as usize),
            DepthIncrementStrategy::Fibonacci => {
                // Simple Fibonacci-like increment
                Ok(current_depth + (current_depth / 2).max(1))
            }
        }
    }
}

/// Create default infinite-depth QAOA optimizer
pub fn create_infinite_qaoa_optimizer() -> InfiniteDepthQAOA {
    InfiniteDepthQAOA::new(InfiniteQAOAConfig::default())
}

/// Create infinite-depth QAOA with custom configuration
pub fn create_custom_infinite_qaoa(
    max_depth: usize,
    depth_strategy: DepthIncrementStrategy,
    initialization_method: ParameterInitializationMethod,
) -> InfiniteDepthQAOA {
    let mut config = InfiniteQAOAConfig::default();
    config.max_depth = max_depth;
    config.depth_strategy = depth_strategy;
    config.initialization_method = initialization_method;

    InfiniteDepthQAOA::new(config)
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_infinite_qaoa_creation() {
        let optimizer = create_infinite_qaoa_optimizer();
        assert_eq!(optimizer.config.initial_depth, 1);
        assert_eq!(optimizer.config.max_depth, 100);
        assert_eq!(optimizer.depth_controller.current_depth, 1);
    }

    #[test]
    fn test_parameter_initialization() {
        let optimizer = create_infinite_qaoa_optimizer();
        let params = optimizer.initialize_parameters(3).unwrap();
        assert_eq!(params.len(), 6); // 2 * depth

        for &param in &params {
            assert!(param >= 0.0 && param <= 2.0 * PI);
        }
    }

    #[test]
    fn test_parameter_interpolation() {
        let optimizer = create_infinite_qaoa_optimizer();
        let prev_params = vec![1.0, 2.0, 3.0, 4.0]; // depth 2
        let interpolated = optimizer.interpolate_parameters(&prev_params, 3).unwrap();

        assert_eq!(interpolated.len(), 6); // 2 * 3
        assert_eq!(interpolated[0], 1.0);
        assert_eq!(interpolated[1], 2.0);
        assert_eq!(interpolated[2], 3.0);
        assert_eq!(interpolated[3], 4.0);
        // Last two should be scaled versions
        assert!(interpolated[4] < 3.0);
        assert!(interpolated[5] < 4.0);
    }

    #[test]
    fn test_depth_increment_strategies() {
        let mut optimizer = create_infinite_qaoa_optimizer();

        // Test linear increment
        optimizer.config.depth_strategy = DepthIncrementStrategy::Linear;
        assert_eq!(optimizer.determine_next_depth(5).unwrap(), 6);

        // Test exponential increment
        optimizer.config.depth_strategy = DepthIncrementStrategy::Exponential;
        assert_eq!(optimizer.determine_next_depth(4).unwrap(), 6); // 4 * 1.5 = 6

        // Test golden ratio increment
        optimizer.config.depth_strategy = DepthIncrementStrategy::GoldenRatio;
        assert_eq!(optimizer.determine_next_depth(3).unwrap(), 4); // 3 * 1.618 ≈ 4.85 -> 4
    }
}
