//! Keras-style model building API for QuantRS2-ML
//!
//! This module provides a Keras-like interface for building quantum machine learning
//! models, with both Sequential and Functional API patterns familiar to Keras users.

use crate::circuit_integration::{QuantumLayer, QuantumMLExecutor};
use crate::error::{MLError, Result};
use crate::simulator_backends::{
    BackendCapabilities, DynamicCircuit, Observable, SimulatorBackend, StatevectorBackend,
};
use ndarray::{s, Array1, Array2, Array3, ArrayD, Axis, IxDyn};
use quantrs2_circuit::prelude::*;
use quantrs2_core::prelude::*;
use std::collections::HashMap;
use std::sync::Arc;

/// Keras-style layer trait
pub trait KerasLayer: Send + Sync {
    /// Build the layer (called during model compilation)
    fn build(&mut self, input_shape: &[usize]) -> Result<()>;

    /// Forward pass through the layer
    fn call(&self, inputs: &ArrayD<f64>) -> Result<ArrayD<f64>>;

    /// Compute output shape given input shape
    fn compute_output_shape(&self, input_shape: &[usize]) -> Vec<usize>;

    /// Get layer name
    fn name(&self) -> &str;

    /// Get trainable parameters
    fn get_weights(&self) -> Vec<ArrayD<f64>>;

    /// Set trainable parameters
    fn set_weights(&mut self, weights: Vec<ArrayD<f64>>) -> Result<()>;

    /// Get number of parameters
    fn count_params(&self) -> usize {
        self.get_weights().iter().map(|w| w.len()).sum()
    }

    /// Check if layer is built
    fn built(&self) -> bool;
}

/// Dense (fully connected) layer
pub struct Dense {
    /// Number of units
    units: usize,
    /// Activation function
    activation: Option<ActivationFunction>,
    /// Use bias
    use_bias: bool,
    /// Kernel initializer
    kernel_initializer: InitializerType,
    /// Bias initializer
    bias_initializer: InitializerType,
    /// Layer name
    name: String,
    /// Built flag
    built: bool,
    /// Input shape
    input_shape: Option<Vec<usize>>,
    /// Weights (kernel and bias)
    weights: Vec<ArrayD<f64>>,
}

impl Dense {
    /// Create new dense layer
    pub fn new(units: usize) -> Self {
        Self {
            units,
            activation: None,
            use_bias: true,
            kernel_initializer: InitializerType::GlorotUniform,
            bias_initializer: InitializerType::Zeros,
            name: format!("dense_{}", fastrand::u32(..)),
            built: false,
            input_shape: None,
            weights: Vec::new(),
        }
    }

    /// Set activation function
    pub fn activation(mut self, activation: ActivationFunction) -> Self {
        self.activation = Some(activation);
        self
    }

    /// Set use bias
    pub fn use_bias(mut self, use_bias: bool) -> Self {
        self.use_bias = use_bias;
        self
    }

    /// Set layer name
    pub fn name(mut self, name: impl Into<String>) -> Self {
        self.name = name.into();
        self
    }

    /// Set kernel initializer
    pub fn kernel_initializer(mut self, initializer: InitializerType) -> Self {
        self.kernel_initializer = initializer;
        self
    }
}

impl KerasLayer for Dense {
    fn build(&mut self, input_shape: &[usize]) -> Result<()> {
        if input_shape.is_empty() {
            return Err(MLError::InvalidConfiguration(
                "Dense layer requires input shape".to_string(),
            ));
        }

        let input_dim = input_shape[input_shape.len() - 1];
        self.input_shape = Some(input_shape.to_vec());

        // Initialize kernel weights
        let kernel = self.initialize_weights(&[input_dim, self.units], &self.kernel_initializer)?;
        self.weights.push(kernel);

        // Initialize bias weights
        if self.use_bias {
            let bias = self.initialize_weights(&[self.units], &self.bias_initializer)?;
            self.weights.push(bias);
        }

        self.built = true;
        Ok(())
    }

    fn call(&self, inputs: &ArrayD<f64>) -> Result<ArrayD<f64>> {
        if !self.built {
            return Err(MLError::InvalidConfiguration(
                "Layer must be built before calling".to_string(),
            ));
        }

        let kernel = &self.weights[0];
        // Explicitly perform matrix multiplication to avoid deep recursion
        let outputs = match (inputs.ndim(), kernel.ndim()) {
            (2, 2) => {
                // Convert to 2D arrays for explicit dot product
                let inputs_2d = inputs
                    .clone()
                    .into_dimensionality::<ndarray::Ix2>()
                    .map_err(|_| MLError::InvalidConfiguration("Input must be 2D".to_string()))?;
                let kernel_2d = kernel
                    .clone()
                    .into_dimensionality::<ndarray::Ix2>()
                    .map_err(|_| MLError::InvalidConfiguration("Kernel must be 2D".to_string()))?;
                inputs_2d.dot(&kernel_2d).into_dyn()
            }
            _ => {
                return Err(MLError::InvalidConfiguration(
                    "Unsupported array dimensions for matrix multiplication".to_string(),
                ));
            }
        };
        let mut outputs = outputs;

        // Add bias if used
        if self.use_bias && self.weights.len() > 1 {
            let bias = &self.weights[1];
            outputs = outputs + bias;
        }

        // Apply activation
        if let Some(ref activation) = self.activation {
            outputs = self.apply_activation(&outputs, activation)?;
        }

        Ok(outputs)
    }

    fn compute_output_shape(&self, input_shape: &[usize]) -> Vec<usize> {
        let mut output_shape = input_shape.to_vec();
        let last_idx = output_shape.len() - 1;
        output_shape[last_idx] = self.units;
        output_shape
    }

    fn name(&self) -> &str {
        &self.name
    }

    fn get_weights(&self) -> Vec<ArrayD<f64>> {
        self.weights.clone()
    }

    fn set_weights(&mut self, weights: Vec<ArrayD<f64>>) -> Result<()> {
        if weights.len() != self.weights.len() {
            return Err(MLError::InvalidConfiguration(
                "Number of weight arrays doesn't match layer structure".to_string(),
            ));
        }
        self.weights = weights;
        Ok(())
    }

    fn built(&self) -> bool {
        self.built
    }
}

impl Dense {
    /// Initialize weights
    fn initialize_weights(
        &self,
        shape: &[usize],
        initializer: &InitializerType,
    ) -> Result<ArrayD<f64>> {
        match initializer {
            InitializerType::Zeros => Ok(ArrayD::zeros(shape)),
            InitializerType::Ones => Ok(ArrayD::ones(shape)),
            InitializerType::GlorotUniform => {
                let fan_in = if shape.len() >= 2 { shape[0] } else { 1 };
                let fan_out = if shape.len() >= 2 { shape[1] } else { shape[0] };
                let limit = (6.0 / (fan_in + fan_out) as f64).sqrt();

                Ok(ArrayD::from_shape_fn(shape, |_| {
                    fastrand::f64() * 2.0 * limit - limit
                }))
            }
            InitializerType::GlorotNormal => {
                let fan_in = if shape.len() >= 2 { shape[0] } else { 1 };
                let fan_out = if shape.len() >= 2 { shape[1] } else { shape[0] };
                let std = (2.0 / (fan_in + fan_out) as f64).sqrt();

                Ok(ArrayD::from_shape_fn(shape, |_| {
                    // Box-Muller transform for normal distribution
                    let u1 = fastrand::f64();
                    let u2 = fastrand::f64();
                    let z = (-2.0 * u1.ln()).sqrt() * (2.0 * std::f64::consts::PI * u2).cos();
                    z * std
                }))
            }
            InitializerType::HeUniform => {
                let fan_in = if shape.len() >= 2 { shape[0] } else { 1 };
                let limit = (6.0 / fan_in as f64).sqrt();

                Ok(ArrayD::from_shape_fn(shape, |_| {
                    fastrand::f64() * 2.0 * limit - limit
                }))
            }
        }
    }

    /// Apply activation function
    fn apply_activation(
        &self,
        inputs: &ArrayD<f64>,
        activation: &ActivationFunction,
    ) -> Result<ArrayD<f64>> {
        Ok(match activation {
            ActivationFunction::Linear => inputs.clone(),
            ActivationFunction::ReLU => inputs.mapv(|x| x.max(0.0)),
            ActivationFunction::Sigmoid => inputs.mapv(|x| 1.0 / (1.0 + (-x).exp())),
            ActivationFunction::Tanh => inputs.mapv(|x| x.tanh()),
            ActivationFunction::Softmax => {
                let mut outputs = inputs.clone();
                for mut row in outputs.axis_iter_mut(Axis(0)) {
                    let max_val = row.iter().fold(f64::NEG_INFINITY, |a, &b| a.max(b));
                    row.mapv_inplace(|x| (x - max_val).exp());
                    let sum = row.sum();
                    row /= sum;
                }
                outputs
            }
            ActivationFunction::LeakyReLU(alpha) => {
                inputs.mapv(|x| if x > 0.0 { x } else { alpha * x })
            }
            ActivationFunction::ELU(alpha) => {
                inputs.mapv(|x| if x > 0.0 { x } else { alpha * (x.exp() - 1.0) })
            }
        })
    }
}

/// Quantum Dense layer
pub struct QuantumDense {
    /// Number of qubits
    num_qubits: usize,
    /// Number of output features
    units: usize,
    /// Quantum circuit ansatz
    ansatz_type: QuantumAnsatzType,
    /// Number of layers in ansatz
    num_layers: usize,
    /// Observable for measurement
    observable: Observable,
    /// Backend
    backend: Arc<dyn SimulatorBackend>,
    /// Layer name
    name: String,
    /// Built flag
    built: bool,
    /// Input shape
    input_shape: Option<Vec<usize>>,
    /// Quantum parameters
    quantum_weights: Vec<ArrayD<f64>>,
}

/// Quantum ansatz types
#[derive(Debug, Clone)]
pub enum QuantumAnsatzType {
    /// Hardware efficient ansatz
    HardwareEfficient,
    /// Real amplitudes ansatz
    RealAmplitudes,
    /// Strongly entangling layers
    StronglyEntangling,
    /// Custom ansatz
    Custom(DynamicCircuit),
}

impl QuantumDense {
    /// Create new quantum dense layer
    pub fn new(num_qubits: usize, units: usize) -> Self {
        Self {
            num_qubits,
            units,
            ansatz_type: QuantumAnsatzType::HardwareEfficient,
            num_layers: 1,
            observable: Observable::PauliZ(vec![0]),
            backend: Arc::new(StatevectorBackend::new(10)),
            name: format!("quantum_dense_{}", fastrand::u32(..)),
            built: false,
            input_shape: None,
            quantum_weights: Vec::new(),
        }
    }

    /// Set ansatz type
    pub fn ansatz_type(mut self, ansatz_type: QuantumAnsatzType) -> Self {
        self.ansatz_type = ansatz_type;
        self
    }

    /// Set number of layers
    pub fn num_layers(mut self, num_layers: usize) -> Self {
        self.num_layers = num_layers;
        self
    }

    /// Set observable
    pub fn observable(mut self, observable: Observable) -> Self {
        self.observable = observable;
        self
    }

    /// Set backend
    pub fn backend(mut self, backend: Arc<dyn SimulatorBackend>) -> Self {
        self.backend = backend;
        self
    }

    /// Set layer name
    pub fn name(mut self, name: impl Into<String>) -> Self {
        self.name = name.into();
        self
    }
}

impl KerasLayer for QuantumDense {
    fn build(&mut self, input_shape: &[usize]) -> Result<()> {
        self.input_shape = Some(input_shape.to_vec());

        // Calculate number of parameters needed
        let num_params = match &self.ansatz_type {
            QuantumAnsatzType::HardwareEfficient => {
                // 2 rotation gates per qubit per layer + entangling gates
                self.num_qubits * 2 * self.num_layers
            }
            QuantumAnsatzType::RealAmplitudes => {
                // Y rotation per qubit per layer
                self.num_qubits * self.num_layers
            }
            QuantumAnsatzType::StronglyEntangling => {
                // 3 rotation gates per qubit per layer
                self.num_qubits * 3 * self.num_layers
            }
            QuantumAnsatzType::Custom(_) => {
                // Would need to count parameterized gates in custom circuit
                10 // Placeholder
            }
        };

        // Initialize quantum parameters
        let params = ArrayD::from_shape_fn(IxDyn(&[self.units, num_params]), |_| {
            fastrand::f64() * 2.0 * std::f64::consts::PI
        });
        self.quantum_weights.push(params);

        self.built = true;
        Ok(())
    }

    fn call(&self, inputs: &ArrayD<f64>) -> Result<ArrayD<f64>> {
        if !self.built {
            return Err(MLError::InvalidConfiguration(
                "Layer must be built before calling".to_string(),
            ));
        }

        let batch_size = inputs.shape()[0];
        let mut outputs = ArrayD::zeros(IxDyn(&[batch_size, self.units]));

        for batch_idx in 0..batch_size {
            for unit_idx in 0..self.units {
                // Build quantum circuit for this unit
                let circuit = self.build_quantum_circuit()?;

                // Get input data and parameters
                let input_slice = inputs.slice(s![batch_idx, ..]);
                let param_slice = self.quantum_weights[0].slice(s![unit_idx, ..]);

                // Combine input data with parameters
                let combined_params: Vec<f64> = input_slice
                    .iter()
                    .chain(param_slice.iter())
                    .copied()
                    .collect();

                // Execute quantum circuit
                let expectation =
                    self.backend
                        .expectation_value(&circuit, &combined_params, &self.observable)?;

                outputs[[batch_idx, unit_idx]] = expectation;
            }
        }

        Ok(outputs)
    }

    fn compute_output_shape(&self, input_shape: &[usize]) -> Vec<usize> {
        let mut output_shape = input_shape.to_vec();
        let last_idx = output_shape.len() - 1;
        output_shape[last_idx] = self.units;
        output_shape
    }

    fn name(&self) -> &str {
        &self.name
    }

    fn get_weights(&self) -> Vec<ArrayD<f64>> {
        self.quantum_weights.clone()
    }

    fn set_weights(&mut self, weights: Vec<ArrayD<f64>>) -> Result<()> {
        if weights.len() != self.quantum_weights.len() {
            return Err(MLError::InvalidConfiguration(
                "Number of weight arrays doesn't match layer structure".to_string(),
            ));
        }
        self.quantum_weights = weights;
        Ok(())
    }

    fn built(&self) -> bool {
        self.built
    }
}

impl QuantumDense {
    /// Build quantum circuit based on ansatz type
    fn build_quantum_circuit(&self) -> Result<DynamicCircuit> {
        let mut builder: Circuit<8> = Circuit::new();

        match &self.ansatz_type {
            QuantumAnsatzType::HardwareEfficient => {
                for layer in 0..self.num_layers {
                    // Data encoding (if first layer)
                    if layer == 0 {
                        for qubit in 0..self.num_qubits {
                            builder.ry(qubit, 0.0)?; // Input parameter
                        }
                    }

                    // Variational part
                    for qubit in 0..self.num_qubits {
                        builder.ry(qubit, 0.0)?; // Trainable parameter
                        builder.rz(qubit, 0.0)?; // Trainable parameter
                    }

                    // Entangling gates
                    for qubit in 0..self.num_qubits - 1 {
                        builder.cnot(qubit, qubit + 1)?;
                    }
                    if self.num_qubits > 2 {
                        builder.cnot(self.num_qubits - 1, 0)?;
                    }
                }
            }
            QuantumAnsatzType::RealAmplitudes => {
                for layer in 0..self.num_layers {
                    // Data encoding (if first layer)
                    if layer == 0 {
                        for qubit in 0..self.num_qubits {
                            builder.ry(qubit, 0.0)?; // Input parameter
                        }
                    }

                    // Variational part
                    for qubit in 0..self.num_qubits {
                        builder.ry(qubit, 0.0)?; // Trainable parameter
                    }

                    // Entangling gates
                    for qubit in 0..self.num_qubits - 1 {
                        builder.cnot(qubit, qubit + 1)?;
                    }
                }
            }
            QuantumAnsatzType::StronglyEntangling => {
                for layer in 0..self.num_layers {
                    // Data encoding (if first layer)
                    if layer == 0 {
                        for qubit in 0..self.num_qubits {
                            builder.ry(qubit, 0.0)?; // Input parameter
                        }
                    }

                    // Variational part - all rotation gates
                    for qubit in 0..self.num_qubits {
                        builder.rx(qubit, 0.0)?; // Trainable parameter
                        builder.ry(qubit, 0.0)?; // Trainable parameter
                        builder.rz(qubit, 0.0)?; // Trainable parameter
                    }

                    // Entangling gates
                    for qubit in 0..self.num_qubits - 1 {
                        builder.cnot(qubit, qubit + 1)?;
                    }
                    if self.num_qubits > 2 {
                        builder.cnot(self.num_qubits - 1, 0)?;
                    }
                }
            }
            QuantumAnsatzType::Custom(circuit) => {
                return Ok(circuit.clone());
            }
        }

        let circuit = builder.build();
        DynamicCircuit::from_circuit(circuit)
    }
}

/// Activation function types
#[derive(Debug, Clone)]
pub enum ActivationFunction {
    /// Linear activation (identity)
    Linear,
    /// ReLU activation
    ReLU,
    /// Sigmoid activation
    Sigmoid,
    /// Tanh activation
    Tanh,
    /// Softmax activation
    Softmax,
    /// Leaky ReLU with alpha
    LeakyReLU(f64),
    /// ELU with alpha
    ELU(f64),
}

/// Activation layer
pub struct Activation {
    /// Activation function
    function: ActivationFunction,
    /// Layer name
    name: String,
    /// Built flag
    built: bool,
}

impl Activation {
    /// Create new activation layer
    pub fn new(function: ActivationFunction) -> Self {
        Self {
            function,
            name: format!("activation_{}", fastrand::u32(..)),
            built: false,
        }
    }

    /// Set layer name
    pub fn name(mut self, name: impl Into<String>) -> Self {
        self.name = name.into();
        self
    }
}

impl KerasLayer for Activation {
    fn build(&mut self, _input_shape: &[usize]) -> Result<()> {
        self.built = true;
        Ok(())
    }

    fn call(&self, inputs: &ArrayD<f64>) -> Result<ArrayD<f64>> {
        Ok(match &self.function {
            ActivationFunction::Linear => inputs.clone(),
            ActivationFunction::ReLU => inputs.mapv(|x| x.max(0.0)),
            ActivationFunction::Sigmoid => inputs.mapv(|x| 1.0 / (1.0 + (-x).exp())),
            ActivationFunction::Tanh => inputs.mapv(|x| x.tanh()),
            ActivationFunction::Softmax => {
                let mut outputs = inputs.clone();
                for mut row in outputs.axis_iter_mut(Axis(0)) {
                    let max_val = row.iter().fold(f64::NEG_INFINITY, |a, &b| a.max(b));
                    row.mapv_inplace(|x| (x - max_val).exp());
                    let sum = row.sum();
                    row /= sum;
                }
                outputs
            }
            ActivationFunction::LeakyReLU(alpha) => {
                inputs.mapv(|x| if x > 0.0 { x } else { alpha * x })
            }
            ActivationFunction::ELU(alpha) => {
                inputs.mapv(|x| if x > 0.0 { x } else { alpha * (x.exp() - 1.0) })
            }
        })
    }

    fn compute_output_shape(&self, input_shape: &[usize]) -> Vec<usize> {
        input_shape.to_vec()
    }

    fn name(&self) -> &str {
        &self.name
    }

    fn get_weights(&self) -> Vec<ArrayD<f64>> {
        Vec::new()
    }

    fn set_weights(&mut self, _weights: Vec<ArrayD<f64>>) -> Result<()> {
        Ok(())
    }

    fn built(&self) -> bool {
        self.built
    }
}

/// Weight initializer types
#[derive(Debug, Clone)]
pub enum InitializerType {
    /// All zeros
    Zeros,
    /// All ones
    Ones,
    /// Glorot uniform (Xavier uniform)
    GlorotUniform,
    /// Glorot normal (Xavier normal)
    GlorotNormal,
    /// He uniform
    HeUniform,
}

/// Sequential model
pub struct Sequential {
    /// Layers in the model
    layers: Vec<Box<dyn KerasLayer>>,
    /// Model name
    name: String,
    /// Built flag
    built: bool,
    /// Compiled flag
    compiled: bool,
    /// Input shape
    input_shape: Option<Vec<usize>>,
    /// Loss function
    loss: Option<LossFunction>,
    /// Optimizer
    optimizer: Option<OptimizerType>,
    /// Metrics
    metrics: Vec<MetricType>,
}

impl Sequential {
    /// Create new sequential model
    pub fn new() -> Self {
        Self {
            layers: Vec::new(),
            name: format!("sequential_{}", fastrand::u32(..)),
            built: false,
            compiled: false,
            input_shape: None,
            loss: None,
            optimizer: None,
            metrics: Vec::new(),
        }
    }

    /// Set model name
    pub fn name(mut self, name: impl Into<String>) -> Self {
        self.name = name.into();
        self
    }

    /// Add layer to model
    pub fn add(&mut self, layer: Box<dyn KerasLayer>) {
        self.layers.push(layer);
        self.built = false; // Mark as needing rebuild
    }

    /// Build the model with given input shape
    pub fn build(&mut self, input_shape: Vec<usize>) -> Result<()> {
        self.input_shape = Some(input_shape.clone());
        let mut current_shape = input_shape;

        for layer in &mut self.layers {
            layer.build(&current_shape)?;
            current_shape = layer.compute_output_shape(&current_shape);
        }

        self.built = true;
        Ok(())
    }

    /// Compile the model
    pub fn compile(
        mut self,
        loss: LossFunction,
        optimizer: OptimizerType,
        metrics: Vec<MetricType>,
    ) -> Self {
        self.loss = Some(loss);
        self.optimizer = Some(optimizer);
        self.metrics = metrics;
        self.compiled = true;
        self
    }

    /// Get model summary
    pub fn summary(&self) -> ModelSummary {
        let mut layers_info = Vec::new();
        let mut total_params = 0;
        let mut trainable_params = 0;

        let mut current_shape = self.input_shape.clone().unwrap_or_default();

        for layer in &self.layers {
            let output_shape = layer.compute_output_shape(&current_shape);
            let params = layer.count_params();

            layers_info.push(LayerInfo {
                name: layer.name().to_string(),
                layer_type: "Layer".to_string(), // Would be more specific in real implementation
                output_shape: output_shape.clone(),
                param_count: params,
            });

            total_params += params;
            trainable_params += params; // Assuming all params are trainable
            current_shape = output_shape;
        }

        ModelSummary {
            layers: layers_info,
            total_params,
            trainable_params,
            non_trainable_params: 0,
        }
    }

    /// Forward pass (predict)
    pub fn predict(&self, inputs: &ArrayD<f64>) -> Result<ArrayD<f64>> {
        if !self.built {
            return Err(MLError::InvalidConfiguration(
                "Model must be built before prediction".to_string(),
            ));
        }

        let mut current = inputs.clone();

        for layer in &self.layers {
            current = layer.call(&current)?;
        }

        Ok(current)
    }

    /// Train the model
    pub fn fit(
        &mut self,
        X: &ArrayD<f64>,
        y: &ArrayD<f64>,
        epochs: usize,
        batch_size: Option<usize>,
        validation_data: Option<(&ArrayD<f64>, &ArrayD<f64>)>,
        callbacks: Vec<Box<dyn Callback>>,
    ) -> Result<TrainingHistory> {
        if !self.compiled {
            return Err(MLError::InvalidConfiguration(
                "Model must be compiled before training".to_string(),
            ));
        }

        let batch_size = batch_size.unwrap_or(32);
        let n_samples = X.shape()[0];
        let n_batches = (n_samples + batch_size - 1) / batch_size;

        let mut history = TrainingHistory::new();

        for epoch in 0..epochs {
            let mut epoch_loss = 0.0;
            let mut epoch_metrics: HashMap<String, f64> = HashMap::new();

            // Initialize metrics
            for metric in &self.metrics {
                epoch_metrics.insert(metric.name(), 0.0);
            }

            // Training loop
            for batch_idx in 0..n_batches {
                let start_idx = batch_idx * batch_size;
                let end_idx = ((batch_idx + 1) * batch_size).min(n_samples);

                // Get batch data
                let X_batch = X.slice(s![start_idx..end_idx, ..]);
                let y_batch = y.slice(s![start_idx..end_idx, ..]);

                // Forward pass
                let predictions = self.predict(&X_batch.to_owned().into_dyn())?;

                // Compute loss
                let loss = self.compute_loss(&predictions, &y_batch.to_owned().into_dyn())?;
                epoch_loss += loss;

                // Compute gradients and update weights (placeholder)
                self.backward_pass(&predictions, &y_batch.to_owned().into_dyn())?;

                // Compute metrics
                for metric in &self.metrics {
                    let metric_value =
                        metric.compute(&predictions, &y_batch.to_owned().into_dyn())?;
                    *epoch_metrics.get_mut(&metric.name()).unwrap() += metric_value;
                }
            }

            // Average loss and metrics
            epoch_loss /= n_batches as f64;
            for value in epoch_metrics.values_mut() {
                *value /= n_batches as f64;
            }

            // Validation
            let (val_loss, val_metrics) = if let Some((X_val, y_val)) = validation_data {
                let val_predictions = self.predict(X_val)?;
                let val_loss = self.compute_loss(&val_predictions, y_val)?;

                let mut val_metrics = HashMap::new();
                for metric in &self.metrics {
                    let metric_value = metric.compute(&val_predictions, y_val)?;
                    val_metrics.insert(format!("val_{}", metric.name()), metric_value);
                }

                (Some(val_loss), val_metrics)
            } else {
                (None, HashMap::new())
            };

            // Update history
            history.add_epoch(epoch_loss, epoch_metrics, val_loss, val_metrics);

            // Call callbacks
            for callback in &callbacks {
                callback.on_epoch_end(epoch, &history)?;
            }

            println!("Epoch {}/{} - loss: {:.4}", epoch + 1, epochs, epoch_loss);
        }

        Ok(history)
    }

    /// Evaluate the model
    pub fn evaluate(
        &self,
        X: &ArrayD<f64>,
        y: &ArrayD<f64>,
        batch_size: Option<usize>,
    ) -> Result<HashMap<String, f64>> {
        let predictions = self.predict(X)?;
        let loss = self.compute_loss(&predictions, y)?;

        let mut results = HashMap::new();
        results.insert("loss".to_string(), loss);

        for metric in &self.metrics {
            let metric_value = metric.compute(&predictions, y)?;
            results.insert(metric.name(), metric_value);
        }

        Ok(results)
    }

    /// Compute loss
    fn compute_loss(&self, predictions: &ArrayD<f64>, targets: &ArrayD<f64>) -> Result<f64> {
        if let Some(ref loss_fn) = self.loss {
            loss_fn.compute(predictions, targets)
        } else {
            Err(MLError::InvalidConfiguration(
                "Loss function not specified".to_string(),
            ))
        }
    }

    /// Backward pass (placeholder)
    fn backward_pass(&mut self, _predictions: &ArrayD<f64>, _targets: &ArrayD<f64>) -> Result<()> {
        // Placeholder for gradient computation and weight updates
        // In a real implementation, this would compute gradients and update weights
        Ok(())
    }
}

/// Loss functions
#[derive(Debug, Clone)]
pub enum LossFunction {
    /// Mean squared error
    MeanSquaredError,
    /// Binary crossentropy
    BinaryCrossentropy,
    /// Categorical crossentropy
    CategoricalCrossentropy,
    /// Sparse categorical crossentropy
    SparseCategoricalCrossentropy,
    /// Mean absolute error
    MeanAbsoluteError,
    /// Huber loss
    Huber(f64),
}

impl LossFunction {
    /// Compute loss
    pub fn compute(&self, predictions: &ArrayD<f64>, targets: &ArrayD<f64>) -> Result<f64> {
        match self {
            LossFunction::MeanSquaredError => {
                let diff = predictions - targets;
                Ok(diff.mapv(|x| x * x).mean().unwrap())
            }
            LossFunction::BinaryCrossentropy => {
                let epsilon = 1e-15;
                let clipped_preds = predictions.mapv(|x| x.max(epsilon).min(1.0 - epsilon));
                let loss = targets * clipped_preds.mapv(|x| x.ln())
                    + (1.0 - targets) * clipped_preds.mapv(|x| (1.0 - x).ln());
                Ok(-loss.mean().unwrap())
            }
            LossFunction::MeanAbsoluteError => {
                let diff = predictions - targets;
                Ok(diff.mapv(|x| x.abs()).mean().unwrap())
            }
            _ => Err(MLError::InvalidConfiguration(
                "Loss function not implemented".to_string(),
            )),
        }
    }
}

/// Optimizer types
#[derive(Debug, Clone)]
pub enum OptimizerType {
    /// Stochastic Gradient Descent
    SGD { learning_rate: f64, momentum: f64 },
    /// Adam optimizer
    Adam {
        learning_rate: f64,
        beta1: f64,
        beta2: f64,
        epsilon: f64,
    },
    /// RMSprop optimizer
    RMSprop {
        learning_rate: f64,
        rho: f64,
        epsilon: f64,
    },
    /// AdaGrad optimizer
    AdaGrad { learning_rate: f64, epsilon: f64 },
}

/// Metric types
#[derive(Debug, Clone)]
pub enum MetricType {
    /// Accuracy
    Accuracy,
    /// Precision
    Precision,
    /// Recall
    Recall,
    /// F1 Score
    F1Score,
    /// Mean Absolute Error
    MeanAbsoluteError,
    /// Mean Squared Error
    MeanSquaredError,
}

impl MetricType {
    /// Get metric name
    pub fn name(&self) -> String {
        match self {
            MetricType::Accuracy => "accuracy".to_string(),
            MetricType::Precision => "precision".to_string(),
            MetricType::Recall => "recall".to_string(),
            MetricType::F1Score => "f1_score".to_string(),
            MetricType::MeanAbsoluteError => "mean_absolute_error".to_string(),
            MetricType::MeanSquaredError => "mean_squared_error".to_string(),
        }
    }

    /// Compute metric
    pub fn compute(&self, predictions: &ArrayD<f64>, targets: &ArrayD<f64>) -> Result<f64> {
        match self {
            MetricType::Accuracy => {
                let pred_classes = predictions.mapv(|x| if x > 0.5 { 1.0 } else { 0.0 });
                let correct = pred_classes
                    .iter()
                    .zip(targets.iter())
                    .filter(|(&pred, &target)| (pred - target).abs() < 1e-6)
                    .count();
                Ok(correct as f64 / targets.len() as f64)
            }
            MetricType::MeanAbsoluteError => {
                let diff = predictions - targets;
                Ok(diff.mapv(|x| x.abs()).mean().unwrap())
            }
            MetricType::MeanSquaredError => {
                let diff = predictions - targets;
                Ok(diff.mapv(|x| x * x).mean().unwrap())
            }
            _ => Err(MLError::InvalidConfiguration(
                "Metric not implemented".to_string(),
            )),
        }
    }
}

/// Callback trait for training
pub trait Callback: Send + Sync {
    /// Called at the end of each epoch
    fn on_epoch_end(&self, epoch: usize, history: &TrainingHistory) -> Result<()>;
}

/// Early stopping callback
pub struct EarlyStopping {
    /// Metric to monitor
    monitor: String,
    /// Minimum change to qualify as improvement
    min_delta: f64,
    /// Number of epochs with no improvement to wait
    patience: usize,
    /// Best value seen so far
    best: f64,
    /// Number of epochs without improvement
    wait: usize,
    /// Whether to stop training
    stopped: bool,
}

impl EarlyStopping {
    /// Create new early stopping callback
    pub fn new(monitor: String, min_delta: f64, patience: usize) -> Self {
        Self {
            monitor,
            min_delta,
            patience,
            best: f64::INFINITY,
            wait: 0,
            stopped: false,
        }
    }
}

impl Callback for EarlyStopping {
    fn on_epoch_end(&self, _epoch: usize, _history: &TrainingHistory) -> Result<()> {
        // Placeholder implementation
        Ok(())
    }
}

/// Training history
#[derive(Debug, Clone)]
pub struct TrainingHistory {
    /// Training loss for each epoch
    pub loss: Vec<f64>,
    /// Training metrics for each epoch
    pub metrics: Vec<HashMap<String, f64>>,
    /// Validation loss for each epoch
    pub val_loss: Vec<f64>,
    /// Validation metrics for each epoch
    pub val_metrics: Vec<HashMap<String, f64>>,
}

impl TrainingHistory {
    /// Create new training history
    pub fn new() -> Self {
        Self {
            loss: Vec::new(),
            metrics: Vec::new(),
            val_loss: Vec::new(),
            val_metrics: Vec::new(),
        }
    }

    /// Add epoch results
    pub fn add_epoch(
        &mut self,
        loss: f64,
        metrics: HashMap<String, f64>,
        val_loss: Option<f64>,
        val_metrics: HashMap<String, f64>,
    ) {
        self.loss.push(loss);
        self.metrics.push(metrics);

        if let Some(val_loss) = val_loss {
            self.val_loss.push(val_loss);
        }
        self.val_metrics.push(val_metrics);
    }
}

/// Model summary information
#[derive(Debug)]
pub struct ModelSummary {
    /// Layer information
    pub layers: Vec<LayerInfo>,
    /// Total number of parameters
    pub total_params: usize,
    /// Number of trainable parameters
    pub trainable_params: usize,
    /// Number of non-trainable parameters
    pub non_trainable_params: usize,
}

/// Layer information for summary
#[derive(Debug)]
pub struct LayerInfo {
    /// Layer name
    pub name: String,
    /// Layer type
    pub layer_type: String,
    /// Output shape
    pub output_shape: Vec<usize>,
    /// Parameter count
    pub param_count: usize,
}

/// Model input specification
pub struct Input {
    /// Input shape (excluding batch dimension)
    pub shape: Vec<usize>,
    /// Input name
    pub name: Option<String>,
    /// Data type
    pub dtype: DataType,
}

impl Input {
    /// Create new input specification
    pub fn new(shape: Vec<usize>) -> Self {
        Self {
            shape,
            name: None,
            dtype: DataType::Float64,
        }
    }

    /// Set input name
    pub fn name(mut self, name: impl Into<String>) -> Self {
        self.name = Some(name.into());
        self
    }

    /// Set data type
    pub fn dtype(mut self, dtype: DataType) -> Self {
        self.dtype = dtype;
        self
    }
}

/// Data types
#[derive(Debug, Clone)]
pub enum DataType {
    /// 32-bit float
    Float32,
    /// 64-bit float
    Float64,
    /// 32-bit integer
    Int32,
    /// 64-bit integer
    Int64,
}

/// Utility functions for building models
pub mod utils {
    use super::*;

    /// Create a simple sequential model for classification
    pub fn create_classification_model(
        input_dim: usize,
        num_classes: usize,
        hidden_layers: Vec<usize>,
    ) -> Sequential {
        let mut model = Sequential::new();

        // Add hidden layers
        for (i, &units) in hidden_layers.iter().enumerate() {
            model.add(Box::new(
                Dense::new(units)
                    .activation(ActivationFunction::ReLU)
                    .name(format!("dense_{}", i)),
            ));
        }

        // Add output layer
        let output_activation = if num_classes == 2 {
            ActivationFunction::Sigmoid
        } else {
            ActivationFunction::Softmax
        };

        model.add(Box::new(
            Dense::new(num_classes)
                .activation(output_activation)
                .name("output"),
        ));

        model
    }

    /// Create a quantum neural network model
    pub fn create_quantum_model(
        num_qubits: usize,
        num_classes: usize,
        num_layers: usize,
    ) -> Sequential {
        let mut model = Sequential::new();

        // Add quantum layer
        model.add(Box::new(
            QuantumDense::new(num_qubits, num_classes)
                .num_layers(num_layers)
                .ansatz_type(QuantumAnsatzType::HardwareEfficient)
                .name("quantum_layer"),
        ));

        // Add classical output processing if needed
        if num_classes > 1 {
            model.add(Box::new(
                Activation::new(ActivationFunction::Softmax).name("softmax"),
            ));
        }

        model
    }

    /// Create a hybrid quantum-classical model
    pub fn create_hybrid_model(
        input_dim: usize,
        num_qubits: usize,
        num_classes: usize,
        classical_hidden: Vec<usize>,
    ) -> Sequential {
        let mut model = Sequential::new();

        // Classical preprocessing
        for (i, &units) in classical_hidden.iter().enumerate() {
            model.add(Box::new(
                Dense::new(units)
                    .activation(ActivationFunction::ReLU)
                    .name(format!("classical_{}", i)),
            ));
        }

        // Quantum layer
        model.add(Box::new(
            QuantumDense::new(num_qubits, 64)
                .num_layers(2)
                .ansatz_type(QuantumAnsatzType::HardwareEfficient)
                .name("quantum_layer"),
        ));

        // Classical postprocessing
        model.add(Box::new(
            Dense::new(num_classes)
                .activation(if num_classes == 2 {
                    ActivationFunction::Sigmoid
                } else {
                    ActivationFunction::Softmax
                })
                .name("output"),
        ));

        model
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use ndarray::Array;

    #[test]
    fn test_dense_layer() {
        let mut dense = Dense::new(10)
            .activation(ActivationFunction::ReLU)
            .name("test_dense");

        assert!(!dense.built());

        dense.build(&[5]).unwrap();
        assert!(dense.built());
        assert_eq!(dense.compute_output_shape(&[32, 5]), vec![32, 10]);

        let input = ArrayD::from_shape_vec(
            vec![2, 5],
            vec![1.0, 2.0, 3.0, 4.0, 5.0, 0.5, 1.5, 2.5, 3.5, 4.5],
        )
        .unwrap();

        let output = dense.call(&input);
        assert!(output.is_ok());
        assert_eq!(output.unwrap().shape(), &[2, 10]);
    }

    #[test]
    fn test_activation_layer() {
        let mut activation = Activation::new(ActivationFunction::ReLU);
        activation.build(&[10]).unwrap();

        let input =
            ArrayD::from_shape_vec(vec![2, 3], vec![-1.0, 0.0, 1.0, -2.0, 0.5, 2.0]).unwrap();

        let output = activation.call(&input).unwrap();
        let expected =
            ArrayD::from_shape_vec(vec![2, 3], vec![0.0, 0.0, 1.0, 0.0, 0.5, 2.0]).unwrap();

        assert_eq!(output.shape(), expected.shape());
    }

    #[test]
    fn test_sequential_model() {
        let mut model = Sequential::new();

        model.add(Box::new(
            Dense::new(10).activation(ActivationFunction::ReLU),
        ));
        model.add(Box::new(
            Dense::new(1).activation(ActivationFunction::Sigmoid),
        ));

        model.build(vec![5]).unwrap();
        assert!(model.built);

        let summary = model.summary();
        assert_eq!(summary.layers.len(), 2);

        let input = ArrayD::from_shape_vec(
            vec![2, 5],
            vec![1.0, 2.0, 3.0, 4.0, 5.0, 0.5, 1.5, 2.5, 3.5, 4.5],
        )
        .unwrap();

        let output = model.predict(&input);
        assert!(output.is_ok());
        assert_eq!(output.unwrap().shape(), &[2, 1]);
    }

    #[test]
    fn test_loss_functions() {
        let predictions = ArrayD::from_shape_vec(vec![2, 1], vec![0.8, 0.3]).unwrap();
        let targets = ArrayD::from_shape_vec(vec![2, 1], vec![1.0, 0.0]).unwrap();

        let mse = LossFunction::MeanSquaredError;
        let loss = mse.compute(&predictions, &targets).unwrap();
        assert!(loss > 0.0);

        let bce = LossFunction::BinaryCrossentropy;
        let loss = bce.compute(&predictions, &targets).unwrap();
        assert!(loss > 0.0);
    }

    #[test]
    fn test_metrics() {
        let predictions = ArrayD::from_shape_vec(vec![4, 1], vec![0.8, 0.3, 0.9, 0.1]).unwrap();
        let targets = ArrayD::from_shape_vec(vec![4, 1], vec![1.0, 0.0, 1.0, 0.0]).unwrap();

        let accuracy = MetricType::Accuracy;
        let acc_value = accuracy.compute(&predictions, &targets).unwrap();
        assert!(acc_value >= 0.0 && acc_value <= 1.0);
    }

    #[test]
    #[ignore]
    fn test_model_utils() {
        let model = utils::create_classification_model(10, 3, vec![20, 15]);
        let summary = model.summary();
        assert_eq!(summary.layers.len(), 3); // 2 hidden + 1 output

        let quantum_model = utils::create_quantum_model(4, 2, 2);
        let summary = quantum_model.summary();
        assert!(summary.layers.len() >= 1);
    }
}
