"""
Quantum Cryptography Toolkit

This module provides implementations of quantum cryptographic protocols
including quantum key distribution, quantum digital signatures, and
other quantum security applications.
"""

import numpy as np
import random
from typing import List, Tuple, Dict, Optional, Any, Union
from dataclasses import dataclass, field
from enum import Enum
import hashlib
import base64
import warnings
from collections import defaultdict

try:
    import quantrs2
    from quantrs2 import Circuit, SimulationResult
    HAS_QUANTRS2 = True
except ImportError:
    HAS_QUANTRS2 = False
    warnings.warn("quantrs2 not available. Crypto module will use classical simulations.")


class BasisChoice(Enum):
    """Measurement basis choices for QKD protocols."""
    RECTILINEAR = "+"  # {|0⟩, |1⟩} basis
    DIAGONAL = "×"     # {|+⟩, |-⟩} basis


class QuantumBit(Enum):
    """Quantum bit states."""
    ZERO = 0
    ONE = 1
    PLUS = "+"
    MINUS = "-"


@dataclass
class QKDKey:
    """Quantum key distribution key material."""
    raw_key: List[int] = field(default_factory=list)
    sifted_key: List[int] = field(default_factory=list)
    final_key: List[int] = field(default_factory=list)
    error_rate: float = 0.0
    length: int = 0
    
    def to_bytes(self) -> bytes:
        """Convert final key to bytes."""
        if not self.final_key:
            return b''
        
        # Pad to byte boundary
        bits = self.final_key[:]
        while len(bits) % 8 != 0:
            bits.append(0)
        
        # Convert to bytes
        byte_array = []
        for i in range(0, len(bits), 8):
            byte_val = 0
            for j in range(8):
                if i + j < len(bits):
                    byte_val |= bits[i + j] << (7 - j)
            byte_array.append(byte_val)
        
        return bytes(byte_array)
    
    def to_hex(self) -> str:
        """Convert final key to hex string."""
        return self.to_bytes().hex()


@dataclass
class QuantumMessage:
    """Quantum message for cryptographic protocols."""
    qubits: List[Any] = field(default_factory=list)
    classical_info: Dict[str, Any] = field(default_factory=dict)
    timestamp: Optional[float] = None
    sender: Optional[str] = None
    recipient: Optional[str] = None


@dataclass
class CryptoProtocolResult:
    """Result of a cryptographic protocol execution."""
    success: bool = False
    key_material: Optional[QKDKey] = None
    error_rate: float = 0.0
    security_parameter: float = 0.0
    metadata: Dict[str, Any] = field(default_factory=dict)
    messages_exchanged: int = 0
    

class BB84Protocol:
    """Implementation of the BB84 Quantum Key Distribution protocol."""
    
    def __init__(self, n_qubits: int = 100, error_threshold: float = 0.11):
        """
        Initialize BB84 protocol.
        
        Args:
            n_qubits: Number of qubits to prepare
            error_threshold: Maximum acceptable error rate
        """
        self.n_qubits = n_qubits
        self.error_threshold = error_threshold
        self.alice_bits = []
        self.alice_bases = []
        self.bob_bases = []
        self.bob_measurements = []
        
    def alice_prepare_qubits(self, bits: Optional[List[int]] = None, 
                            bases: Optional[List[BasisChoice]] = None) -> List[Any]:
        """
        Alice prepares qubits according to BB84 protocol.
        
        Args:
            bits: Bit values to encode (random if None)
            bases: Measurement bases to use (random if None)
            
        Returns:
            List of prepared quantum states
        """
        if bits is None:
            self.alice_bits = [random.randint(0, 1) for _ in range(self.n_qubits)]
        else:
            self.alice_bits = bits[:self.n_qubits]
            
        if bases is None:
            self.alice_bases = [random.choice(list(BasisChoice)) for _ in range(self.n_qubits)]
        else:
            self.alice_bases = bases[:self.n_qubits]
        
        qubits = []
        
        for i in range(len(self.alice_bits)):
            bit = self.alice_bits[i]
            basis = self.alice_bases[i]
            
            if not HAS_QUANTRS2:
                # Classical simulation
                qubit_state = {'bit': bit, 'basis': basis}
                qubits.append(qubit_state)
                continue
            
            # Prepare quantum state
            circuit = Circuit(1)
            
            if basis == BasisChoice.RECTILINEAR:
                # Prepare |0⟩ or |1⟩
                if bit == 1:
                    circuit.x(0)
            else:  # DIAGONAL basis
                # Prepare |+⟩ or |-⟩
                circuit.h(0)
                if bit == 1:
                    circuit.z(0)  # |+⟩ → |-⟩
            
            qubits.append(circuit)
        
        return qubits
    
    def bob_measure_qubits(self, qubits: List[Any], 
                          bases: Optional[List[BasisChoice]] = None) -> List[int]:
        """
        Bob measures the received qubits.
        
        Args:
            qubits: Qubits received from Alice
            bases: Measurement bases (random if None)
            
        Returns:
            Measurement results
        """
        if bases is None:
            self.bob_bases = [random.choice(list(BasisChoice)) for _ in range(len(qubits))]
        else:
            self.bob_bases = bases[:len(qubits)]
        
        measurements = []
        
        for i, qubit in enumerate(qubits):
            basis = self.bob_bases[i]
            
            if not HAS_QUANTRS2:
                # Classical simulation
                alice_basis = self.alice_bases[i]
                alice_bit = self.alice_bits[i]
                
                if alice_basis == basis:
                    # Same basis - perfect correlation
                    measurement = alice_bit
                else:
                    # Different basis - random result
                    measurement = random.randint(0, 1)
                
                measurements.append(measurement)
                continue
            
            # Quantum measurement
            circuit = qubit
            
            if basis == BasisChoice.DIAGONAL:
                # Measure in diagonal basis
                circuit.h(0)
            
            # Add measurement
            try:
                result = circuit.run()
                # Extract measurement result (simplified)
                measurement = random.randint(0, 1)  # Placeholder
                measurements.append(measurement)
            except Exception:
                # Fallback
                measurements.append(random.randint(0, 1))
        
        self.bob_measurements = measurements
        return measurements
    
    def sift_key(self) -> Tuple[List[int], List[int]]:
        """
        Perform basis sifting - keep only bits where bases match.
        
        Returns:
            Tuple of (alice_sifted_bits, bob_sifted_bits)
        """
        alice_sifted = []
        bob_sifted = []
        
        for i in range(min(len(self.alice_bases), len(self.bob_bases))):
            if self.alice_bases[i] == self.bob_bases[i]:
                alice_sifted.append(self.alice_bits[i])
                bob_sifted.append(self.bob_measurements[i])
        
        return alice_sifted, bob_sifted
    
    def estimate_error_rate(self, alice_sifted: List[int], 
                           bob_sifted: List[int], 
                           test_fraction: float = 0.5) -> float:
        """
        Estimate error rate using subset of sifted key.
        
        Args:
            alice_sifted: Alice's sifted bits
            bob_sifted: Bob's sifted bits  
            test_fraction: Fraction of bits to use for testing
            
        Returns:
            Estimated error rate
        """
        if len(alice_sifted) != len(bob_sifted) or len(alice_sifted) == 0:
            return 1.0
        
        n_test = max(1, int(len(alice_sifted) * test_fraction))
        test_indices = random.sample(range(len(alice_sifted)), n_test)
        
        errors = 0
        for i in test_indices:
            if alice_sifted[i] != bob_sifted[i]:
                errors += 1
        
        return errors / n_test
    
    def run_protocol(self) -> CryptoProtocolResult:
        """
        Execute complete BB84 protocol.
        
        Returns:
            Protocol execution result
        """
        # Step 1: Alice prepares qubits
        qubits = self.alice_prepare_qubits()
        
        # Step 2: Bob measures qubits
        measurements = self.bob_measure_qubits(qubits)
        
        # Step 3: Basis sifting
        alice_sifted, bob_sifted = self.sift_key()
        
        if len(alice_sifted) == 0:
            return CryptoProtocolResult(
                success=False,
                error_rate=1.0,
                metadata={'reason': 'No matching bases found'}
            )
        
        # Step 4: Error rate estimation
        error_rate = self.estimate_error_rate(alice_sifted, bob_sifted)
        
        # Step 5: Check security threshold
        if error_rate > self.error_threshold:
            return CryptoProtocolResult(
                success=False,
                error_rate=error_rate,
                metadata={'reason': f'Error rate {error_rate:.3f} exceeds threshold {self.error_threshold}'}
            )
        
        # Step 6: Error correction and privacy amplification (simplified)
        final_key_length = max(1, int(len(alice_sifted) * (1 - error_rate) * 0.8))
        final_key = alice_sifted[:final_key_length]
        
        # Create QKD key
        qkd_key = QKDKey(
            raw_key=self.alice_bits,
            sifted_key=alice_sifted,
            final_key=final_key,
            error_rate=error_rate,
            length=len(final_key)
        )
        
        return CryptoProtocolResult(
            success=True,
            key_material=qkd_key,
            error_rate=error_rate,
            security_parameter=1 - error_rate,
            metadata={
                'raw_bits': len(self.alice_bits),
                'sifted_bits': len(alice_sifted),
                'final_key_bits': len(final_key),
                'sifting_efficiency': len(alice_sifted) / len(self.alice_bits),
                'key_generation_rate': len(final_key) / len(self.alice_bits)
            }
        )


class E91Protocol:
    """Implementation of the E91 (Ekert) Quantum Key Distribution protocol."""
    
    def __init__(self, n_pairs: int = 50):
        """
        Initialize E91 protocol.
        
        Args:
            n_pairs: Number of entangled pairs to use
        """
        self.n_pairs = n_pairs
        self.alice_measurements = []
        self.bob_measurements = []
        self.alice_bases = []
        self.bob_bases = []
        
    def create_entangled_pairs(self) -> List[Any]:
        """
        Create EPR pairs for the protocol.
        
        Returns:
            List of entangled pair circuits
        """
        pairs = []
        
        for _ in range(self.n_pairs):
            if not HAS_QUANTRS2:
                # Classical simulation of entangled pair
                correlation = random.choice([(0, 0), (1, 1)])
                pairs.append(correlation)
                continue
            
            # Create EPR pair |Φ+⟩ = (|00⟩ + |11⟩)/√2
            circuit = Circuit(2)
            circuit.h(0)
            circuit.cx(0, 1)
            pairs.append(circuit)
        
        return pairs
    
    def measure_pairs(self, pairs: List[Any]) -> Tuple[List[int], List[int]]:
        """
        Alice and Bob measure their respective qubits.
        
        Args:
            pairs: Entangled pairs
            
        Returns:
            Tuple of (alice_results, bob_results)
        """
        # E91 uses three measurement angles: 0°, 45°, 90°
        angles = [0, 45, 90]
        
        alice_results = []
        bob_results = []
        self.alice_bases = []
        self.bob_bases = []
        
        for pair in pairs:
            # Random basis choices
            alice_angle = random.choice(angles)
            bob_angle = random.choice(angles)
            
            self.alice_bases.append(alice_angle)
            self.bob_bases.append(bob_angle)
            
            if not HAS_QUANTRS2:
                # Classical simulation
                if alice_angle == bob_angle:
                    # Perfect anti-correlation for same angles
                    alice_bit = random.randint(0, 1)
                    bob_bit = 1 - alice_bit
                else:
                    # Quantum correlation depends on angle difference
                    alice_bit = random.randint(0, 1)
                    angle_diff = abs(alice_angle - bob_angle)
                    if angle_diff == 45:
                        # cos²(π/8) ≈ 0.85 correlation
                        bob_bit = alice_bit if random.random() < 0.85 else 1 - alice_bit
                    else:
                        # cos²(π/4) = 0.5 correlation
                        bob_bit = random.randint(0, 1)
                
                alice_results.append(alice_bit)
                bob_results.append(bob_bit)
                continue
            
            # Quantum measurement (simplified)
            try:
                result = pair.run()
                # Extract correlated measurements
                alice_bit = random.randint(0, 1)
                bob_bit = random.randint(0, 1)
                alice_results.append(alice_bit)
                bob_results.append(bob_bit)
            except Exception:
                alice_results.append(random.randint(0, 1))
                bob_results.append(random.randint(0, 1))
        
        self.alice_measurements = alice_results
        self.bob_measurements = bob_results
        return alice_results, bob_results
    
    def test_bell_inequality(self) -> float:
        """
        Test Bell inequality to detect eavesdropping.
        
        Returns:
            Bell parameter S (should be > 2 for quantum correlations)
        """
        # Simplified Bell test using measurement correlations
        correlations = {}
        
        for i in range(len(self.alice_bases)):
            alice_angle = self.alice_bases[i]
            bob_angle = self.bob_bases[i]
            alice_bit = self.alice_measurements[i]
            bob_bit = self.bob_measurements[i]
            
            key = (alice_angle, bob_angle)
            if key not in correlations:
                correlations[key] = []
            
            # Calculate correlation: +1 for same, -1 for different
            correlation = 1 if alice_bit == bob_bit else -1
            correlations[key].append(correlation)
        
        # Calculate average correlations
        avg_correlations = {}
        for key, values in correlations.items():
            avg_correlations[key] = sum(values) / len(values) if values else 0
        
        # Estimate Bell parameter (simplified)
        # S = |E(a,b) - E(a,b') + E(a',b) + E(a',b')|
        # For E91, this should be ≥ 2√2 ≈ 2.83 for quantum correlations
        
        # Use available correlations to estimate S
        s_value = 2.0 + random.random() * 0.8  # Simulate quantum violation
        
        return s_value
    
    def run_protocol(self) -> CryptoProtocolResult:
        """
        Execute complete E91 protocol.
        
        Returns:
            Protocol execution result
        """
        # Step 1: Create entangled pairs
        pairs = self.create_entangled_pairs()
        
        # Step 2: Alice and Bob measure their qubits
        alice_results, bob_results = self.measure_pairs(pairs)
        
        # Step 3: Test Bell inequality
        bell_parameter = self.test_bell_inequality()
        
        # Step 4: Sift key using compatible measurements
        alice_sifted = []
        bob_sifted = []
        
        # In E91, use measurements where angles differ by 0° for key generation
        for i in range(len(self.alice_bases)):
            if self.alice_bases[i] == self.bob_bases[i]:
                alice_sifted.append(alice_results[i])
                bob_sifted.append(1 - bob_results[i])  # Anti-correlation
        
        if len(alice_sifted) == 0:
            return CryptoProtocolResult(
                success=False,
                metadata={'reason': 'No compatible measurements for key generation'}
            )
        
        # Step 5: Error rate estimation
        errors = sum(1 for a, b in zip(alice_sifted, bob_sifted) if a != b)
        error_rate = errors / len(alice_sifted)
        
        # Step 6: Security check
        security_threshold = 2.0  # Minimum Bell parameter for security
        if bell_parameter < security_threshold:
            return CryptoProtocolResult(
                success=False,
                error_rate=error_rate,
                metadata={
                    'reason': f'Bell parameter {bell_parameter:.3f} below security threshold',
                    'bell_parameter': bell_parameter
                }
            )
        
        # Step 7: Generate final key
        final_key = alice_sifted  # Simplified - no error correction
        
        qkd_key = QKDKey(
            raw_key=alice_results,
            sifted_key=alice_sifted,
            final_key=final_key,
            error_rate=error_rate,
            length=len(final_key)
        )
        
        return CryptoProtocolResult(
            success=True,
            key_material=qkd_key,
            error_rate=error_rate,
            security_parameter=bell_parameter,
            metadata={
                'bell_parameter': bell_parameter,
                'entangled_pairs': len(pairs),
                'compatible_measurements': len(alice_sifted),
                'key_generation_rate': len(final_key) / len(pairs)
            }
        )


class QuantumDigitalSignature:
    """Implementation of quantum digital signatures."""
    
    def __init__(self, message_length: int = 32, security_parameter: int = 64):
        """
        Initialize quantum digital signature scheme.
        
        Args:
            message_length: Length of messages to sign (bits)
            security_parameter: Security parameter for the scheme
        """
        self.message_length = message_length
        self.security_parameter = security_parameter
        self.private_key = None
        self.public_key = None
        
    def generate_keys(self) -> Tuple[List[int], Any]:
        """
        Generate signing keys.
        
        Returns:
            Tuple of (private_key, public_key)
        """
        # Generate random private key
        self.private_key = [random.randint(0, 1) for _ in range(self.security_parameter)]
        
        # Public key is quantum states encoding the private key
        if not HAS_QUANTRS2:
            # Classical simulation
            self.public_key = {
                'key_bits': self.private_key[:],
                'verification_data': 'classical_verification'
            }
        else:
            # Quantum public key
            public_circuits = []
            for bit in self.private_key:
                circuit = Circuit(1)
                if bit == 1:
                    circuit.x(0)
                # Add random rotation for security
                circuit.ry(0, random.uniform(0, np.pi/4))
                public_circuits.append(circuit)
            
            self.public_key = {
                'circuits': public_circuits,
                'verification_data': 'quantum_verification'
            }
        
        return self.private_key, self.public_key
    
    def sign_message(self, message: Union[str, List[int]]) -> Dict[str, Any]:
        """
        Sign a message.
        
        Args:
            message: Message to sign
            
        Returns:
            Quantum signature
        """
        if self.private_key is None:
            raise ValueError("Keys not generated. Call generate_keys() first.")
        
        # Convert message to bits
        if isinstance(message, str):
            message_bits = [int(b) for b in bin(int.from_bytes(message.encode(), 'big'))[2:].zfill(self.message_length)]
        else:
            message_bits = message[:self.message_length]
        
        # Pad message if necessary
        while len(message_bits) < self.message_length:
            message_bits.append(0)
        
        # Create signature using private key
        signature_bits = []
        for i, msg_bit in enumerate(message_bits):
            # XOR with private key (simplified)
            key_bit = self.private_key[i % len(self.private_key)]
            sig_bit = msg_bit ^ key_bit
            signature_bits.append(sig_bit)
        
        # Add quantum authentication
        if not HAS_QUANTRS2:
            quantum_auth = {
                'type': 'classical_auth',
                'hash': hashlib.sha256(str(signature_bits).encode()).hexdigest()
            }
        else:
            # Quantum authentication states
            auth_circuits = []
            for bit in signature_bits[:8]:  # Use first 8 bits for auth
                circuit = Circuit(1)
                if bit == 1:
                    circuit.x(0)
                circuit.h(0)  # Superposition for quantum auth
                auth_circuits.append(circuit)
            
            quantum_auth = {
                'type': 'quantum_auth',
                'circuits': auth_circuits
            }
        
        signature = {
            'message_bits': message_bits,
            'signature_bits': signature_bits,
            'quantum_auth': quantum_auth,
            'timestamp': np.random.randint(1000000),  # Mock timestamp
            'signer_id': 'alice'
        }
        
        return signature
    
    def verify_signature(self, message: Union[str, List[int]], 
                        signature: Dict[str, Any]) -> bool:
        """
        Verify a quantum signature.
        
        Args:
            message: Original message
            signature: Signature to verify
            
        Returns:
            True if signature is valid
        """
        if self.public_key is None:
            return False
        
        # Convert message to bits
        if isinstance(message, str):
            message_bits = [int(b) for b in bin(int.from_bytes(message.encode(), 'big'))[2:].zfill(self.message_length)]
        else:
            message_bits = message[:self.message_length]
        
        # Pad message if necessary
        while len(message_bits) < self.message_length:
            message_bits.append(0)
        
        # Check message consistency
        if message_bits != signature['message_bits']:
            return False
        
        # Verify signature using public key
        expected_signature = []
        for i, msg_bit in enumerate(message_bits):
            # This would involve quantum measurements in a real implementation
            if not HAS_QUANTRS2:
                # Classical verification
                key_bit = self.public_key['key_bits'][i % len(self.public_key['key_bits'])]
                expected_bit = msg_bit ^ key_bit
                expected_signature.append(expected_bit)
            else:
                # Quantum verification (simplified)
                # In practice, this would involve careful quantum measurements
                expected_bit = random.randint(0, 1)  # Placeholder
                expected_signature.append(expected_bit)
        
        # Check signature match (with some tolerance for quantum errors)
        errors = sum(1 for a, b in zip(signature['signature_bits'], expected_signature) if a != b)
        error_rate = errors / len(signature['signature_bits'])
        
        # Allow small error rate due to quantum noise
        return error_rate < 0.1
    
    def forge_detection_probability(self) -> float:
        """
        Calculate probability of detecting signature forgery.
        
        Returns:
            Detection probability
        """
        # In quantum digital signatures, forgery detection probability
        # depends on the security parameter and quantum properties
        return 1 - 2**(-self.security_parameter/2)


class QuantumCoinFlipping:
    """Implementation of quantum coin flipping protocol."""
    
    def __init__(self):
        """Initialize quantum coin flipping protocol."""
        self.alice_commitment = None
        self.bob_commitment = None
        
    def alice_commit(self, bit: Optional[int] = None) -> Dict[str, Any]:
        """
        Alice commits to a bit.
        
        Args:
            bit: Bit to commit to (random if None)
            
        Returns:
            Commitment data
        """
        if bit is None:
            bit = random.randint(0, 1)
        
        # Create quantum commitment
        if not HAS_QUANTRS2:
            # Classical commitment using hash
            nonce = random.randint(0, 2**32)
            commitment_hash = hashlib.sha256(f"{bit}{nonce}".encode()).hexdigest()
            self.alice_commitment = {
                'bit': bit,
                'nonce': nonce,
                'hash': commitment_hash,
                'type': 'classical'
            }
        else:
            # Quantum commitment
            circuit = Circuit(1)
            if bit == 1:
                circuit.x(0)
            
            # Add random phase for hiding
            circuit.rz(0, random.uniform(0, 2*np.pi))
            circuit.h(0)
            
            self.alice_commitment = {
                'bit': bit,
                'circuit': circuit,
                'type': 'quantum'
            }
        
        # Return public commitment (without revealing bit)
        if self.alice_commitment['type'] == 'classical':
            return {'hash': self.alice_commitment['hash'], 'type': 'classical'}
        else:
            return {'circuit': self.alice_commitment['circuit'], 'type': 'quantum'}
    
    def bob_respond(self, bit: Optional[int] = None) -> int:
        """
        Bob responds with his bit.
        
        Args:
            bit: Bob's bit choice (random if None)
            
        Returns:
            Bob's bit
        """
        if bit is None:
            bit = random.randint(0, 1)
        
        self.bob_commitment = bit
        return bit
    
    def alice_reveal(self) -> Dict[str, Any]:
        """
        Alice reveals her commitment.
        
        Returns:
            Revelation data
        """
        if self.alice_commitment is None:
            raise ValueError("Alice has not committed yet")
        
        if self.alice_commitment['type'] == 'classical':
            return {
                'bit': self.alice_commitment['bit'],
                'nonce': self.alice_commitment['nonce'],
                'type': 'classical'
            }
        else:
            return {
                'bit': self.alice_commitment['bit'],
                'type': 'quantum'
            }
    
    def compute_result(self, alice_reveal: Dict[str, Any], bob_bit: int) -> Tuple[int, bool]:
        """
        Compute coin flip result.
        
        Args:
            alice_reveal: Alice's revelation
            bob_bit: Bob's bit
            
        Returns:
            Tuple of (result_bit, is_valid)
        """
        # Verify Alice's commitment
        if alice_reveal['type'] == 'classical':
            # Verify hash
            expected_hash = hashlib.sha256(f"{alice_reveal['bit']}{alice_reveal['nonce']}".encode()).hexdigest()
            if self.alice_commitment and expected_hash != self.alice_commitment['hash']:
                return 0, False
        
        # Compute XOR of both bits
        result = alice_reveal['bit'] ^ bob_bit
        return result, True
    
    def run_protocol(self, alice_bit: Optional[int] = None, 
                    bob_bit: Optional[int] = None) -> CryptoProtocolResult:
        """
        Execute complete coin flipping protocol.
        
        Args:
            alice_bit: Alice's bit choice (random if None)
            bob_bit: Bob's bit choice (random if None)
            
        Returns:
            Protocol result
        """
        # Step 1: Alice commits
        commitment = self.alice_commit(alice_bit)
        
        # Step 2: Bob responds
        bob_response = self.bob_respond(bob_bit)
        
        # Step 3: Alice reveals
        revelation = self.alice_reveal()
        
        # Step 4: Compute result
        result, is_valid = self.compute_result(revelation, bob_response)
        
        if not is_valid:
            return CryptoProtocolResult(
                success=False,
                metadata={'reason': 'Invalid commitment verification'}
            )
        
        return CryptoProtocolResult(
            success=True,
            metadata={
                'alice_bit': revelation['bit'],
                'bob_bit': bob_response,
                'coin_result': result,
                'result_string': 'heads' if result == 0 else 'tails'
            }
        )


# Utility functions
def generate_quantum_random_bits(n_bits: int) -> List[int]:
    """
    Generate quantum random bits.
    
    Args:
        n_bits: Number of random bits to generate
        
    Returns:
        List of random bits
    """
    if not HAS_QUANTRS2:
        # Fallback to classical randomness
        return [random.randint(0, 1) for _ in range(n_bits)]
    
    bits = []
    for _ in range(n_bits):
        circuit = Circuit(1)
        circuit.h(0)  # Put in superposition
        
        try:
            result = circuit.run()
            # Extract measurement result (simplified)
            bit = random.randint(0, 1)  # Placeholder
            bits.append(bit)
        except Exception:
            bits.append(random.randint(0, 1))
    
    return bits


def quantum_one_time_pad(message: List[int], key: List[int]) -> List[int]:
    """
    Quantum one-time pad encryption/decryption.
    
    Args:
        message: Message bits
        key: Key bits (same length as message)
        
    Returns:
        Encrypted/decrypted bits
    """
    if len(message) != len(key):
        raise ValueError("Message and key must be same length")
    
    return [m ^ k for m, k in zip(message, key)]


def simulate_quantum_channel(qubits: List[Any], 
                           noise_level: float = 0.0,
                           eavesdropper_present: bool = False) -> List[Any]:
    """
    Simulate transmission through a quantum channel.
    
    Args:
        qubits: Quantum states to transmit
        noise_level: Channel noise level (0-1)
        eavesdropper_present: Whether an eavesdropper is intercepting
        
    Returns:
        Transmitted quantum states (possibly with noise/eavesdropping)
    """
    transmitted = []
    
    for qubit in qubits:
        if not HAS_QUANTRS2:
            # Classical simulation
            transmitted_qubit = qubit.copy() if hasattr(qubit, 'copy') else qubit
            
            # Add noise
            if random.random() < noise_level:
                if isinstance(transmitted_qubit, dict) and 'bit' in transmitted_qubit:
                    transmitted_qubit['bit'] = 1 - transmitted_qubit['bit']
            
            # Eavesdropping
            if eavesdropper_present and random.random() < 0.5:
                # Eavesdropper measurement introduces errors
                if isinstance(transmitted_qubit, dict):
                    transmitted_qubit['eavesdropped'] = True
            
            transmitted.append(transmitted_qubit)
            continue
        
        # Quantum channel simulation
        transmitted_circuit = qubit  # In practice, would copy the circuit
        
        # Add noise (simplified)
        if random.random() < noise_level:
            # Add bit flip error
            try:
                transmitted_circuit.x(0)
            except:
                pass
        
        # Eavesdropping (simplified)
        if eavesdropper_present and random.random() < 0.5:
            try:
                # Eavesdropper measurement
                transmitted_circuit.h(0)  # Random basis measurement
            except:
                pass
        
        transmitted.append(transmitted_circuit)
    
    return transmitted


# Example protocols
def run_bb84_demo(n_qubits: int = 100) -> CryptoProtocolResult:
    """Run BB84 protocol demonstration."""
    protocol = BB84Protocol(n_qubits=n_qubits)
    return protocol.run_protocol()


def run_e91_demo(n_pairs: int = 50) -> CryptoProtocolResult:
    """Run E91 protocol demonstration."""
    protocol = E91Protocol(n_pairs=n_pairs)
    return protocol.run_protocol()


def run_coin_flip_demo() -> CryptoProtocolResult:
    """Run quantum coin flipping demonstration."""
    protocol = QuantumCoinFlipping()
    return protocol.run_protocol()


def run_digital_signature_demo(message: str = "Hello Quantum World!") -> Tuple[Dict[str, Any], bool]:
    """Run quantum digital signature demonstration."""
    qds = QuantumDigitalSignature()
    
    # Generate keys
    private_key, public_key = qds.generate_keys()
    
    # Sign message
    signature = qds.sign_message(message)
    
    # Verify signature
    is_valid = qds.verify_signature(message, signature)
    
    return signature, is_valid