"""
PennyLane Plugin for QuantRS2

This module provides seamless integration between QuantRS2 and PennyLane,
enabling hybrid quantum-classical machine learning workflows.
"""

import warnings
from typing import Dict, List, Optional, Any, Union, Callable, Tuple
import numpy as np

try:
    import pennylane as qml
    from pennylane import Device
    from pennylane.operation import Operation
    from pennylane.tape import QuantumTape
    PENNYLANE_AVAILABLE = True
except ImportError:
    PENNYLANE_AVAILABLE = False
    warnings.warn("PennyLane not available. Install with: pip install pennylane")
    
    # Mock classes for when PennyLane is not available
    class Device:
        pass
    class Operation:
        pass
    class QuantumTape:
        pass

try:
    from quantrs2 import Circuit as QuantRS2Circuit
    QUANTRS2_AVAILABLE = True
except ImportError:
    QUANTRS2_AVAILABLE = False
    warnings.warn("QuantRS2 core not available. Using mock implementation.")
    
    class QuantRS2Circuit:
        def __init__(self, n_qubits):
            self.n_qubits = n_qubits
            self.gates = []
        def h(self, qubit): self.gates.append(('h', qubit))
        def x(self, qubit): self.gates.append(('x', qubit))
        def cnot(self, control, target): self.gates.append(('cnot', control, target))
        def run(self): 
            return type('MockResult', (), {
                'state_vector': np.array([0.7071, 0, 0, 0.7071]),
                'measurements': [0, 1]
            })()


class QuantRS2PennyLaneError(Exception):
    """Exception raised for QuantRS2-PennyLane integration issues."""
    pass


class QuantRS2Device(Device):
    """PennyLane device that uses QuantRS2 as the backend."""
    
    name = "quantrs2.simulator"
    short_name = "quantrs2"
    pennylane_requires = ">=0.28.0"
    version = "0.1.0"
    author = "QuantRS2 Team"
    
    operations = {
        "PauliX", "PauliY", "PauliZ", "Hadamard", "S", "T", "CNOT", "CZ", "SWAP",
        "RX", "RY", "RZ", "Rot", "PhaseShift", "ControlledPhaseShift",
        "MultiControlledX", "Toffoli", "CSWAP"
    }
    
    observables = {
        "PauliX", "PauliY", "PauliZ", "Identity", "Hadamard", "Hermitian"
    }
    
    def __init__(self, wires, *, shots=None, **kwargs):
        """Initialize the QuantRS2 device.
        
        Args:
            wires: Number of wires/qubits or wire labels
            shots: Number of measurement shots (None for exact simulation)
            **kwargs: Additional device options
        """
        if not QUANTRS2_AVAILABLE:
            raise QuantRS2PennyLaneError("QuantRS2 not available")
        if not PENNYLANE_AVAILABLE:
            raise QuantRS2PennyLaneError("PennyLane not available")
            
        super().__init__(wires, shots=shots, **kwargs)
        
        self.n_qubits = len(self.wires)
        self.shots = shots
        self._circuit = None
        self._state = None
        
        # Device capabilities
        self.capabilities().update({
            "provides_state": True,
            "provides_jacobian": True,
            "tensor_observables": True,
            "inverse_operations": True,
            "supports_finite_shots": True,
        })
    
    def reset(self):
        """Reset the device state."""
        self._circuit = None
        self._state = None
    
    def apply(self, operations, rotations=None, **kwargs):
        """Apply quantum operations to the device.
        
        Args:
            operations: List of operations to apply
            rotations: Basis rotations (not used in state vector simulation)
            **kwargs: Additional arguments
        """
        # Create new QuantRS2 circuit
        self._circuit = QuantRS2Circuit(self.n_qubits)
        
        # Convert PennyLane operations to QuantRS2 gates
        for op in operations:
            self._apply_operation(op)
    
    def _apply_operation(self, operation):
        """Convert and apply a single PennyLane operation to QuantRS2 circuit."""
        name = operation.name
        wires = [self.wire_map[w] for w in operation.wires]
        params = operation.parameters
        
        # Single-qubit gates
        if name == "PauliX":
            self._circuit.x(wires[0])
        elif name == "PauliY":
            self._circuit.y(wires[0])
        elif name == "PauliZ":
            self._circuit.z(wires[0])
        elif name == "Hadamard":
            self._circuit.h(wires[0])
        elif name == "S":
            self._circuit.s(wires[0])
        elif name == "T":
            self._circuit.t(wires[0])
        
        # Parametric single-qubit gates
        elif name == "RX":
            if hasattr(self._circuit, 'rx'):
                self._circuit.rx(wires[0], params[0])
            else:
                # Fallback: decompose RX into RZ and Hadamard
                self._circuit.h(wires[0])
                self._circuit.rz(wires[0], params[0])
                self._circuit.h(wires[0])
        elif name == "RY":
            if hasattr(self._circuit, 'ry'):
                self._circuit.ry(wires[0], params[0])
            else:
                # Fallback: approximate with RZ gates
                self._circuit.rz(wires[0], np.pi/2)
                self._circuit.rx(wires[0], params[0])
                self._circuit.rz(wires[0], -np.pi/2)
        elif name == "RZ":
            if hasattr(self._circuit, 'rz'):
                self._circuit.rz(wires[0], params[0])
            else:
                # Fallback: phase gate approximation
                self._circuit.z(wires[0])
        elif name == "PhaseShift":
            if hasattr(self._circuit, 'phase'):
                self._circuit.phase(wires[0], params[0])
            else:
                # Fallback: use RZ
                if hasattr(self._circuit, 'rz'):
                    self._circuit.rz(wires[0], params[0])
        
        # Two-qubit gates
        elif name == "CNOT":
            self._circuit.cnot(wires[0], wires[1])
        elif name == "CZ":
            if hasattr(self._circuit, 'cz'):
                self._circuit.cz(wires[0], wires[1])
            else:
                # Fallback: CNOT + Hadamard decomposition
                self._circuit.h(wires[1])
                self._circuit.cnot(wires[0], wires[1])
                self._circuit.h(wires[1])
        elif name == "SWAP":
            if hasattr(self._circuit, 'swap'):
                self._circuit.swap(wires[0], wires[1])
            else:
                # Fallback: three CNOT decomposition
                self._circuit.cnot(wires[0], wires[1])
                self._circuit.cnot(wires[1], wires[0])
                self._circuit.cnot(wires[0], wires[1])
        elif name == "ControlledPhaseShift":
            if hasattr(self._circuit, 'controlled_phase'):
                self._circuit.controlled_phase(wires[0], wires[1], params[0])
            else:
                # Fallback: approximate with CZ
                if hasattr(self._circuit, 'cz'):
                    self._circuit.cz(wires[0], wires[1])
        
        # Multi-qubit gates
        elif name == "Toffoli":
            if hasattr(self._circuit, 'ccx'):
                self._circuit.ccx(wires[0], wires[1], wires[2])
            else:
                # Fallback: approximate with multiple CNOTs
                self._circuit.cnot(wires[1], wires[2])
                self._circuit.cnot(wires[0], wires[2])
                self._circuit.cnot(wires[1], wires[2])
        
        # Complex gates
        elif name == "Rot":
            # Rot(phi, theta, omega) = RZ(omega) RY(theta) RZ(phi)
            if len(params) >= 3:
                if hasattr(self._circuit, 'rz') and hasattr(self._circuit, 'ry'):
                    self._circuit.rz(wires[0], params[0])  # phi
                    self._circuit.ry(wires[0], params[1])  # theta
                    self._circuit.rz(wires[0], params[2])  # omega
                else:
                    # Fallback: approximate with basic gates
                    self._circuit.z(wires[0])
                    self._circuit.y(wires[0])
                    self._circuit.z(wires[0])
        
        else:
            warnings.warn(f"Operation {name} not supported, skipping")
    
    def generate_samples(self):
        """Generate measurement samples from the circuit."""
        if self._circuit is None:
            raise QuantRS2PennyLaneError("No circuit has been applied")
        
        # Run the circuit to get the state
        result = self._circuit.run()
        self._state = result.state_vector
        
        if self.shots is None:
            # Return computational basis probabilities
            probabilities = np.abs(self._state)**2
            n_outcomes = len(probabilities)
            n_wires = int(np.log2(n_outcomes))
            
            # Create all possible measurement outcomes
            outcomes = []
            for i in range(n_outcomes):
                if probabilities[i] > 1e-10:  # Only include non-zero probabilities
                    binary = format(i, f'0{n_wires}b')
                    outcomes.append([int(b) for b in binary])
            
            return np.array(outcomes)
        else:
            # Sample from the probability distribution
            probabilities = np.abs(self._state)**2
            n_outcomes = len(probabilities)
            n_wires = int(np.log2(n_outcomes))
            
            # Generate random samples
            samples = []
            for _ in range(self.shots):
                outcome_idx = np.random.choice(n_outcomes, p=probabilities)
                binary = format(outcome_idx, f'0{n_wires}b')
                samples.append([int(b) for b in binary])
            
            return np.array(samples)
    
    def probability(self, wires=None):
        """Return the probability of measuring each computational basis state."""
        if self._state is None:
            raise QuantRS2PennyLaneError("No circuit has been executed")
        
        if wires is None:
            wires = self.wires
        
        # Calculate probabilities for specified wires
        probabilities = np.abs(self._state)**2
        
        if len(wires) == len(self.wires):
            return probabilities
        else:
            # Marginalize over unwanted wires
            n_total = len(self.wires)
            n_wires = len(wires)
            wire_indices = [self.wire_map[w] for w in wires]
            
            marginal_probs = np.zeros(2**n_wires)
            
            for i, prob in enumerate(probabilities):
                # Extract bits for the specified wires
                full_binary = format(i, f'0{n_total}b')
                selected_bits = ''.join([full_binary[idx] for idx in wire_indices])
                marginal_idx = int(selected_bits, 2)
                marginal_probs[marginal_idx] += prob
            
            return marginal_probs
    
    def state(self):
        """Return the quantum state vector."""
        if self._state is None:
            raise QuantRS2PennyLaneError("No circuit has been executed")
        return self._state
    
    def expval(self, observable, wires, par):
        """Calculate expectation value of an observable."""
        if self._state is None:
            raise QuantRS2PennyLaneError("No circuit has been executed")
        
        # Get the observable matrix
        obs_matrix = self._get_observable_matrix(observable, wires)
        
        # Calculate expectation value: <ψ|O|ψ>
        expectation = np.conj(self._state) @ obs_matrix @ self._state
        return np.real(expectation)
    
    def _get_observable_matrix(self, observable, wires):
        """Get the matrix representation of an observable."""
        name = observable.__class__.__name__
        
        # Single-qubit Pauli operators
        if name == "PauliX":
            single_op = np.array([[0, 1], [1, 0]], dtype=complex)
        elif name == "PauliY":
            single_op = np.array([[0, -1j], [1j, 0]], dtype=complex)
        elif name == "PauliZ":
            single_op = np.array([[1, 0], [0, -1]], dtype=complex)
        elif name == "Identity":
            single_op = np.array([[1, 0], [0, 1]], dtype=complex)
        elif name == "Hadamard":
            single_op = np.array([[1, 1], [1, -1]], dtype=complex) / np.sqrt(2)
        elif name == "Hermitian":
            # Custom Hermitian observable
            single_op = observable.matrix()
        else:
            raise QuantRS2PennyLaneError(f"Observable {name} not supported")
        
        # Expand to full system size
        if len(wires) == 1:
            wire_idx = self.wire_map[wires[0]]
            full_op = np.eye(1, dtype=complex)
            
            for i in range(self.n_qubits):
                if i == wire_idx:
                    full_op = np.kron(full_op, single_op)
                else:
                    full_op = np.kron(full_op, np.eye(2))
            
            return full_op
        else:
            # Multi-qubit observable - tensor product
            full_op = single_op
            for i in range(1, len(wires)):
                full_op = np.kron(full_op, single_op)
            
            # Embed in full system (simplified for same observable on all wires)
            return self._embed_operator(full_op, wires)
    
    def _embed_operator(self, operator, target_wires):
        """Embed an operator acting on target wires into the full system."""
        n_target = len(target_wires)
        n_op = operator.shape[0]
        
        if n_op != 2**n_target:
            raise QuantRS2PennyLaneError("Operator size mismatch")
        
        # For simplicity, assume target wires are the first n_target wires
        # In a full implementation, this would handle arbitrary wire ordering
        identity_size = 2**(self.n_qubits - n_target)
        identity = np.eye(identity_size)
        
        return np.kron(operator, identity)


class QuantRS2QMLModel:
    """Quantum Machine Learning model using QuantRS2 backend."""
    
    def __init__(self, n_qubits: int, n_layers: int = 1, shots: Optional[int] = None):
        """Initialize QML model.
        
        Args:
            n_qubits: Number of qubits in the quantum circuit
            n_layers: Number of variational layers
            shots: Number of measurement shots (None for exact simulation)
        """
        if not PENNYLANE_AVAILABLE:
            raise QuantRS2PennyLaneError("PennyLane not available")
        
        self.n_qubits = n_qubits
        self.n_layers = n_layers
        self.shots = shots
        
        # Create QuantRS2 device
        self.device = QuantRS2Device(wires=n_qubits, shots=shots)
        
        # Initialize parameters
        self.params = None
        self.qnode = None
        
        self._build_circuit()
    
    def _build_circuit(self):
        """Build the variational quantum circuit."""
        @qml.qnode(self.device, diff_method="parameter-shift")
        def circuit(params, x=None):
            # Data encoding (if input data provided)
            if x is not None:
                for i in range(min(len(x), self.n_qubits)):
                    qml.RY(x[i], wires=i)
            
            # Variational layers
            for layer in range(self.n_layers):
                # Rotation gates with parameters
                for qubit in range(self.n_qubits):
                    param_idx = layer * self.n_qubits * 3 + qubit * 3
                    qml.RX(params[param_idx], wires=qubit)
                    qml.RY(params[param_idx + 1], wires=qubit)
                    qml.RZ(params[param_idx + 2], wires=qubit)
                
                # Entangling gates
                for qubit in range(self.n_qubits - 1):
                    qml.CNOT(wires=[qubit, qubit + 1])
                
                # Ring connectivity
                if self.n_qubits > 2:
                    qml.CNOT(wires=[self.n_qubits - 1, 0])
            
            # Measurement
            return [qml.expval(qml.PauliZ(wires=i)) for i in range(self.n_qubits)]
        
        self.qnode = circuit
    
    def initialize_params(self, seed: Optional[int] = None):
        """Initialize variational parameters.
        
        Args:
            seed: Random seed for reproducibility
        """
        if seed is not None:
            np.random.seed(seed)
        
        # Each layer has 3 parameters per qubit (RX, RY, RZ)
        n_params = self.n_layers * self.n_qubits * 3
        self.params = np.random.uniform(0, 2*np.pi, n_params)
        return self.params
    
    def forward(self, x, params=None):
        """Forward pass through the quantum circuit.
        
        Args:
            x: Input data
            params: Variational parameters (uses self.params if None)
        
        Returns:
            Quantum circuit output
        """
        if params is None:
            params = self.params
        
        if params is None:
            raise QuantRS2PennyLaneError("Parameters not initialized")
        
        return self.qnode(params, x=x)
    
    def predict(self, X):
        """Make predictions on input data.
        
        Args:
            X: Input data array
        
        Returns:
            Predictions array
        """
        if self.params is None:
            raise QuantRS2PennyLaneError("Model not trained")
        
        predictions = []
        for x in X:
            output = self.forward(x)
            # Simple prediction: sign of the sum of expectation values
            prediction = 1 if np.sum(output) > 0 else -1
            predictions.append(prediction)
        
        return np.array(predictions)
    
    def train(self, X, y, n_epochs: int = 100, learning_rate: float = 0.1):
        """Train the quantum model.
        
        Args:
            X: Training data
            y: Training labels
            n_epochs: Number of training epochs
            learning_rate: Learning rate for optimization
        
        Returns:
            Training history
        """
        if self.params is None:
            self.initialize_params()
        
        # Create cost function
        def cost_function(params):
            total_cost = 0
            for x, target in zip(X, y):
                output = self.qnode(params, x=x)
                prediction = np.sum(output)  # Simple aggregation
                total_cost += (prediction - target)**2
            return total_cost / len(X)
        
        # Gradient descent optimization
        history = []
        
        for epoch in range(n_epochs):
            # Calculate gradients (simplified finite difference)
            gradients = np.zeros_like(self.params)
            epsilon = 1e-4
            
            for i in range(len(self.params)):
                params_plus = self.params.copy()
                params_minus = self.params.copy()
                params_plus[i] += epsilon
                params_minus[i] -= epsilon
                
                cost_plus = cost_function(params_plus)
                cost_minus = cost_function(params_minus)
                
                gradients[i] = (cost_plus - cost_minus) / (2 * epsilon)
            
            # Update parameters
            self.params -= learning_rate * gradients
            
            # Record cost
            current_cost = cost_function(self.params)
            history.append(current_cost)
            
            if epoch % 10 == 0:
                print(f"Epoch {epoch}: Cost = {current_cost:.6f}")
        
        return history


class QuantRS2VQC:
    """Variational Quantum Classifier using QuantRS2."""
    
    def __init__(self, n_features: int, n_qubits: int = None, n_layers: int = 2):
        """Initialize VQC.
        
        Args:
            n_features: Number of input features
            n_qubits: Number of qubits (defaults to n_features)
            n_layers: Number of variational layers
        """
        self.n_features = n_features
        self.n_qubits = n_qubits or n_features
        self.n_layers = n_layers
        
        self.model = QuantRS2QMLModel(self.n_qubits, n_layers)
        self.is_trained = False
    
    def fit(self, X, y, **kwargs):
        """Train the classifier.
        
        Args:
            X: Training features
            y: Training labels
            **kwargs: Additional training arguments
        """
        # Normalize features to [0, π]
        X_normalized = self._normalize_features(X)
        
        # Train the model
        history = self.model.train(X_normalized, y, **kwargs)
        self.is_trained = True
        
        return history
    
    def predict(self, X):
        """Make predictions.
        
        Args:
            X: Input features
        
        Returns:
            Predicted labels
        """
        if not self.is_trained:
            raise QuantRS2PennyLaneError("Model not trained")
        
        X_normalized = self._normalize_features(X)
        return self.model.predict(X_normalized)
    
    def score(self, X, y):
        """Calculate classification accuracy.
        
        Args:
            X: Test features
            y: True labels
        
        Returns:
            Accuracy score
        """
        predictions = self.predict(X)
        return np.mean(predictions == y)
    
    def _normalize_features(self, X):
        """Normalize features to [0, π] range."""
        X_min = np.min(X, axis=0)
        X_max = np.max(X, axis=0)
        
        # Avoid division by zero
        X_range = X_max - X_min
        X_range[X_range == 0] = 1
        
        X_normalized = (X - X_min) / X_range * np.pi
        return X_normalized


# Device registration and plugin utilities
def register_quantrs2_device():
    """Register QuantRS2 device with PennyLane."""
    if PENNYLANE_AVAILABLE:
        # Register the device
        qml.device._device_map["quantrs2.simulator"] = QuantRS2Device


def create_quantrs2_device(wires, **kwargs):
    """Create a QuantRS2 PennyLane device.
    
    Args:
        wires: Number of wires or wire labels
        **kwargs: Additional device arguments
    
    Returns:
        QuantRS2Device instance
    """
    return QuantRS2Device(wires, **kwargs)


# Convenience functions
def quantrs2_qnode(qfunc, wires, **kwargs):
    """Create a QNode using QuantRS2 device.
    
    Args:
        qfunc: Quantum function
        wires: Number of wires
        **kwargs: Additional QNode arguments
    
    Returns:
        QNode using QuantRS2 device
    """
    if not PENNYLANE_AVAILABLE:
        raise QuantRS2PennyLaneError("PennyLane not available")
    
    device = QuantRS2Device(wires, **kwargs)
    return qml.QNode(qfunc, device)


def test_quantrs2_pennylane_integration():
    """Test the QuantRS2-PennyLane integration."""
    if not (PENNYLANE_AVAILABLE and QUANTRS2_AVAILABLE):
        print("Cannot test integration: missing dependencies")
        return False
    
    try:
        # Test device creation
        device = QuantRS2Device(wires=2)
        print("✓ Device creation successful")
        
        # Test simple circuit
        @qml.qnode(device)
        def test_circuit():
            qml.Hadamard(wires=0)
            qml.CNOT(wires=[0, 1])
            return qml.expval(qml.PauliZ(0))
        
        result = test_circuit()
        print(f"✓ Circuit execution successful: {result}")
        
        # Test VQC
        vqc = QuantRS2VQC(n_features=2, n_qubits=2)
        print("✓ VQC creation successful")
        
        # Generate dummy data
        X = np.random.randn(10, 2)
        y = np.random.choice([-1, 1], 10)
        
        # Train VQC
        vqc.fit(X, y, n_epochs=5, learning_rate=0.1)
        predictions = vqc.predict(X)
        print(f"✓ VQC training and prediction successful: {len(predictions)} predictions")
        
        print("✅ All tests passed!")
        return True
        
    except Exception as e:
        print(f"❌ Test failed: {e}")
        return False


# Export main classes and functions
__all__ = [
    'QuantRS2Device',
    'QuantRS2QMLModel',
    'QuantRS2VQC',
    'QuantRS2PennyLaneError',
    'register_quantrs2_device',
    'create_quantrs2_device',
    'quantrs2_qnode',
    'test_quantrs2_pennylane_integration'
]