#!/usr/bin/env python3
"""
Test suite for Bell state functionality.
"""

import pytest
import numpy as np
from math import sqrt

try:
    from quantrs2.bell_state import (
        BellState, create_bell_state, bell_state_probabilities,
        simulate_bell_circuit
    )
    HAS_BELL_STATE = True
except ImportError:
    HAS_BELL_STATE = False


@pytest.mark.skipif(not HAS_BELL_STATE, reason="bell_state module not available")
class TestBellState:
    """Test BellState class functionality."""
    
    def test_phi_plus_creation(self):
        """Test creation of Φ⁺ Bell state."""
        result = BellState.phi_plus()
        assert result is not None
        
        # Should have methods for getting probabilities
        assert hasattr(result, 'state_probabilities')
    
    def test_phi_minus_creation(self):
        """Test creation of Φ⁻ Bell state."""
        result = BellState.phi_minus()
        assert result is not None
        assert hasattr(result, 'state_probabilities')
    
    def test_psi_plus_creation(self):
        """Test creation of Ψ⁺ Bell state."""
        result = BellState.psi_plus()
        assert result is not None
        assert hasattr(result, 'state_probabilities')
    
    def test_psi_minus_creation(self):
        """Test creation of Ψ⁻ Bell state."""
        result = BellState.psi_minus()
        assert result is not None
        assert hasattr(result, 'state_probabilities')
    
    def test_bell_state_probabilities(self):
        """Test that Bell states have correct probability structure."""
        bell_states = [
            BellState.phi_plus(),
            BellState.phi_minus(),
            BellState.psi_plus(),
            BellState.psi_minus()
        ]
        
        for state in bell_states:
            probs = state.state_probabilities()
            assert isinstance(probs, dict)
            
            # Should have probabilities for 2-qubit system
            total_prob = sum(probs.values())
            assert abs(total_prob - 1.0) < 1e-10  # Should sum to 1
            
            # All probabilities should be non-negative
            for prob in probs.values():
                assert prob >= 0
    
    def test_phi_plus_specific_probabilities(self):
        """Test that Φ⁺ has correct probabilities."""
        result = BellState.phi_plus()
        probs = result.state_probabilities()
        
        # Φ⁺ = (|00⟩ + |11⟩)/√2 should have P(00) = P(11) = 0.5, P(01) = P(10) = 0
        expected_prob = 0.5
        tolerance = 1e-10
        
        # Check that we have the right basis states
        basis_states = set(probs.keys())
        
        # The exact format depends on implementation, but should have 2-qubit states
        if '00' in probs and '11' in probs:
            assert abs(probs['00'] - expected_prob) < tolerance
            assert abs(probs['11'] - expected_prob) < tolerance
            if '01' in probs:
                assert abs(probs['01']) < tolerance
            if '10' in probs:
                assert abs(probs['10']) < tolerance
    
    def test_psi_plus_specific_probabilities(self):
        """Test that Ψ⁺ has correct probabilities."""
        result = BellState.psi_plus()
        probs = result.state_probabilities()
        
        # Ψ⁺ = (|01⟩ + |10⟩)/√2 should have P(01) = P(10) = 0.5, P(00) = P(11) = 0
        expected_prob = 0.5
        tolerance = 1e-10
        
        if '01' in probs and '10' in probs:
            assert abs(probs['01'] - expected_prob) < tolerance
            assert abs(probs['10'] - expected_prob) < tolerance
            if '00' in probs:
                assert abs(probs['00']) < tolerance
            if '11' in probs:
                assert abs(probs['11']) < tolerance
    
    def test_bell_states_are_maximally_entangled(self):
        """Test that Bell states exhibit maximal entanglement."""
        bell_states = [
            BellState.phi_plus(),
            BellState.phi_minus(),
            BellState.psi_plus(),
            BellState.psi_minus()
        ]
        
        for state in bell_states:
            probs = state.state_probabilities()
            
            # For maximally entangled states, there should be exactly 2 non-zero probabilities
            non_zero_probs = [p for p in probs.values() if p > 1e-10]
            assert len(non_zero_probs) == 2
            
            # Each non-zero probability should be 0.5
            for prob in non_zero_probs:
                assert abs(prob - 0.5) < 1e-10
    
    def test_different_bell_states_are_distinct(self):
        """Test that different Bell states produce different probability distributions."""
        phi_plus = BellState.phi_plus().state_probabilities()
        phi_minus = BellState.phi_minus().state_probabilities()
        psi_plus = BellState.psi_plus().state_probabilities()
        psi_minus = BellState.psi_minus().state_probabilities()
        
        # States should be different (allowing for implementation differences)
        # At minimum, they shouldn't all be identical
        all_states = [phi_plus, phi_minus, psi_plus, psi_minus]
        
        # Check that not all states are identical
        all_identical = True
        for i in range(1, len(all_states)):
            if all_states[i] != all_states[0]:
                all_identical = False
                break
        
        assert not all_identical, "All Bell states appear identical"


@pytest.mark.skipif(not HAS_BELL_STATE, reason="bell_state module not available")
class TestLegacyFunctions:
    """Test legacy function compatibility."""
    
    def test_create_bell_state(self):
        """Test legacy create_bell_state function."""
        result = create_bell_state()
        assert result is not None
        assert hasattr(result, 'state_probabilities')
        
        # Should be equivalent to Φ⁺
        phi_plus_result = BellState.phi_plus()
        
        probs1 = result.state_probabilities()
        probs2 = phi_plus_result.state_probabilities()
        
        # Should have the same probability distribution
        assert probs1 == probs2
    
    def test_bell_state_probabilities(self):
        """Test legacy bell_state_probabilities function."""
        probs = bell_state_probabilities()
        
        assert isinstance(probs, dict)
        
        # Should sum to 1
        total_prob = sum(probs.values())
        assert abs(total_prob - 1.0) < 1e-10
        
        # Should be the same as calling BellState.phi_plus().state_probabilities()
        direct_probs = BellState.phi_plus().state_probabilities()
        assert probs == direct_probs
    
    def test_simulate_bell_circuit(self):
        """Test legacy simulate_bell_circuit function."""
        result = simulate_bell_circuit()
        assert result is not None
        assert hasattr(result, 'state_probabilities')
        
        probs = result.state_probabilities()
        assert isinstance(probs, dict)
        
        # Should sum to 1
        total_prob = sum(probs.values())
        assert abs(total_prob - 1.0) < 1e-10
        
        # Should be equivalent to other Bell state creation methods
        equivalent_result = create_bell_state()
        equivalent_probs = equivalent_result.state_probabilities()
        assert probs == equivalent_probs


@pytest.mark.skipif(not HAS_BELL_STATE, reason="bell_state module not available")
class TestBellStateProperties:
    """Test mathematical properties of Bell states."""
    
    def test_bell_state_normalization(self):
        """Test that all Bell states are properly normalized."""
        bell_states = [
            BellState.phi_plus(),
            BellState.phi_minus(),
            BellState.psi_plus(),
            BellState.psi_minus()
        ]
        
        for state in bell_states:
            probs = state.state_probabilities()
            total_prob = sum(probs.values())
            assert abs(total_prob - 1.0) < 1e-10, f"State not normalized: {total_prob}"
    
    def test_bell_state_orthogonality(self):
        """Test that Bell states are orthogonal (through probability distributions)."""
        states = {
            'phi_plus': BellState.phi_plus().state_probabilities(),
            'phi_minus': BellState.phi_minus().state_probabilities(),
            'psi_plus': BellState.psi_plus().state_probabilities(),
            'psi_minus': BellState.psi_minus().state_probabilities()
        }
        
        # Orthogonal states should have non-overlapping support
        # (i.e., they should have high probabilities for different basis states)
        state_names = list(states.keys())
        
        for i in range(len(state_names)):
            for j in range(i + 1, len(state_names)):
                state1 = states[state_names[i]]
                state2 = states[state_names[j]]
                
                # Find the basis states with highest probability for each
                max_basis1 = max(state1.keys(), key=lambda k: state1[k])
                max_basis2 = max(state2.keys(), key=lambda k: state2[k])
                
                # For orthogonal states, the maximum probability basis states should be different
                # or the overlap should be minimal
                overlap = min(state1.get(max_basis1, 0), state2.get(max_basis1, 0))
                overlap += min(state1.get(max_basis2, 0), state2.get(max_basis2, 0))
                
                # This is a weak test - just ensuring they're not identical
                assert not (state1 == state2), f"{state_names[i]} and {state_names[j]} are identical"
    
    def test_bell_state_parity(self):
        """Test parity properties of Bell states."""
        # Phi states should have even parity, Psi states should have odd parity
        phi_plus = BellState.phi_plus().state_probabilities()
        phi_minus = BellState.phi_minus().state_probabilities()
        psi_plus = BellState.psi_plus().state_probabilities()
        psi_minus = BellState.psi_minus().state_probabilities()
        
        def get_parity_states(probs):
            """Get even and odd parity basis states."""
            even_parity = []
            odd_parity = []
            
            for basis_state, prob in probs.items():
                if prob > 1e-10:  # Non-zero probability
                    # Count number of 1s in basis state
                    if isinstance(basis_state, str):
                        ones_count = basis_state.count('1')
                    else:
                        # Handle other representations
                        ones_count = 0
                    
                    if ones_count % 2 == 0:
                        even_parity.append(basis_state)
                    else:
                        odd_parity.append(basis_state)
            
            return even_parity, odd_parity
        
        # Phi states should primarily have even parity components
        phi_plus_even, phi_plus_odd = get_parity_states(phi_plus)
        phi_minus_even, phi_minus_odd = get_parity_states(phi_minus)
        
        # Psi states should primarily have odd parity components
        psi_plus_even, psi_plus_odd = get_parity_states(psi_plus)
        psi_minus_even, psi_minus_odd = get_parity_states(psi_minus)
        
        # This test is implementation-dependent, so we just check that states differ
        all_states = [phi_plus, phi_minus, psi_plus, psi_minus]
        unique_states = []
        for state in all_states:
            if state not in unique_states:
                unique_states.append(state)
        
        assert len(unique_states) >= 2, "Bell states should be distinguishable"


@pytest.mark.skipif(not HAS_BELL_STATE, reason="bell_state module not available")
class TestBellStateConsistency:
    """Test consistency between different methods of creating Bell states."""
    
    def test_consistency_between_methods(self):
        """Test that different methods produce consistent results."""
        # Test that legacy functions are consistent with class methods
        legacy_result = create_bell_state()
        class_result = BellState.phi_plus()
        direct_result = simulate_bell_circuit()
        
        legacy_probs = legacy_result.state_probabilities()
        class_probs = class_result.state_probabilities()
        direct_probs = direct_result.state_probabilities()
        
        # All should produce the same Φ⁺ state
        assert legacy_probs == class_probs
        assert legacy_probs == direct_probs
        assert class_probs == direct_probs
    
    def test_probability_function_consistency(self):
        """Test that bell_state_probabilities is consistent with other methods."""
        probs_function = bell_state_probabilities()
        probs_create = create_bell_state().state_probabilities()
        probs_class = BellState.phi_plus().state_probabilities()
        
        assert probs_function == probs_create
        assert probs_function == probs_class
    
    def test_repeated_calls_consistency(self):
        """Test that repeated calls produce consistent results."""
        # Multiple calls should produce the same results
        results1 = [BellState.phi_plus().state_probabilities() for _ in range(5)]
        results2 = [create_bell_state().state_probabilities() for _ in range(5)]
        
        # All results should be identical
        for i in range(1, len(results1)):
            assert results1[i] == results1[0]
        
        for i in range(1, len(results2)):
            assert results2[i] == results2[0]
        
        # Both methods should produce the same result
        assert results1[0] == results2[0]


@pytest.mark.skipif(not HAS_BELL_STATE, reason="bell_state module not available")
class TestBellStateErrorHandling:
    """Test error handling in Bell state operations."""
    
    def test_robust_probability_access(self):
        """Test that probability access is robust."""
        bell_states = [
            BellState.phi_plus(),
            BellState.phi_minus(),
            BellState.psi_plus(),
            BellState.psi_minus()
        ]
        
        for state in bell_states:
            # Should not raise exceptions
            probs = state.state_probabilities()
            assert isinstance(probs, dict)
            
            # Should handle empty keys gracefully
            for key in probs.keys():
                assert isinstance(key, (str, int, tuple))
    
    def test_legacy_function_robustness(self):
        """Test that legacy functions are robust."""
        # Should not raise exceptions
        result1 = create_bell_state()
        assert result1 is not None
        
        probs1 = bell_state_probabilities()
        assert isinstance(probs1, dict)
        
        result2 = simulate_bell_circuit()
        assert result2 is not None


@pytest.mark.skipif(not HAS_BELL_STATE, reason="bell_state module not available")
class TestBellStatePerformance:
    """Test performance characteristics of Bell state operations."""
    
    def test_bell_state_creation_performance(self):
        """Test that Bell state creation is efficient."""
        import time
        
        start_time = time.time()
        
        # Create many Bell states
        for _ in range(100):
            BellState.phi_plus()
            BellState.phi_minus()
            BellState.psi_plus()
            BellState.psi_minus()
        
        end_time = time.time()
        
        # Should complete within reasonable time
        assert end_time - start_time < 5.0  # 5 seconds max for 400 states
    
    def test_legacy_function_performance(self):
        """Test that legacy functions are efficient."""
        import time
        
        start_time = time.time()
        
        # Call legacy functions many times
        for _ in range(100):
            create_bell_state()
            bell_state_probabilities()
            simulate_bell_circuit()
        
        end_time = time.time()
        
        # Should complete within reasonable time
        assert end_time - start_time < 5.0  # 5 seconds max for 300 calls
    
    def test_probability_access_performance(self):
        """Test that probability access is efficient."""
        import time
        
        # Create Bell states once
        states = [
            BellState.phi_plus(),
            BellState.phi_minus(),
            BellState.psi_plus(),
            BellState.psi_minus()
        ]
        
        start_time = time.time()
        
        # Access probabilities many times
        for _ in range(100):
            for state in states:
                probs = state.state_probabilities()
                # Do some computation to ensure it's not optimized away
                total = sum(probs.values())
        
        end_time = time.time()
        
        # Should complete within reasonable time
        assert end_time - start_time < 2.0  # 2 seconds max for 400 probability accesses


@pytest.mark.skipif(not HAS_BELL_STATE, reason="bell_state module not available")
class TestBellStateIntegration:
    """Test integration with other parts of the quantrs2 system."""
    
    def test_bell_state_result_interface(self):
        """Test that Bell state results have the expected interface."""
        states = [
            BellState.phi_plus(),
            BellState.phi_minus(),
            BellState.psi_plus(),
            BellState.psi_minus(),
            create_bell_state(),
            simulate_bell_circuit()
        ]
        
        for state in states:
            # Should have state_probabilities method
            assert hasattr(state, 'state_probabilities')
            
            # state_probabilities should return a dict
            probs = state.state_probabilities()
            assert isinstance(probs, dict)
            
            # Should be compatible with other quantum result interfaces
            # (exact interface depends on implementation)
    
    def test_bell_state_with_visualization(self):
        """Test that Bell states work with visualization components."""
        # This is a basic integration test
        result = BellState.phi_plus()
        probs = result.state_probabilities()
        
        # Should be able to use with visualization functions
        # (if visualization is available)
        try:
            from quantrs2.visualization import visualize_probabilities
            viz = visualize_probabilities(result)
            assert viz is not None
        except ImportError:
            # Visualization not available, skip this part
            pass
    
    def test_module_imports(self):
        """Test that all expected functions are importable."""
        # All functions should be importable
        from quantrs2.bell_state import BellState
        from quantrs2.bell_state import create_bell_state
        from quantrs2.bell_state import bell_state_probabilities
        from quantrs2.bell_state import simulate_bell_circuit
        
        # All should be callable
        assert callable(BellState.phi_plus)
        assert callable(BellState.phi_minus)
        assert callable(BellState.psi_plus)
        assert callable(BellState.psi_minus)
        assert callable(create_bell_state)
        assert callable(bell_state_probabilities)
        assert callable(simulate_bell_circuit)


if __name__ == "__main__":
    pytest.main([__file__])