//! GPU benchmarking framework for performance testing and analysis.
//!
//! This module provides comprehensive benchmarking tools for GPU samplers,
//! including automated testing, scaling analysis, and energy efficiency metrics.

use crate::gpu_performance::{GpuProfiler, PerformanceReport};
use crate::sampler::{SampleResult, Sampler};
use ndarray::{Array, Array2};
use rand::thread_rng;
use std::collections::HashMap;
use std::fs::File;
use std::io::Write;
use std::time::{Duration, Instant};

#[cfg(feature = "scirs")]
use crate::scirs_stub::{
    scirs2_core::gpu::{get_device_count, GpuContext},
    scirs2_plot::{Bar, Line, Plot, Scatter},
};

/// Benchmark configuration
#[derive(Clone)]
pub struct BenchmarkConfig {
    /// Problem sizes to test
    pub problem_sizes: Vec<usize>,
    /// Number of samples per problem
    pub samples_per_problem: usize,
    /// Number of repetitions for timing
    pub repetitions: usize,
    /// Test different batch sizes
    pub batch_sizes: Vec<usize>,
    /// Test different temperature schedules
    pub temperature_schedules: Vec<(f64, f64)>,
    /// Enable energy measurement
    pub measure_energy: bool,
    /// Output directory for results
    pub output_dir: String,
    /// Verbose output
    pub verbose: bool,
}

impl Default for BenchmarkConfig {
    fn default() -> Self {
        Self {
            problem_sizes: vec![10, 50, 100, 250, 500, 1000],
            samples_per_problem: 1000,
            repetitions: 5,
            batch_sizes: vec![32, 64, 128, 256, 512, 1024],
            temperature_schedules: vec![(10.0, 0.01), (5.0, 0.1), (1.0, 0.01)],
            measure_energy: false,
            output_dir: "benchmark_results".to_string(),
            verbose: true,
        }
    }
}

/// Benchmark results
#[derive(Clone)]
pub struct BenchmarkResults {
    /// Results by problem size
    pub size_results: HashMap<usize, SizeResults>,
    /// Results by batch size
    pub batch_results: HashMap<usize, BatchResults>,
    /// Temperature schedule comparison
    pub temp_results: HashMap<String, TempResults>,
    /// Energy efficiency metrics
    pub energy_metrics: Option<EnergyMetrics>,
    /// Device information
    pub device_info: String,
    /// Timestamp
    pub timestamp: chrono::DateTime<chrono::Utc>,
}

#[derive(Clone)]
pub struct SizeResults {
    /// Average execution time
    pub avg_time: Duration,
    /// Standard deviation
    pub std_dev: Duration,
    /// Throughput (samples/second)
    pub throughput: f64,
    /// Solution quality (best energy found)
    pub best_energy: f64,
    /// Memory usage (MB)
    pub memory_usage: f64,
}

#[derive(Clone)]
pub struct BatchResults {
    /// Execution time
    pub exec_time: Duration,
    /// GPU utilization
    pub gpu_utilization: f64,
    /// Memory bandwidth utilization
    pub bandwidth_util: f64,
}

#[derive(Clone)]
pub struct TempResults {
    /// Convergence time
    pub convergence_time: Duration,
    /// Final solution quality
    pub final_quality: f64,
    /// Number of iterations to convergence
    pub iterations: usize,
}

#[derive(Clone)]
pub struct EnergyMetrics {
    /// Power consumption (watts)
    pub avg_power: f64,
    /// Energy per sample (joules)
    pub energy_per_sample: f64,
    /// Performance per watt
    pub perf_per_watt: f64,
}

/// GPU benchmark runner
pub struct GpuBenchmark<S: Sampler> {
    /// Sampler to benchmark
    sampler: S,
    /// Configuration
    config: BenchmarkConfig,
    /// Performance profiler
    profiler: GpuProfiler,
}

impl<S: Sampler> GpuBenchmark<S> {
    /// Create new benchmark
    pub fn new(sampler: S, config: BenchmarkConfig) -> Self {
        Self {
            sampler,
            config,
            profiler: GpuProfiler::new(),
        }
    }

    /// Run complete benchmark suite
    pub fn run_benchmark(&mut self) -> Result<BenchmarkResults, String> {
        if self.config.verbose {
            println!("Starting GPU benchmark...");
        }

        // Create output directory
        std::fs::create_dir_all(&self.config.output_dir)
            .map_err(|e| format!("Failed to create output directory: {}", e))?;

        let mut results = BenchmarkResults {
            size_results: HashMap::new(),
            batch_results: HashMap::new(),
            temp_results: HashMap::new(),
            energy_metrics: None,
            device_info: self.get_device_info(),
            timestamp: chrono::Utc::now(),
        };

        // Run problem size scaling tests
        self.benchmark_problem_sizes(&mut results)?;

        // Run batch size optimization tests
        self.benchmark_batch_sizes(&mut results)?;

        // Run temperature schedule comparison
        self.benchmark_temperature_schedules(&mut results)?;

        // Measure energy efficiency if enabled
        if self.config.measure_energy {
            self.benchmark_energy_efficiency(&mut results)?;
        }

        // Generate report
        self.generate_report(&results)?;

        Ok(results)
    }

    /// Benchmark different problem sizes
    fn benchmark_problem_sizes(&mut self, results: &mut BenchmarkResults) -> Result<(), String> {
        if self.config.verbose {
            println!("\nBenchmarking problem size scaling...");
        }

        for &size in &self.config.problem_sizes {
            if self.config.verbose {
                println!("  Testing size {}...", size);
            }

            // Generate random QUBO problem
            let (qubo, var_map) = generate_random_qubo(size);

            let mut times = Vec::new();
            let mut best_energy = f64::INFINITY;

            for rep in 0..self.config.repetitions {
                let start = Instant::now();

                let solutions = self
                    .sampler
                    .run_qubo(
                        &(qubo.clone(), var_map.clone()),
                        self.config.samples_per_problem,
                    )
                    .map_err(|e| e.to_string())?;

                let elapsed = start.elapsed();
                times.push(elapsed);

                if let Some(best) = solutions.first() {
                    best_energy = best_energy.min(best.energy);
                }

                if self.config.verbose && rep == 0 {
                    println!("    First run: {:?}", elapsed);
                }
            }

            // Calculate statistics
            let avg_time = times.iter().sum::<Duration>() / times.len() as u32;
            let variance = times
                .iter()
                .map(|&t| {
                    let diff = if t > avg_time {
                        (t - avg_time).as_secs_f64()
                    } else {
                        (avg_time - t).as_secs_f64()
                    };
                    diff * diff
                })
                .sum::<f64>()
                / times.len() as f64;
            let std_dev = Duration::from_secs_f64(variance.sqrt());

            let throughput = self.config.samples_per_problem as f64 / avg_time.as_secs_f64();

            results.size_results.insert(
                size,
                SizeResults {
                    avg_time,
                    std_dev,
                    throughput,
                    best_energy,
                    memory_usage: estimate_memory_usage(size, self.config.samples_per_problem),
                },
            );
        }

        Ok(())
    }

    /// Benchmark different batch sizes
    fn benchmark_batch_sizes(&mut self, results: &mut BenchmarkResults) -> Result<(), String> {
        if self.config.verbose {
            println!("\nBenchmarking batch size optimization...");
        }

        // Use a fixed medium-sized problem
        let test_size = 100;
        let (qubo, var_map) = generate_random_qubo(test_size);

        for &batch_size in &self.config.batch_sizes {
            if self.config.verbose {
                println!("  Testing batch size {}...", batch_size);
            }

            // Configure sampler with batch size (if supported)
            // This would need to be implemented in the actual sampler

            let start = Instant::now();

            let _solutions = self
                .sampler
                .run_qubo(&(qubo.clone(), var_map.clone()), batch_size)
                .map_err(|e| e.to_string())?;

            let elapsed = start.elapsed();

            // Get GPU metrics from profiler
            let gpu_util = 0.75; // Placeholder - would get from profiler
            let bandwidth_util = 0.60; // Placeholder

            results.batch_results.insert(
                batch_size,
                BatchResults {
                    exec_time: elapsed,
                    gpu_utilization: gpu_util,
                    bandwidth_util,
                },
            );
        }

        Ok(())
    }

    /// Benchmark temperature schedules
    fn benchmark_temperature_schedules(
        &mut self,
        results: &mut BenchmarkResults,
    ) -> Result<(), String> {
        if self.config.verbose {
            println!("\nBenchmarking temperature schedules...");
        }

        let test_size = 50;
        let (qubo, var_map) = generate_random_qubo(test_size);

        for &(initial, final_) in &self.config.temperature_schedules {
            let schedule_name = format!("{:.1}-{:.2}", initial, final_);

            if self.config.verbose {
                println!("  Testing schedule {}...", schedule_name);
            }

            // Would need to configure sampler with temperature schedule

            let start = Instant::now();

            let solutions = self
                .sampler
                .run_qubo(
                    &(qubo.clone(), var_map.clone()),
                    self.config.samples_per_problem,
                )
                .map_err(|e| e.to_string())?;

            let elapsed = start.elapsed();

            let final_quality = solutions.first().map(|s| s.energy).unwrap_or(f64::INFINITY);

            results.temp_results.insert(
                schedule_name,
                TempResults {
                    convergence_time: elapsed,
                    final_quality,
                    iterations: 1000, // Placeholder
                },
            );
        }

        Ok(())
    }

    /// Benchmark energy efficiency
    fn benchmark_energy_efficiency(
        &mut self,
        results: &mut BenchmarkResults,
    ) -> Result<(), String> {
        if self.config.verbose {
            println!("\nMeasuring energy efficiency...");
        }

        // This would require GPU power monitoring capabilities
        // Using placeholder values for demonstration

        let avg_power = 150.0; // Watts
        let total_samples = self.config.problem_sizes.len()
            * self.config.samples_per_problem
            * self.config.repetitions;
        let total_time: Duration = results.size_results.values().map(|r| r.avg_time).sum();

        let total_energy = avg_power * total_time.as_secs_f64();
        let energy_per_sample = total_energy / total_samples as f64;
        let perf_per_watt = total_samples as f64 / total_energy;

        results.energy_metrics = Some(EnergyMetrics {
            avg_power,
            energy_per_sample,
            perf_per_watt,
        });

        Ok(())
    }

    /// Get device information
    fn get_device_info(&self) -> String {
        #[cfg(feature = "scirs")]
        {
            if let Ok(ctx) = GpuContext::new(0) {
                let info = ctx.get_device_info();
                return format!(
                    "GPU: {} MB, {} compute units @ {} MHz",
                    info.memory_mb, info.compute_units, info.clock_mhz
                );
            }
        }

        "GPU information not available".to_string()
    }

    /// Generate benchmark report
    fn generate_report(&self, results: &BenchmarkResults) -> Result<(), String> {
        // Generate plots
        self.plot_scaling_results(results)?;
        self.plot_batch_optimization(results)?;
        self.plot_temperature_comparison(results)?;

        // Generate text report
        let report_path = format!("{}/benchmark_report.txt", self.config.output_dir);
        let mut file = File::create(&report_path)
            .map_err(|e| format!("Failed to create report file: {}", e))?;

        writeln!(file, "GPU Benchmark Report").unwrap();
        writeln!(file, "====================").unwrap();
        writeln!(file, "Timestamp: {}", results.timestamp).unwrap();
        writeln!(file, "Device: {}", results.device_info).unwrap();
        writeln!(file).unwrap();

        writeln!(file, "Problem Size Scaling:").unwrap();
        for (size, res) in &results.size_results {
            writeln!(
                file,
                "  Size {}: {:.2} ms avg, {:.0} samples/sec",
                size,
                res.avg_time.as_secs_f64() * 1000.0,
                res.throughput
            )
            .unwrap();
        }

        if let Some(energy) = &results.energy_metrics {
            writeln!(file).unwrap();
            writeln!(file, "Energy Efficiency:").unwrap();
            writeln!(file, "  Average Power: {:.1} W", energy.avg_power).unwrap();
            writeln!(
                file,
                "  Energy per Sample: {:.3} mJ",
                energy.energy_per_sample * 1000.0
            )
            .unwrap();
            writeln!(
                file,
                "  Performance per Watt: {:.1} samples/J",
                energy.perf_per_watt
            )
            .unwrap();
        }

        if self.config.verbose {
            println!("\nReport saved to: {}", report_path);
        }

        Ok(())
    }

    /// Plot scaling results
    fn plot_scaling_results(&self, results: &BenchmarkResults) -> Result<(), String> {
        #[cfg(feature = "scirs")]
        {
            let mut plot = Plot::new();

            let mut sizes = Vec::new();
            let mut times = Vec::new();
            let mut throughputs = Vec::new();

            for (size, res) in &results.size_results {
                sizes.push(*size as f64);
                times.push(res.avg_time.as_secs_f64() * 1000.0);
                throughputs.push(res.throughput);
            }

            // Sort by size
            let mut indices: Vec<usize> = (0..sizes.len()).collect();
            indices.sort_by_key(|&i| sizes[i] as usize);

            let sizes: Vec<f64> = indices.iter().map(|&i| sizes[i]).collect();
            let times: Vec<f64> = indices.iter().map(|&i| times[i]).collect();
            let throughputs: Vec<f64> = indices.iter().map(|&i| throughputs[i]).collect();

            let time_line = Line::new(sizes.clone(), times).name("Execution Time (ms)");
            let throughput_line = Line::new(sizes, throughputs).name("Throughput (samples/sec)");

            plot.add_trace(time_line);
            plot.add_trace(throughput_line);
            plot.set_title("GPU Performance Scaling");
            plot.set_xlabel("Problem Size");
            plot.set_ylabel("Performance");

            let plot_path = format!("{}/scaling_plot.html", self.config.output_dir);
            plot.save(&plot_path)?;
        }

        Ok(())
    }

    /// Plot batch size optimization
    fn plot_batch_optimization(&self, results: &BenchmarkResults) -> Result<(), String> {
        #[cfg(feature = "scirs")]
        {
            let mut plot = Plot::new();

            let mut batch_sizes = Vec::new();
            let mut exec_times = Vec::new();
            let mut gpu_utils = Vec::new();

            for (batch, res) in &results.batch_results {
                batch_sizes.push(*batch as f64);
                exec_times.push(res.exec_time.as_secs_f64() * 1000.0);
                gpu_utils.push(res.gpu_utilization * 100.0);
            }

            let time_bar = Bar::new(
                batch_sizes.iter().map(|&b| b.to_string()).collect(),
                exec_times,
            )
            .name("Execution Time (ms)");

            let util_bar = Bar::new(
                batch_sizes.iter().map(|&b| b.to_string()).collect(),
                gpu_utils,
            )
            .name("GPU Utilization (%)");

            plot.add_trace(time_bar);
            plot.add_trace(util_bar);
            plot.set_title("Batch Size Optimization");
            plot.set_xlabel("Batch Size");

            let plot_path = format!("{}/batch_optimization.html", self.config.output_dir);
            plot.save(&plot_path)?;
        }

        Ok(())
    }

    /// Plot temperature schedule comparison
    fn plot_temperature_comparison(&self, results: &BenchmarkResults) -> Result<(), String> {
        #[cfg(feature = "scirs")]
        {
            let mut plot = Plot::new();

            let schedules: Vec<String> = results.temp_results.keys().cloned().collect();
            let qualities: Vec<f64> = schedules
                .iter()
                .map(|s| results.temp_results[s].final_quality)
                .collect();

            let bar = Bar::new(schedules, qualities).name("Final Solution Quality");

            plot.add_trace(bar);
            plot.set_title("Temperature Schedule Comparison");
            plot.set_xlabel("Schedule (Initial-Final)");
            plot.set_ylabel("Solution Quality");

            let plot_path = format!("{}/temperature_comparison.html", self.config.output_dir);
            plot.save(&plot_path)?;
        }

        Ok(())
    }
}

/// Generate random QUBO problem for benchmarking
fn generate_random_qubo(size: usize) -> (Array2<f64>, HashMap<String, usize>) {
    use rand::prelude::*;
    let mut rng = thread_rng();

    let mut qubo = Array2::zeros((size, size));

    // Generate random coefficients
    for i in 0..size {
        // Linear terms
        qubo[[i, i]] = rng.gen_range(-1.0..1.0);

        // Quadratic terms
        for j in i + 1..size {
            let value = rng.gen_range(-2.0..2.0);
            qubo[[i, j]] = value;
            qubo[[j, i]] = value;
        }
    }

    // Create variable map
    let mut var_map = HashMap::new();
    for i in 0..size {
        var_map.insert(format!("x{}", i), i);
    }

    (qubo, var_map)
}

/// Estimate memory usage in MB
fn estimate_memory_usage(problem_size: usize, batch_size: usize) -> f64 {
    // QUBO matrix: n^2 * 8 bytes
    let matrix_size = problem_size * problem_size * 8;

    // States: batch_size * n bytes
    let states_size = batch_size * problem_size;

    // Additional overhead
    let overhead = matrix_size / 10;

    (matrix_size + states_size + overhead) as f64 / (1024.0 * 1024.0)
}

/// Compare multiple GPU implementations
pub struct GpuComparison {
    /// Configurations to compare
    configs: Vec<ComparisonConfig>,
    /// Benchmark configuration
    benchmark_config: BenchmarkConfig,
}

struct ComparisonConfig {
    name: String,
    sampler: Box<dyn Sampler>,
}

impl GpuComparison {
    /// Create new comparison
    pub fn new(benchmark_config: BenchmarkConfig) -> Self {
        Self {
            configs: Vec::new(),
            benchmark_config,
        }
    }

    /// Add implementation to compare
    pub fn add_implementation(&mut self, name: &str, sampler: Box<dyn Sampler>) {
        self.configs.push(ComparisonConfig {
            name: name.to_string(),
            sampler,
        });
    }

    /// Run comparison
    pub fn run_comparison(&mut self) -> Result<ComparisonResults, String> {
        let mut results = ComparisonResults {
            implementations: HashMap::new(),
            best_performer: String::new(),
        };

        for config in &mut self.configs {
            println!("\nBenchmarking {}...", config.name);

            // Run benchmark for this implementation
            // Would need trait object support for benchmark

            // Placeholder results
            results.implementations.insert(
                config.name.clone(),
                ImplementationResult {
                    avg_performance: 1000.0,
                    best_quality: -100.0,
                    memory_efficiency: 0.8,
                },
            );
        }

        // Determine best performer
        results.best_performer = results
            .implementations
            .iter()
            .max_by(|a, b| {
                a.1.avg_performance
                    .partial_cmp(&b.1.avg_performance)
                    .unwrap()
            })
            .map(|(name, _)| name.clone())
            .unwrap_or_default();

        Ok(results)
    }
}

/// Comparison results
pub struct ComparisonResults {
    pub implementations: HashMap<String, ImplementationResult>,
    pub best_performer: String,
}

pub struct ImplementationResult {
    pub avg_performance: f64,
    pub best_quality: f64,
    pub memory_efficiency: f64,
}

#[cfg(test)]
mod tests {
    use super::*;
    use crate::sampler::SASampler;

    #[test]
    fn test_benchmark_config() {
        let config = BenchmarkConfig::default();
        assert!(!config.problem_sizes.is_empty());
        assert!(config.samples_per_problem > 0);
    }

    #[test]
    fn test_generate_random_qubo() {
        let (qubo, var_map) = generate_random_qubo(10);
        assert_eq!(qubo.shape(), &[10, 10]);
        assert_eq!(var_map.len(), 10);
    }

    #[test]
    fn test_memory_estimation() {
        let mem = estimate_memory_usage(100, 1000);
        assert!(mem > 0.0);
        assert!(mem < 1000.0); // Should be reasonable
    }
}
