//! GPU memory pooling for efficient allocation and reuse.
//!
//! This module provides memory pooling functionality to reduce allocation
//! overhead in GPU computations, particularly for iterative algorithms.

use std::collections::{HashMap, VecDeque};
use std::ptr::NonNull;
use std::sync::{Arc, Mutex};

#[cfg(feature = "scirs")]
use crate::scirs_stub::scirs2_core::gpu::{GpuContext, GpuMemory};

/// Memory block information
#[derive(Clone)]
struct MemoryBlock {
    /// Pointer to GPU memory
    ptr: NonNull<u8>,
    /// Size in bytes
    size: usize,
    /// Whether the block is currently in use
    in_use: bool,
    /// Last access time for LRU eviction
    last_access: std::time::Instant,
}

/// GPU memory pool for efficient allocation
pub struct GpuMemoryPool {
    /// GPU context
    #[cfg(feature = "scirs")]
    context: Arc<GpuContext>,
    /// Pool of memory blocks by size
    blocks_by_size: HashMap<usize, VecDeque<MemoryBlock>>,
    /// All allocated blocks
    all_blocks: Vec<MemoryBlock>,
    /// Maximum pool size in bytes
    max_size: usize,
    /// Current allocated size
    current_size: usize,
    /// Allocation statistics
    stats: AllocationStats,
    /// Mutex for thread safety
    mutex: Arc<Mutex<()>>,
}

/// Allocation statistics
#[derive(Default, Clone)]
pub struct AllocationStats {
    /// Total allocations
    pub total_allocations: usize,
    /// Cache hits (reused blocks)
    pub cache_hits: usize,
    /// Cache misses (new allocations)
    pub cache_misses: usize,
    /// Total bytes allocated
    pub total_bytes_allocated: usize,
    /// Peak memory usage
    pub peak_memory_usage: usize,
    /// Number of evictions
    pub evictions: usize,
}

#[cfg(feature = "scirs")]
impl GpuMemoryPool {
    /// Create a new memory pool
    pub fn new(context: Arc<GpuContext>, max_size: usize) -> Self {
        Self {
            context,
            blocks_by_size: HashMap::new(),
            all_blocks: Vec::new(),
            max_size,
            current_size: 0,
            stats: AllocationStats::default(),
            mutex: Arc::new(Mutex::new(())),
        }
    }

    /// Allocate memory from the pool
    #[cfg(feature = "scirs")]
    pub fn allocate(&mut self, size: usize) -> Result<GpuMemory, String> {
        let _lock = self.mutex.lock().unwrap();

        self.stats.total_allocations += 1;

        // Round up to nearest power of 2 for better reuse
        let aligned_size = size.next_power_of_two();

        // Check if we have a free block of the right size
        if let Some(blocks) = self.blocks_by_size.get_mut(&aligned_size) {
            if let Some(mut block) = blocks.pop_front() {
                if !block.in_use {
                    block.in_use = true;
                    block.last_access = std::time::Instant::now();
                    self.stats.cache_hits += 1;

                    return Ok(GpuMemory::from_raw(
                        self.context.clone(),
                        block.ptr,
                        block.size,
                    ));
                }
            }
        }

        // No suitable block found, allocate new
        self.stats.cache_misses += 1;

        // Check if we need to evict blocks
        if self.current_size + aligned_size > self.max_size {
            self.evict_lru_blocks(aligned_size)?;
        }

        // Allocate new block
        let gpu_mem = self
            .context
            .allocate_raw(aligned_size)
            .map_err(|e| format!("GPU allocation failed: {}", e))?;

        let block = MemoryBlock {
            ptr: gpu_mem.as_ptr(),
            size: aligned_size,
            in_use: true,
            last_access: std::time::Instant::now(),
        };

        self.all_blocks.push(block.clone());
        self.current_size += aligned_size;
        self.stats.total_bytes_allocated += aligned_size;

        if self.current_size > self.stats.peak_memory_usage {
            self.stats.peak_memory_usage = self.current_size;
        }

        Ok(gpu_mem)
    }

    /// Release memory back to the pool
    #[cfg(feature = "scirs")]
    pub fn release(&mut self, memory: GpuMemory) {
        let _lock = self.mutex.lock().unwrap();

        let ptr = memory.as_ptr();
        let size = memory.size();

        // Find the block and mark it as free
        for block in &mut self.all_blocks {
            if block.ptr == ptr && block.size == size {
                block.in_use = false;
                block.last_access = std::time::Instant::now();

                // Add to the pool for reuse
                self.blocks_by_size
                    .entry(size)
                    .or_insert_with(VecDeque::new)
                    .push_back(block.clone());

                break;
            }
        }
    }

    /// Evict least recently used blocks to make space
    #[cfg(feature = "scirs")]
    fn evict_lru_blocks(&mut self, required_size: usize) -> Result<(), String> {
        let mut freed_size = 0;
        let mut blocks_to_evict = Vec::new();

        // Sort blocks by last access time
        let mut free_blocks: Vec<_> = self.all_blocks.iter().filter(|b| !b.in_use).collect();
        free_blocks.sort_by_key(|b| b.last_access);

        // Evict blocks until we have enough space
        for block in free_blocks {
            if freed_size >= required_size {
                break;
            }

            blocks_to_evict.push(block.ptr);
            freed_size += block.size;
            self.stats.evictions += 1;
        }

        if freed_size < required_size {
            return Err("Insufficient memory in pool even after eviction".to_string());
        }

        // Actually evict the blocks
        for ptr in blocks_to_evict {
            self.all_blocks.retain(|b| b.ptr != ptr);

            // Remove from size-based pools
            for blocks in self.blocks_by_size.values_mut() {
                blocks.retain(|b| b.ptr != ptr);
            }

            // Free GPU memory
            unsafe {
                self.context
                    .free_raw(ptr)
                    .map_err(|e| format!("Failed to free GPU memory: {}", e))?;
            }
        }

        self.current_size -= freed_size;

        Ok(())
    }

    /// Get allocation statistics
    pub fn stats(&self) -> AllocationStats {
        self.stats.clone()
    }

    /// Clear the entire pool
    #[cfg(feature = "scirs")]
    pub fn clear(&mut self) -> Result<(), String> {
        let _lock = self.mutex.lock().unwrap();

        // Free all GPU memory
        for block in &self.all_blocks {
            unsafe {
                self.context
                    .free_raw(block.ptr)
                    .map_err(|e| format!("Failed to free GPU memory: {}", e))?;
            }
        }

        self.blocks_by_size.clear();
        self.all_blocks.clear();
        self.current_size = 0;

        Ok(())
    }

    /// Defragment the pool to reduce fragmentation
    #[cfg(feature = "scirs")]
    pub fn defragment(&mut self) -> Result<(), String> {
        let _lock = self.mutex.lock().unwrap();

        // This is a complex operation that would involve:
        // 1. Identifying fragmented regions
        // 2. Allocating new contiguous blocks
        // 3. Copying data
        // 4. Updating pointers
        // 5. Freeing old blocks

        // For now, we just compact the free block lists
        for blocks in self.blocks_by_size.values_mut() {
            blocks.retain(|b| !b.in_use);
        }

        Ok(())
    }
}

/// Scoped memory allocation that automatically returns to pool
pub struct ScopedGpuMemory {
    memory: Option<GpuMemory>,
    pool: Arc<Mutex<GpuMemoryPool>>,
}

impl ScopedGpuMemory {
    /// Create a new scoped allocation
    #[cfg(feature = "scirs")]
    pub fn new(pool: Arc<Mutex<GpuMemoryPool>>, size: usize) -> Result<Self, String> {
        let memory = pool.lock().unwrap().allocate(size)?;
        Ok(Self {
            memory: Some(memory),
            pool,
        })
    }

    /// Get the underlying memory
    #[cfg(feature = "scirs")]
    pub fn memory(&self) -> &GpuMemory {
        self.memory.as_ref().unwrap()
    }

    /// Get mutable access to memory
    #[cfg(feature = "scirs")]
    pub fn memory_mut(&mut self) -> &mut GpuMemory {
        self.memory.as_mut().unwrap()
    }
}

#[cfg(feature = "scirs")]
impl Drop for ScopedGpuMemory {
    fn drop(&mut self) {
        if let Some(memory) = self.memory.take() {
            self.pool.lock().unwrap().release(memory);
        }
    }
}

/// Memory pool manager for multiple devices
pub struct MultiDeviceMemoryPool {
    /// Pools for each device
    device_pools: HashMap<usize, Arc<Mutex<GpuMemoryPool>>>,
}

impl MultiDeviceMemoryPool {
    /// Create a new multi-device pool
    pub fn new() -> Self {
        Self {
            device_pools: HashMap::new(),
        }
    }

    /// Add a device pool
    #[cfg(feature = "scirs")]
    pub fn add_device(&mut self, device_id: usize, context: Arc<GpuContext>, max_size: usize) {
        let pool = Arc::new(Mutex::new(GpuMemoryPool::new(context, max_size)));
        self.device_pools.insert(device_id, pool);
    }

    /// Get pool for a device
    pub fn get_pool(&self, device_id: usize) -> Option<Arc<Mutex<GpuMemoryPool>>> {
        self.device_pools.get(&device_id).cloned()
    }

    /// Allocate from a specific device
    #[cfg(feature = "scirs")]
    pub fn allocate(&self, device_id: usize, size: usize) -> Result<ScopedGpuMemory, String> {
        let pool = self
            .get_pool(device_id)
            .ok_or_else(|| format!("No pool for device {}", device_id))?;

        ScopedGpuMemory::new(pool, size)
    }

    /// Get combined statistics
    pub fn combined_stats(&self) -> AllocationStats {
        let mut combined = AllocationStats::default();

        for pool in self.device_pools.values() {
            let stats = pool.lock().unwrap().stats();
            combined.total_allocations += stats.total_allocations;
            combined.cache_hits += stats.cache_hits;
            combined.cache_misses += stats.cache_misses;
            combined.total_bytes_allocated += stats.total_bytes_allocated;
            combined.peak_memory_usage += stats.peak_memory_usage;
            combined.evictions += stats.evictions;
        }

        combined
    }
}

// Placeholder implementations when SciRS2 is not available
#[cfg(not(feature = "scirs"))]
pub struct GpuMemory;

#[cfg(not(feature = "scirs"))]
impl GpuMemoryPool {
    pub fn new(_max_size: usize) -> Self {
        Self {
            blocks_by_size: HashMap::new(),
            all_blocks: Vec::new(),
            max_size: 0,
            current_size: 0,
            stats: AllocationStats::default(),
            mutex: Arc::new(Mutex::new(())),
        }
    }

    pub fn allocate(&mut self, _size: usize) -> Result<GpuMemory, String> {
        Err("GPU memory pooling requires SciRS2 feature".to_string())
    }

    pub fn stats(&self) -> AllocationStats {
        self.stats.clone()
    }
}

#[cfg(test)]
mod tests {
    use super::*;

    #[test]
    fn test_allocation_stats() {
        let stats = AllocationStats {
            total_allocations: 100,
            cache_hits: 80,
            cache_misses: 20,
            total_bytes_allocated: 1024 * 1024,
            peak_memory_usage: 512 * 1024,
            evictions: 5,
        };

        assert_eq!(stats.total_allocations, 100);
        assert_eq!(stats.cache_hits, 80);

        let hit_rate = stats.cache_hits as f64 / stats.total_allocations as f64;
        assert!(hit_rate > 0.79 && hit_rate < 0.81);
    }

    #[test]
    #[cfg(feature = "scirs")]
    fn test_memory_pool_basic() {
        use crate::scirs_stub::scirs2_core::gpu::GpuContext;

        let context = Arc::new(GpuContext::new(0).unwrap());
        let mut pool = GpuMemoryPool::new(context, 1024 * 1024); // 1MB pool

        // First allocation should be a cache miss
        let mem1 = pool.allocate(1024).unwrap();
        assert_eq!(pool.stats().cache_misses, 1);
        assert_eq!(pool.stats().cache_hits, 0);

        // Release and reallocate should be a cache hit
        pool.release(mem1);
        let _mem2 = pool.allocate(1024).unwrap();
        assert_eq!(pool.stats().cache_misses, 1);
        assert_eq!(pool.stats().cache_hits, 1);
    }
}
