//! Standard library for the problem DSL.

use super::ast::{Expression, Value};
use super::types::{FunctionSignature, VarType};
use std::collections::HashMap;

/// Standard library
#[derive(Debug, Clone)]
pub struct StandardLibrary {
    /// Built-in functions
    functions: HashMap<String, BuiltinFunction>,
    /// Common patterns
    patterns: HashMap<String, Pattern>,
    /// Problem templates
    templates: HashMap<String, Template>,
}

/// Built-in function
#[derive(Debug, Clone)]
pub struct BuiltinFunction {
    pub name: String,
    pub signature: FunctionSignature,
    pub description: String,
    pub implementation: FunctionImpl,
}

/// Function implementation
#[derive(Debug, Clone)]
pub enum FunctionImpl {
    /// Native Rust implementation
    Native,
    /// DSL implementation
    DSL { body: Expression },
}

/// Common pattern
#[derive(Debug, Clone)]
pub struct Pattern {
    pub name: String,
    pub description: String,
    pub parameters: Vec<String>,
    pub expansion: super::ast::AST,
}

/// Problem template
#[derive(Debug, Clone)]
pub struct Template {
    pub name: String,
    pub description: String,
    pub parameters: Vec<TemplateParam>,
    pub body: String,
}

#[derive(Debug, Clone)]
pub struct TemplateParam {
    pub name: String,
    pub param_type: String,
    pub default: Option<Value>,
}

impl StandardLibrary {
    /// Create a new standard library
    pub fn new() -> Self {
        let mut stdlib = Self {
            functions: HashMap::new(),
            patterns: HashMap::new(),
            templates: HashMap::new(),
        };

        stdlib.register_builtin_functions();
        stdlib.register_common_patterns();
        stdlib.register_templates();

        stdlib
    }

    /// Register built-in functions
    fn register_builtin_functions(&mut self) {
        // Mathematical functions
        self.functions.insert(
            "abs".to_string(),
            BuiltinFunction {
                name: "abs".to_string(),
                signature: FunctionSignature {
                    param_types: vec![VarType::Continuous],
                    return_type: VarType::Continuous,
                },
                description: "Absolute value function".to_string(),
                implementation: FunctionImpl::Native,
            },
        );

        self.functions.insert(
            "sqrt".to_string(),
            BuiltinFunction {
                name: "sqrt".to_string(),
                signature: FunctionSignature {
                    param_types: vec![VarType::Continuous],
                    return_type: VarType::Continuous,
                },
                description: "Square root function".to_string(),
                implementation: FunctionImpl::Native,
            },
        );

        // Aggregation functions
        self.functions.insert(
            "sum".to_string(),
            BuiltinFunction {
                name: "sum".to_string(),
                signature: FunctionSignature {
                    param_types: vec![VarType::Array {
                        element_type: Box::new(VarType::Continuous),
                        dimensions: vec![0],
                    }],
                    return_type: VarType::Continuous,
                },
                description: "Sum aggregation function".to_string(),
                implementation: FunctionImpl::Native,
            },
        );
    }

    /// Register common patterns
    fn register_common_patterns(&mut self) {
        // Placeholder for common optimization patterns
        // e.g., all-different, cardinality constraints, etc.
    }

    /// Register problem templates
    fn register_templates(&mut self) {
        // Placeholder for problem templates
        // e.g., TSP, graph coloring, etc.
    }

    /// Get function by name
    pub fn get_function(&self, name: &str) -> Option<&BuiltinFunction> {
        self.functions.get(name)
    }

    /// Get pattern by name
    pub fn get_pattern(&self, name: &str) -> Option<&Pattern> {
        self.patterns.get(name)
    }

    /// Get template by name
    pub fn get_template(&self, name: &str) -> Option<&Template> {
        self.templates.get(name)
    }
}

impl Default for StandardLibrary {
    fn default() -> Self {
        Self::new()
    }
}
