//! Simulated Quantum Annealing implementation with SciRS2
//!
//! This module implements quantum annealing simulation using the transverse field Ising model
//! and provides advanced features like noise modeling and diabatic transitions.

use crate::{
    sampler::{SampleResult, Sampler, SamplerError, SamplerResult},
    QuboModel,
};
use ndarray::{Array, Array1, Array2};
use rand::prelude::*;
use std::collections::HashMap;
use std::f64::consts::PI;

#[cfg(feature = "scirs")]
use scirs2::{
    linalg::{eigendecomposition, matrix_exp},
    quantum::{pauli_matrices, tensor_product},
    sparse::{CsrMatrix, SparseMatrix},
};

/// Quantum annealing schedule types
#[derive(Debug, Clone)]
pub enum AnnealingSchedule {
    /// Linear interpolation between initial and final Hamiltonians
    Linear,
    /// Quadratic schedule for slower transitions
    Quadratic,
    /// Exponential schedule for rapid quenching
    Exponential,
    /// Custom schedule with control points
    Custom { times: Vec<f64>, values: Vec<f64> },
}

impl AnnealingSchedule {
    /// Get the schedule parameter s(t) at time t ∈ [0, 1]
    pub fn s(&self, t: f64) -> f64 {
        match self {
            Self::Linear => t,
            Self::Quadratic => t * t,
            Self::Exponential => (t.exp() - 1.0) / (1_f64.exp() - 1.0),
            Self::Custom { times, values } => {
                // Linear interpolation between control points
                if t <= times[0] {
                    values[0]
                } else if t >= times[times.len() - 1] {
                    values[values.len() - 1]
                } else {
                    for i in 1..times.len() {
                        if t <= times[i] {
                            let frac = (t - times[i - 1]) / (times[i] - times[i - 1]);
                            return values[i - 1] + frac * (values[i] - values[i - 1]);
                        }
                    }
                    values[values.len() - 1]
                }
            }
        }
    }

    /// Get the transverse field strength A(t)
    pub fn transverse_field(&self, t: f64) -> f64 {
        1.0 - self.s(t)
    }

    /// Get the problem Hamiltonian strength B(t)
    pub fn problem_strength(&self, t: f64) -> f64 {
        self.s(t)
    }
}

/// Noise model for quantum annealing
#[derive(Debug, Clone)]
pub struct NoiseModel {
    /// Temperature (in units of GHz)
    pub temperature: f64,
    /// Dephasing rate
    pub dephasing_rate: f64,
    /// Energy relaxation rate
    pub relaxation_rate: f64,
    /// Control noise amplitude
    pub control_noise: f64,
}

impl Default for NoiseModel {
    fn default() -> Self {
        Self {
            temperature: 0.015, // ~15 mK typical for D-Wave
            dephasing_rate: 1e-6,
            relaxation_rate: 1e-7,
            control_noise: 0.01,
        }
    }
}

/// Configuration for simulated quantum annealing
#[derive(Debug, Clone)]
pub struct QuantumAnnealingConfig {
    /// Total annealing time (in microseconds)
    pub annealing_time: f64,
    /// Number of time steps for simulation
    pub num_steps: usize,
    /// Annealing schedule
    pub schedule: AnnealingSchedule,
    /// Noise model (optional)
    pub noise_model: Option<NoiseModel>,
    /// Whether to use sparse matrix operations
    pub use_sparse: bool,
    /// Maximum number of excited states to track
    pub max_excited_states: usize,
    /// Whether to compute diabatic transitions
    pub track_diabatic_transitions: bool,
}

impl Default for QuantumAnnealingConfig {
    fn default() -> Self {
        Self {
            annealing_time: 20.0, // 20 μs
            num_steps: 1000,
            schedule: AnnealingSchedule::Linear,
            noise_model: None,
            use_sparse: true,
            max_excited_states: 10,
            track_diabatic_transitions: false,
        }
    }
}

/// Quantum state during annealing
#[derive(Debug, Clone)]
pub struct QuantumState {
    /// State vector (amplitude for each computational basis state)
    pub amplitudes: Array1<Complex64>,
    /// Current time
    pub time: f64,
    /// Energy expectation value
    pub energy: f64,
    /// Overlap with ground state
    pub ground_state_overlap: f64,
}

/// Complex number type
#[derive(Debug, Clone, Copy)]
pub struct Complex64 {
    pub re: f64,
    pub im: f64,
}

impl Complex64 {
    pub fn new(re: f64, im: f64) -> Self {
        Self { re, im }
    }

    pub fn norm_squared(&self) -> f64 {
        self.re * self.re + self.im * self.im
    }

    pub fn conj(&self) -> Self {
        Self::new(self.re, -self.im)
    }
}

/// Simulated quantum annealing sampler
pub struct QuantumAnnealingSampler {
    config: QuantumAnnealingConfig,
    rng: StdRng,
}

impl QuantumAnnealingSampler {
    pub fn new(config: QuantumAnnealingConfig) -> Self {
        Self {
            config,
            rng: StdRng::from_seed([42; 32]),
        }
    }

    pub fn with_seed(mut self, seed: u64) -> Self {
        self.rng = StdRng::seed_from_u64(seed);
        self
    }

    /// Build the transverse field Hamiltonian
    fn build_transverse_hamiltonian(&self, n: usize) -> Array2<f64> {
        let mut h_transverse = Array2::zeros((1 << n, 1 << n));

        // Apply Pauli-X to each qubit
        for i in 0..n {
            for state in 0..(1 << n) {
                let flipped = state ^ (1 << i);
                h_transverse[[state, flipped]] = -1.0;
            }
        }

        h_transverse
    }

    /// Build the problem Hamiltonian from QUBO (legacy method - unused)
    #[allow(dead_code)]
    fn build_problem_hamiltonian(&self, _qubo: &QuboModel) -> Array2<f64> {
        // This method is no longer used as we switched to matrix-based interface
        // Keeping for backward compatibility but not implementing
        unimplemented!("Use build_problem_hamiltonian_from_matrix instead")
    }

    /// Build the problem Hamiltonian from matrix
    fn build_problem_hamiltonian_from_matrix(
        &self,
        matrix: &Array<f64, ndarray::Ix2>,
    ) -> Array2<f64> {
        let n = matrix.nrows();
        let mut h_problem = Array2::zeros((1 << n, 1 << n));

        // Diagonal elements (classical energies)
        for state in 0..(1 << n) {
            let mut energy = 0.0;

            // Calculate energy for this binary state
            for i in 0..n {
                for j in 0..n {
                    let bit_i = (state >> i) & 1;
                    let bit_j = (state >> j) & 1;
                    energy += matrix[[i, j]] * bit_i as f64 * bit_j as f64;
                }
            }

            h_problem[[state, state]] = energy;
        }

        h_problem
    }

    /// Evolve the quantum state
    fn evolve_state(
        &self,
        state: &mut QuantumState,
        h_transverse: &Array2<f64>,
        h_problem: &Array2<f64>,
        dt: f64,
        t: f64,
    ) {
        let s = self.config.schedule.s(t);
        let a = self.config.schedule.transverse_field(t);
        let b = self.config.schedule.problem_strength(t);

        // Total Hamiltonian H(t) = A(t) * H_transverse + B(t) * H_problem
        let h_total = a * h_transverse + b * h_problem;

        // Time evolution: |ψ(t+dt)⟩ = exp(-i H dt) |ψ(t)⟩
        // For small dt, use first-order approximation
        let n = state.amplitudes.len();
        let mut new_amplitudes = Array1::from_elem(n, Complex64::new(0.0, 0.0));

        for i in 0..n {
            let mut amp = state.amplitudes[i];

            // Diagonal term
            let energy = h_total[[i, i]];
            let phase = Complex64::new((energy * dt).cos(), -(energy * dt).sin());
            amp = Complex64::new(
                amp.re * phase.re - amp.im * phase.im,
                amp.re * phase.im + amp.im * phase.re,
            );

            // Off-diagonal terms (simplified)
            for j in 0..n {
                if i != j && h_total[[i, j]].abs() > 1e-10 {
                    let coupling = h_total[[i, j]] * dt;
                    let other_amp = state.amplitudes[j];
                    amp.re += -coupling * other_amp.im;
                    amp.im += coupling * other_amp.re;
                }
            }

            new_amplitudes[i] = amp;
        }

        // Normalize
        let norm: f64 = new_amplitudes
            .iter()
            .map(|a| a.norm_squared())
            .sum::<f64>()
            .sqrt();

        for amp in new_amplitudes.iter_mut() {
            amp.re /= norm;
            amp.im /= norm;
        }

        state.amplitudes = new_amplitudes;
        state.time = t + dt;

        // Update energy expectation
        state.energy = self.compute_energy_expectation(&state.amplitudes, h_problem);
    }

    /// Add noise to the quantum state
    fn apply_noise(&self, state: &mut QuantumState, dt: f64) {
        if let Some(noise) = &self.config.noise_model {
            let n = state.amplitudes.len();
            let mut rng = StdRng::from_seed([42; 32]); // Create local RNG

            // Dephasing noise
            if noise.dephasing_rate > 0.0 {
                for amp in state.amplitudes.iter_mut() {
                    let phase_noise = rng.gen_range(-1.0..1.0) * (noise.dephasing_rate * dt).sqrt();
                    let phase = Complex64::new(phase_noise.cos(), phase_noise.sin());
                    let new_amp = Complex64::new(
                        amp.re * phase.re - amp.im * phase.im,
                        amp.re * phase.im + amp.im * phase.re,
                    );
                    *amp = new_amp;
                }
            }

            // Thermal excitations
            if noise.temperature > 0.0 {
                // Simplified thermal noise model
                let thermal_prob = (noise.temperature * dt).min(0.1);
                if rng.gen::<f64>() < thermal_prob {
                    let i = rng.gen_range(0..n);
                    let j = rng.gen_range(0..n);
                    if i != j {
                        // Mix states i and j
                        let mix_angle: f64 = rng.gen_range(0.0..0.1);
                        let cos_a = mix_angle.cos();
                        let sin_a = mix_angle.sin();

                        let amp_i = state.amplitudes[i];
                        let amp_j = state.amplitudes[j];

                        state.amplitudes[i] = Complex64::new(
                            cos_a * amp_i.re + sin_a * amp_j.re,
                            cos_a * amp_i.im + sin_a * amp_j.im,
                        );
                        state.amplitudes[j] = Complex64::new(
                            -sin_a * amp_i.re + cos_a * amp_j.re,
                            -sin_a * amp_i.im + cos_a * amp_j.im,
                        );
                    }
                }
            }
        }
    }

    /// Compute energy expectation value
    fn compute_energy_expectation(
        &self,
        amplitudes: &Array1<Complex64>,
        h_problem: &Array2<f64>,
    ) -> f64 {
        let n = amplitudes.len();
        let mut energy = 0.0;

        for i in 0..n {
            for j in 0..n {
                let amp_i = amplitudes[i];
                let amp_j = amplitudes[j];
                let h_ij = h_problem[[i, j]];

                // ⟨ψ|H|ψ⟩ = Σ_ij ψ*_i H_ij ψ_j
                energy += (amp_i.conj().re * amp_j.re + amp_i.conj().im * amp_j.im) * h_ij;
            }
        }

        energy
    }

    /// Perform measurement on quantum state
    fn measure_state(&self, state: &QuantumState) -> Vec<bool> {
        let n = (state.amplitudes.len() as f64).log2() as usize;
        let mut probabilities = Vec::new();
        let mut cumulative = 0.0;

        // Compute probabilities
        for amp in state.amplitudes.iter() {
            cumulative += amp.norm_squared();
            probabilities.push(cumulative);
        }

        // Sample according to probability distribution
        let mut rng = StdRng::from_seed([42; 32]); // Create local RNG
        let r = rng.gen::<f64>();
        let mut measured_state = 0;

        for (i, &prob) in probabilities.iter().enumerate() {
            if r <= prob {
                measured_state = i;
                break;
            }
        }

        // Convert to binary representation
        (0..n).map(|i| (measured_state >> i) & 1 == 1).collect()
    }
}

impl Sampler for QuantumAnnealingSampler {
    fn run_qubo(
        &self,
        qubo: &(Array<f64, ndarray::Ix2>, HashMap<String, usize>),
        num_reads: usize,
    ) -> SamplerResult<Vec<SampleResult>> {
        let (matrix, var_map) = qubo;
        let n = matrix.nrows();
        if n > 20 {
            return Err(SamplerError::InvalidParameter(
                "Quantum simulation limited to 20 qubits".into(),
            ));
        }

        // Build Hamiltonians
        let h_transverse = self.build_transverse_hamiltonian(n);
        let h_problem = self.build_problem_hamiltonian_from_matrix(matrix);

        let mut results = Vec::new();

        for read in 0..num_reads {
            // Initialize in ground state of transverse field (uniform superposition)
            let mut state = QuantumState {
                amplitudes: Array1::from_elem(1 << n, Complex64::new(1.0 / (1 << n) as f64, 0.0)),
                time: 0.0,
                energy: 0.0,
                ground_state_overlap: 1.0,
            };

            // Time evolution
            let dt = self.config.annealing_time / self.config.num_steps as f64;

            for step in 0..self.config.num_steps {
                let t = step as f64 / self.config.num_steps as f64;

                // Evolve under Hamiltonian
                self.evolve_state(&mut state, &h_transverse, &h_problem, dt, t);

                // Apply noise if configured
                self.apply_noise(&mut state, dt);
            }

            // Measure final state
            let measured = self.measure_state(&state);

            // Convert to assignments using variable map
            let mut assignments = HashMap::new();
            for (var_name, &idx) in var_map {
                assignments.insert(var_name.clone(), measured[idx]);
            }

            // Calculate classical energy from matrix
            let mut energy = 0.0;
            for i in 0..n {
                for j in 0..n {
                    if measured[i] && measured[j] {
                        energy += matrix[[i, j]];
                    }
                }
            }

            results.push(SampleResult {
                assignments,
                energy,
                occurrences: 1,
            });
        }

        Ok(results)
    }

    fn run_hobo(
        &self,
        hobo: &(Array<f64, ndarray::IxDyn>, HashMap<String, usize>),
        shots: usize,
    ) -> SamplerResult<Vec<SampleResult>> {
        let (matrix_dyn, var_map) = hobo;

        if matrix_dyn.ndim() != 2 {
            return Err(SamplerError::InvalidParameter(
                "HOBO matrix must be 2D for quantum annealing".into(),
            ));
        }

        let matrix_2d = matrix_dyn
            .clone()
            .into_dimensionality::<ndarray::Ix2>()
            .map_err(|_| SamplerError::InvalidParameter("Failed to convert matrix to 2D".into()))?;

        self.run_qubo(&(matrix_2d, var_map.clone()), shots)
    }
}

/// Advanced quantum annealing features
pub mod advanced {
    use super::*;

    /// Reverse annealing configuration
    #[derive(Debug, Clone)]
    pub struct ReverseAnnealingConfig {
        /// Initial classical state
        pub initial_state: Vec<bool>,
        /// Reverse annealing fraction (how far to go back)
        pub reverse_fraction: f64,
        /// Hold time at reversal point
        pub hold_time: f64,
    }

    /// Quantum annealing with pause
    #[derive(Debug, Clone)]
    pub struct PauseConfig {
        /// Pause points (s values)
        pub pause_points: Vec<f64>,
        /// Pause durations
        pub pause_durations: Vec<f64>,
    }

    /// Diabatic transition analyzer
    pub struct DiabaticAnalyzer {
        /// Minimum gap encountered
        pub min_gap: f64,
        /// Gap history
        pub gap_history: Vec<(f64, f64)>, // (time, gap)
        /// Diabatic transition probability
        pub transition_probability: f64,
    }

    impl DiabaticAnalyzer {
        pub fn new() -> Self {
            Self {
                min_gap: f64::INFINITY,
                gap_history: Vec::new(),
                transition_probability: 0.0,
            }
        }

        /// Update with current gap
        pub fn update(&mut self, time: f64, gap: f64, velocity: f64) {
            self.min_gap = self.min_gap.min(gap);
            self.gap_history.push((time, gap));

            // Landau-Zener formula for diabatic transitions
            if gap > 0.0 && velocity > 0.0 {
                let lz_prob = (-2.0 * PI * gap * gap / velocity).exp();
                self.transition_probability = self.transition_probability.max(lz_prob);
            }
        }

        /// Get adiabatic condition recommendation
        pub fn recommend_annealing_time(&self) -> f64 {
            // Based on minimum gap and desired success probability
            let target_success = 0.99;
            let required_time =
                2.0 * PI / (self.min_gap * self.min_gap * (1.0_f64 - target_success).ln().abs());
            required_time.max(1.0) // At least 1 microsecond
        }
    }
}

#[cfg(test)]
mod tests {
    use super::*;
    use ndarray::Array;
    use std::collections::HashMap;

    #[test]
    fn test_annealing_schedule() {
        let schedule = AnnealingSchedule::Linear;
        assert_eq!(schedule.s(0.0), 0.0);
        assert_eq!(schedule.s(0.5), 0.5);
        assert_eq!(schedule.s(1.0), 1.0);

        let schedule = AnnealingSchedule::Quadratic;
        assert_eq!(schedule.s(0.5), 0.25);
    }

    #[test]
    fn test_small_quantum_annealing() {
        // Create small QUBO problem as matrix
        let mut matrix = Array::zeros((2, 2));
        matrix[[0, 0]] = -1.0; // Linear term for x0
        matrix[[1, 1]] = -1.0; // Linear term for x1
        matrix[[0, 1]] = 2.0; // Quadratic term for x0*x1
        matrix[[1, 0]] = 2.0; // Symmetric

        let mut var_map = HashMap::new();
        var_map.insert("x0".to_string(), 0);
        var_map.insert("x1".to_string(), 1);

        let config = QuantumAnnealingConfig {
            annealing_time: 1.0,
            num_steps: 100,
            ..Default::default()
        };

        let sampler = QuantumAnnealingSampler::new(config);
        let results = sampler.run_qubo(&(matrix, var_map), 10).unwrap();

        assert_eq!(results.len(), 10);

        // Check that we get valid solutions
        for result in results {
            assert!(result.energy.is_finite());
        }
    }
}
